/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
    \file        

    \brief       Implementation of operating system error handling utilities 
    
    \date        07-09-27 1815:00

    
*/
/*----------------------------------------------------------------------------*/

#include <scxcorelib/scxcmn.h>
#include <scxcorelib/scxoserror.h>
#include <scxcorelib/stringaid.h>
#include <sstream>
#if defined(WIN32)
#include <windows.h>
#endif

using namespace std;

namespace SCXCoreLib {
    /*----------------------------------------------------------------------------*/
    //! Retrieves the text corresponding to "last error" in windows.
    //! \param[in]     lastError     as returned by GetLastError()
    //! \returns     The corresponding text, if available.
    //!\note If the text is not available due to platform or some other reason,
    //!      the empty string will be returned.
    wstring TextOfWindowsLastError(unsigned int lastError) {
#if defined(WIN32)
        LPVOID  lpMsgBuf;
        wstring message(L"");
        unsigned int nrOfCharsInMessage = FormatMessage(
            FORMAT_MESSAGE_ALLOCATE_BUFFER | 
            FORMAT_MESSAGE_FROM_SYSTEM |
            FORMAT_MESSAGE_IGNORE_INSERTS,
            NULL,
            lastError,
            MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
            reinterpret_cast<LPTSTR> (&lpMsgBuf),
            0, NULL );
        if (0 != nrOfCharsInMessage)
        {
            message = static_cast<LPTSTR>(lpMsgBuf);
            LocalFree(lpMsgBuf);
        }
        return message;
#else
        // Return an empty string in a way that avoids compiler warnings about unused parameter
        wstring text(StrFrom(lastError));
        text = L"";
        return text;
#endif
    }

    /*----------------------------------------------------------------------------*/
    //! Constructs a text describing a problem caused by the occurance of 
    //! an unexpected windows error.
    //! \param[in]     problem     Description of a problem
    //! \param[in]     lastError     as returned by GetLastError()
    //! \returns     A text describing the problem and its cause    
    wstring UnexpectedWindowsError(const std::wstring &problem, 
            unsigned int lastError) {
        wostringstream buf;
        buf << problem << L" due to unexpected windows error " << lastError;
        buf << L" (" << TextOfWindowsLastError(lastError) << L")";
        return buf.str();
    }

    /*----------------------------------------------------------------------------*/
    //! Constructs a text describing a problem caused by the occurance of 
    //! an unexpected errno when making a call to an operating system function.
    //! \param[in]     problem     Description of a problem
    //! \param[in]     errnoValue  The value of global variable erro right after the system call
    //! \returns     A text describing the problem and its cause    
    std::wstring UnexpectedErrno(const std::wstring &problem, int errnoValue) {
        wostringstream buf;
        buf << problem << L" due to unexpected errno " << errnoValue;
        return buf.str();
    }
    
    
}

/*----------------------------E-N-D---O-F---F-I-L-E---------------------------*/
