/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
    \file

    \brief     Regex implementation

    \date      2008-08-20 14:19:15

 */
/*----------------------------------------------------------------------------*/


#include <scxcorelib/scxregex.h>
#include <scxcorelib/stringaid.h>

namespace SCXCoreLib {
    /*--------------------------------------------------------------*/
    /**
        Initializes and compiles a new instance of the Regex class
        for the specified regular expression.

        \param[in] expression Regular expression to compile.
        \throws SCXInvalidRegexException if compilation of regex fails.
    */
    SCXRegex::SCXRegex(const std::wstring& expression)
    {
        int errcode = regcomp(&m_Preq, StrToMultibyte(expression).c_str(), REG_EXTENDED|REG_NOSUB);
        if (errcode != 0)
        {
            throw SCXInvalidRegexException(expression, errcode, &m_Preq, SCXSRCLOCATION);
        }
    }

    /*--------------------------------------------------------------*/
    /**
        Indicates whether the regular expression specified in the
        SCXRegex constructor finds a match in the input string.
        
        \param[in] text Input string to match.
        \returns true if a match is found.
     */
    bool SCXRegex::IsMatch(const std::wstring& text) const
    {
        return (0 == regexec(&m_Preq, StrToMultibyte(text).c_str(), 0, 0, 0));
    }

    /*--------------------------------------------------------------*/
    /**
        Destructor
     */
    SCXRegex::~SCXRegex()
    {
        regfree(&m_Preq);
    }

    /*--------------------------------------------------------------*/
    /**
        Creates a new regex exception

        \param[in] expression Regular expression that failed to compile
        \param[in] errcode    System error code
        \param[in] preq       Pattern buffer storage area
        \param[in] l          Source code location object

    */
    SCXInvalidRegexException::SCXInvalidRegexException(const std::wstring& expression,
                                                       int errcode, const regex_t* preq,
                                                       const SCXCodeLocation& l) :
        SCXException(l),
        m_Expression(expression),
        m_Errcode(errcode)
    {
        char buf[80];
        regerror(m_Errcode, preq, buf, sizeof(buf));
        m_Errtext = buf;
    }

    std::wstring SCXInvalidRegexException::What() const {
        std::wostringstream txt;
        txt << L"Compiling " << m_Expression << L" returned an error code = " << m_Errcode << L" (" << m_Errtext.c_str() << L")";
        return txt.str();
    }
}

/*----------------------------E-N-D---O-F---F-I-L-E---------------------------*/
