/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
    \file        

    \brief       Implements the thread lock PAL.

    \date        2007-05-28 10:46:00

*/
/*----------------------------------------------------------------------*/
#include <scxcorelib/scxcmn.h>
#include <scxcorelib/scxthreadlock.h>

namespace SCXCoreLib
{
/*----------------------------------------------------------------------------*/
/**
    Constructor to create lock using given lock handle.
    
    Parameters:  handle - SCXThreadLockHandle reference to use as lock.
                 aquire - If true (default) the lock will be aquired in constructor.
    Retval:      N/A
        
    Default behaviour is to aquire the lock, but it may be controled using parameter.
    
*/
    SCXThreadLock::SCXThreadLock(const SCXThreadLockHandle& handle, bool aquire /*= true*/)
        : m_lock(handle)
    {
        if (aquire)
            Lock();
    }

/*----------------------------------------------------------------------------*/
/**
    Create a lock using name.
    
    Parameters:  nameOfLock - Name of lock to use.
                 aquire - If true (default) the lock will be aquired in constructor.
    Retval:      N/A
        
    Default behaviour is to aquire the lock, but it may be controled using parameter.
    Will use name to retrieve lock handle in thread lock factory. An empty name is not
    a named lock but rather an anonymous lock.
    
*/
    SCXThreadLock::SCXThreadLock(const std::wstring& nameOfLock, bool aquire /*= true*/)
    {
        m_lock = SCXThreadLockFactory::GetInstance().GetLock(nameOfLock);
        if (aquire)
            Lock();
    }
    
/*----------------------------------------------------------------------------*/
/**
    Virtual destructor.
    
    Parameters:  None
    Retval:      N/A
        
    If the lock is held it will be unlocked. If the lock handle is invalid,
    this method ignores it.
    
*/
    SCXThreadLock::~SCXThreadLock(void)
    {
        try
        {
            if (HaveLock())
            {
                Unlock();
            }
        }
        catch (SCXThreadLockInvalidException&)
        {
            /* HaveLock may throw exception when deleting invalid locks. */
        }
    }

/*----------------------------------------------------------------------------*/
/**
    Dump object as string (for logging).
    
    Parameters:  None
    Retval:      N/A
        
*/
    const std::wstring SCXThreadLock::DumpString() const
    {
        return L"SCXThreadLock=" + m_lock.DumpString();
    }

/*----------------------------------------------------------------------------*/
/**
    Explicitly aquire lock.
    
    Parameters:  None
    Retval:      None
        
    Will block until lock can be aquired. Should not be called if lock is 
    already aquired.
    
*/
    void SCXThreadLock::Lock(void)
    {
        m_lock.Lock();
    }

/*----------------------------------------------------------------------------*/
/**
    Explicitly release lock.
    
    Parameters:  None
    Retval:      None
        
    Will release the lock immediatly. Should not be called if lock is not aquired.
    
*/
    void SCXThreadLock::Unlock(void)
    {
        m_lock.Unlock();
    }

/*----------------------------------------------------------------------------*/
/**
    Try to aquire lock (using a timeout).
    
    Parameters:  timeout - Number of miliseconds (default zero) the method may attempt
                           to aquire the lock before it must return.
    Retval:      true if the lock could be aquired within the given timeout,
                 otherwise false.
        
    This method can be used to try to aquire a lock without blocking.
    
*/
    bool SCXThreadLock::TryLock(const unsigned int timeout /*= 0*/)
    {
        return m_lock.TryLock(timeout);
    }

/*----------------------------------------------------------------------------*/
/**
    Used to check if a lock is aquired already by the calling thread.
    
    Parameters:  None
    Retval:      true if the calling thread is the owner of the lock, otherwise false.
        
    May be used to check if lock is held or not without aquiring it or risking
    a "LockHeld" exception.
    
*/
    bool SCXThreadLock::HaveLock(void) const   
    {
        return m_lock.HaveLock();
    }

/*----------------------------------------------------------------------------*/
/**
    Used to check if any thread has the lock aquired.
    
    Parameters:  None
    Retval:      true if any thread is the owner of the lock, otherwise false.
        
    Created:     07-07-02 14:30:00
    
    May be used to check if a lock is held at all without aquiring it or risking
    a "LockHeld excpetion.
    
*/
    bool SCXThreadLock::IsLocked(void) const
    {
        return m_lock.IsLocked();
    }

} /* namespace SCXCoreLib */
