/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
    \file        

    \brief       Implemet the user PAL.
    
    \date        2008-03-25 11:22:33
    
*/
/*----------------------------------------------------------------------------*/

#include <scxcorelib/scxcmn.h>
#include <scxcorelib/scxuser.h>
#include <scxcorelib/scxdumpstring.h>
#include <scxcorelib/stringaid.h>

#if defined(SCX_UNIX)
#include <unistd.h>
#include <sys/types.h>
#include <pwd.h>
#else
#error "Platform not supported"
#endif


namespace SCXCoreLib
{
/*----------------------------------------------------------------------------*/
/**
    Default constructor, represents current user.
*/
    SCXUser::SCXUser() : m_uid(geteuid())
    {
        SetName();
    }

/*----------------------------------------------------------------------------*/
/**
    Constructor for specific user ID.

    /param uid User ID of user to represent.
*/
    SCXUser::SCXUser(SCXUserID uid) : m_uid(uid)
    {
        SetName();
    }
    
/*----------------------------------------------------------------------------*/
/**
    Virtual destructor.
*/
    SCXUser::~SCXUser()
    {

    }

/*----------------------------------------------------------------------------*/
/**
   Printable representation of object.
   \returns a string suitable for printing in logs describing the object.
*/
    const std::wstring SCXUser::DumpString() const {
        return SCXDumpStringBuilder("SCXUser")
            .Scalar("uid", m_uid);
    }

/*----------------------------------------------------------------------------*/
/**
    Get user ID.

    \returns user ID of the represented user.
*/
    SCXUserID SCXUser::GetUID()
    {
        return m_uid;
    }

/*----------------------------------------------------------------------------*/
/**
    Get user name.

    \returns user name of the represented user.
*/
    std::wstring SCXUser::GetName()
    {
        return m_name;
    }

/*----------------------------------------------------------------------------*/
/**
    Set user name
*/
    void SCXUser::SetName()
    {
        struct passwd pwd;
        struct passwd *ppwd = NULL;
        long bufSize = sysconf(_SC_GETPW_R_SIZE_MAX);

        // Sanity check - all platforms have this, but never hurts to be certain

        if (bufSize < 1024)
        {
            bufSize = 1024;
        }

        std::vector<char> buf(bufSize);

        // Use reentrant form of getpwuid (it's reentrant, and it pacifies purify)
#if !defined(sun)
#ifndef NDEBUG /* used in SCXASSERT only */
        int rc = 
#endif
	getpwuid_r(m_uid, &pwd, &buf[0], buf.size(), &ppwd);
        SCXASSERT( (0 == rc && NULL != ppwd) || (0 != rc && NULL == ppwd) );
#else
        ppwd = getpwuid_r(m_uid, &pwd, &buf[0], buf.size());
#endif

        if (ppwd)
        {
            m_name = StrFromMultibyte(ppwd->pw_name);
        }
        else
        {
            m_name = StrFrom(m_uid);
        }
    }
    
/*----------------------------------------------------------------------------*/
/**
    Check if user is the root user.

    \returns True if the user is the root user.
*/
    bool SCXUser::IsRoot()
    {
        return m_uid == 0;
    }
} /* namespace SCXCoreLib */
/*----------------------------E-N-D---O-F---F-I-L-E---------------------------*/
