/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
    \file        

    \brief       Representation of an instance

    \date        07-05-21 12:00:00

*/
/*----------------------------------------------------------------------------*/

#include <scxcorelib/scxcmn.h>

#include <string>
#include <vector>

#include <math.h>

#include <scxcorelib/scxmath.h>
#include <scxsystemlib/entityinstance.h>

using namespace std;

namespace SCXSystemLib
{

    /*----------------------------------------------------------------------------*/
    /**
        Constructor
    */
    EntityInstance::EntityInstance(const EntityInstanceId& id, bool isTotal) : 
        m_Id(id), 
        m_total(isTotal),
        m_exceptionCaught( false )
    {
    }


    /*----------------------------------------------------------------------------*/
    /**
        Constructor
    */
    EntityInstance::EntityInstance(bool isTotal) : 
        m_total(isTotal),
        m_exceptionCaught( false )
    {
    }

    /*----------------------------------------------------------------------------*/
    /**
        Is this a composite total instance
        
        Retval:      name of processor instance
    */
    bool EntityInstance::IsTotal() const
    {
        return m_total;
    }

    /*----------------------------------------------------------------------------*/
    /**
        Destructor
    */
    EntityInstance::~EntityInstance()
    {
    }

    /*----------------------------------------------------------------------------*/
    /**
        Get instance id
        
        Retval:      Instance id
    */
    const EntityInstanceId& EntityInstance::GetId() const
    {
        return m_Id;
    }

    /*----------------------------------------------------------------------------*/
    /**
        Get instance id
        
        Parameters:  id - Identity to use
    */
    void EntityInstance::SetId(const EntityInstanceId& id)
    {
        m_Id = id;
    }

    /*----------------------------------------------------------------------------*/
    /**
        Update the instance
    */
    void EntityInstance::Update()
    {
        // Empty default implementation
    }

    /*----------------------------------------------------------------------------*/
    /**
        Clean up the instance
    */
    void EntityInstance::CleanUp()
    {
        // Empty default implementation
    }

    /*----------------------------------------------------------------------------*/
    /**
        Fault-tolerance feature:
        should "UpdateInstances" catch any exception, it marks
        'bad' instance and continue updating the rest instead of interrupting the loop
        
        Parameters:  e - excpetion
    */
    void EntityInstance::SetUnexpectedException( const SCXCoreLib::SCXException& e )
    {
        m_exceptionCaught = true;
        m_exceptionText = e.What() + L"; " + e.Where();
    }
    
    /*----------------------------------------------------------------------------*/
    /**
        clears previously set exception (if any)
    */
    void EntityInstance::ResetUnexpectedException()
    {
        m_exceptionCaught = false;
        m_exceptionText.clear();
    }
    
    /*----------------------------------------------------------------------------*/
    /**
        Returns true if last update on this instance had thrown exception
    */
    bool EntityInstance::IsUnexpectedExceptionSet() const
    {
        return m_exceptionCaught;
    }

}

/*----------------------------E-N-D---O-F---F-I-L-E---------------------------*/
