/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
    \file        

    \brief       Implements the logical disk instance pal for statistical information.
    
    \date        2008-04-28 15:20:00

*/
/*----------------------------------------------------------------------------*/

#include <scxcorelib/scxcmn.h>
#include <scxsystemlib/statisticallogicaldiskinstance.h>
#include <scxcorelib/stringaid.h>
#include <scxcorelib/scxmath.h>

namespace SCXSystemLib
{
/*----------------------------------------------------------------------------*/
/**
   \copydoc SCXSystemLib::StatisticalDiskInstance::StatisticalDiskInstance
*/
    StatisticalLogicalDiskInstance::StatisticalLogicalDiskInstance(SCXCoreLib::SCXHandle<DiskDepend> deps, bool isTotal /* = false*/)
        : StatisticalDiskInstance(deps, isTotal),
          m_NrOfFailedFinds(0)
    {
        m_log = SCXCoreLib::SCXLogHandleFactory::GetLogHandle(L"scx.core.common.pal.system.disk.statisticallogicaldiskinstance");
    }

/*----------------------------------------------------------------------------*/
/**
   \copydoc SCXSystemLib::StatisticalDiskInstance::GetReadsPerSecond
*/
    bool StatisticalLogicalDiskInstance::GetReadsPerSecond(scxulong& value) const
    {
#if defined(aix)
        value = 0;
        return false;
#else
        return StatisticalDiskInstance::GetReadsPerSecond(value);
#endif
    }

/*----------------------------------------------------------------------------*/
/**
   \copydoc SCXSystemLib::StatisticalDiskInstance::GetWritesPerSecond
*/
    bool StatisticalLogicalDiskInstance::GetWritesPerSecond(scxulong& value) const
    {
#if defined(aix)
        value = 0;
        return false;
#else
        return StatisticalDiskInstance::GetWritesPerSecond(value);
#endif
    }

/*----------------------------------------------------------------------------*/
/**
   \copydoc SCXSystemLib::StatisticalDiskInstance::GetTransfersPerSecond
*/
    bool StatisticalLogicalDiskInstance::GetTransfersPerSecond(scxulong& value) const
    {
#if defined(aix)
        value = 0;
        return false;
#else
        return StatisticalDiskInstance::GetTransfersPerSecond(value);
#endif
    }

/*----------------------------------------------------------------------------*/
/**
   \copydoc SCXSystemLib::StatisticalDiskInstance::GetBytesPerSecond
*/
    bool StatisticalLogicalDiskInstance::GetBytesPerSecond(scxulong& read, scxulong& write) const
    {
#if defined(aix)
        read = write = 0;
        return false;
#else
        return StatisticalDiskInstance::GetBytesPerSecond(read, write);
#endif
    }

/*----------------------------------------------------------------------------*/
/**
   \copydoc SCXSystemLib::StatisticalDiskInstance::GetBytesPerSecondTotal
*/
    bool StatisticalLogicalDiskInstance::GetBytesPerSecondTotal(scxulong& total) const
    {
#if defined(aix)
        total = 0;
        return false;
#else
        return StatisticalDiskInstance::GetBytesPerSecondTotal(total);
#endif
    }

/*----------------------------------------------------------------------------*/
/**
   \copydoc SCXSystemLib::StatisticalDiskInstance::GetIOTimes
*/
    bool StatisticalLogicalDiskInstance::GetIOTimes(double& read, double& write) const
    {
#if defined(aix) || defined(linux)
        read = write = 0;
        return false;
#else
        return StatisticalDiskInstance::GetIOTimes(read, write);
#endif
    }

/*----------------------------------------------------------------------------*/
/**
   \copydoc SCXSystemLib::StatisticalDiskInstance::GetIOTimesTotal
*/
    bool StatisticalLogicalDiskInstance::GetIOTimesTotal(double& total) const
    {
#if defined(aix) || defined(linux)
        total = 0;
        return false;
#else
        return StatisticalDiskInstance::GetIOTimesTotal(total);
#endif
    }

/*----------------------------------------------------------------------------*/
/**
   \copydoc SCXSystemLib::StatisticalDiskInstance::GetDiskQueueLength
*/
    bool StatisticalLogicalDiskInstance::GetDiskQueueLength(double& queueLength) const
    {
#if defined(sun)
        return StatisticalDiskInstance::GetDiskQueueLength(queueLength);
#else
        queueLength = 0;
        return false;
#endif
    }

    /*----------------------------------------------------------------------------*/
    /**
       \copydoc SCXSystemLib::StatisticalDiskInstance::Sample
    */
    void StatisticalLogicalDiskInstance::Sample()
    {
#if defined(hpux)
        SCXCoreLib::SCXHandle<DeviceInstance> di = m_deps->FindDeviceInstance(m_device);

        if ((0 == di) || (DiskDepend::s_cINVALID_INSTANCE == di->m_instance))
        {
            if (m_NrOfFailedFinds < 10)
            {
                SCX_LOGERROR(m_log, SCXCoreLib::StrAppend(L"Unable to find disk in device map: ", m_device));
                ++m_NrOfFailedFinds;
            }
            else if (m_NrOfFailedFinds == 10)
            {
                SCX_LOGERROR(m_log, SCXCoreLib::StrAppend(L"Unable to find disk in device map: ",
                                                          m_device)
                             .append(L" This has happened 10 times in a row for this device and will not be reported again."));
                ++m_NrOfFailedFinds;
            }
            return;
        }
        m_NrOfFailedFinds = 0;
            
        m_timeStamp.AddSample(time(0));

        struct pst_lvinfo lvi;
        memset(&lvi, 0, sizeof(lvi));
        if (di->m_instance < 0 || 1 != m_deps->pstat_getlv(&lvi, sizeof(lvi), 1, di->m_instance))
        { // Check if the struct has moved
            di->m_instance = FindLVInfoByID(di->m_devID);
            if (di->m_instance < 0 || 1 != m_deps->pstat_getlv(&lvi, sizeof(lvi), 1, di->m_instance))
            {
                SCX_LOGTRACE(m_log, SCXCoreLib::StrAppend(L"No instance for: ", m_device));
                return;
            }
        }
        // Sanity check of cahed instance id
        if (di->m_devID != ((lvi.psl_dev.psd_major << 24) | lvi.psl_dev.psd_minor))
        {
            SCX_LOGWARNING(m_log, L"Instance changed");
            di->m_instance = FindLVInfoByID(di->m_devID);
            return;
        }
        m_reads.AddSample(lvi.psl_rxfer);
        m_writes.AddSample(lvi.psl_wxfer);
        m_rBytes.AddSample(lvi.psl_rcount);
        m_wBytes.AddSample(lvi.psl_wcount);
        m_transfers.AddSample(m_reads[0] + m_writes[0]);
        m_tBytes.AddSample(m_rBytes[0] + m_wBytes[0]);
#elif defined(linux)
        std::vector<std::wstring> parts = m_deps->GetProcDiskStats(m_device);
        for (size_t i=0; parts.size() == 0 && i < m_samplerDevices.size(); ++i)
        {
            parts = m_deps->GetProcDiskStats(m_samplerDevices[i]);
        }
        m_timeStamp.AddSample(time(0));
        if (parts.size() > 6)
        {
            try
            {
                m_reads.AddSample(SCXCoreLib::StrToULong(parts[3]));
                m_writes.AddSample(SCXCoreLib::StrToULong(parts[5]));
                m_rBytes.AddSample(SCXCoreLib::StrToULong(parts[4])*m_sectorSize);
                m_wBytes.AddSample(SCXCoreLib::StrToULong(parts[6])*m_sectorSize);
                m_transfers.AddSample(m_reads[0] + m_writes[0]);
                m_tBytes.AddSample(m_rBytes[0] + m_wBytes[0]);
            }
            catch (const SCXCoreLib::SCXNotSupportedException& e)
            {
                SCX_LOGWARNING(m_log, std::wstring(L"Could not parse line from diskstats: ").append(L" - ").append(e.What()));
            }
        }
#elif defined(sun)
        try
        {
            // some fs types lack link from 'logical' to 'physical' drives; 
            // ignore them in this statistics
            if ( m_deps->FileSystemNoLinkToPhysical(m_fsType) )
                return;
            
            SCXCoreLib::SCXHandle<SCXKstat> kstat = m_deps->GetKstat(m_device, false);

            if (kstat.GetData() == 0)
            {
                SCX_LOGTRACE(m_log, L"Failed to get kstat path for disk: " + m_device);
                return;
            }
            bool isVopstats = SCXCoreLib::StrIsPrefix(kstat->GetName(),L"vopstats_",true);
            
            if (isVopstats)
            {
                SCX_LOGHYSTERICAL(m_log, L"Unable to sample vopstats data");
            }
            else
            {
                m_reads.AddSample(kstat->GetValue(L"reads"));
                m_writes.AddSample(kstat->GetValue(L"writes"));
                m_transfers.AddSample(m_reads[0] + m_writes[0]); 
                m_rBytes.AddSample(kstat->GetValue(L"nread"));
                m_wBytes.AddSample(kstat->GetValue(L"nwritten"));
                m_tBytes.AddSample(m_rBytes[0] + m_wBytes[0]);
                m_runTimes.AddSample(kstat->GetValue(L"rtime")/1000/1000); //Store ms not ns
                m_waitTimes.AddSample(kstat->GetValue(L"wtime")/1000/1000); //Store ms not ns
                m_timeStamp.AddSample(MAX(kstat->GetValue(L"rlastupdate"),kstat->GetValue(L"wlastupdate"))/1000/1000);
                m_qLengths.AddSample(kstat->GetValue(L"wcnt"));
                
                SCX_LOGHYSTERICAL(m_log, L"nR=" + SCXCoreLib::StrFrom(m_reads[0])
                                  + L" nW=" + SCXCoreLib::StrFrom(m_writes[0])
                                  + L" bR=" + SCXCoreLib::StrFrom(m_rBytes[0])
                                  + L" bW=" + SCXCoreLib::StrFrom(m_wBytes[0])
                                  + L" tR=" + SCXCoreLib::StrFrom(m_runTimes[0])
                                  + L" tW=" + SCXCoreLib::StrFrom(m_waitTimes[0])
                                  + L" ts=" + SCXCoreLib::StrFrom(m_timeStamp[0]));
            }
        }
        catch (SCXKstatException& e)
        {
            SCX_LOGERROR(m_log, L"Unable to access kstat: " + e.What() + L" - " + e.Where());
        }
#endif
    }

/*----------------------------------------------------------------------------*/
/**
   \copydoc SCXSystemLib::StatisticalDiskInstance::GetLastMetrics
*/
    bool StatisticalLogicalDiskInstance::GetLastMetrics(scxulong& numR, scxulong& numW, scxulong& bytesR, scxulong& bytesW, scxulong& msR, scxulong& msW) const
    {
#if defined(aix)
        numR = numW = bytesR = bytesW = msR = msW = 0;
        return true;
#else
        if (0 == m_reads.GetNumberOfSamples())
        {
            return false;
        }
        numR = m_reads[0];
        if (0 == m_writes.GetNumberOfSamples())
        {
            return false;
        }
        numW = m_writes[0];
        if (0 == m_rBytes.GetNumberOfSamples())
        {
            return false;
        }
        bytesR = m_rBytes[0];
        if (0 == m_wBytes.GetNumberOfSamples())
        {
            return false;
        }
        bytesW = m_wBytes[0];
#if defined(linux) || defined(hpux)
        msR = 0;
        msW = 0;
#else
        if (0 == m_runTimes.GetNumberOfSamples())
        {
            return false;
        }
        msR = m_runTimes[0];
        if (0 == m_waitTimes.GetNumberOfSamples())
        {
            return false;
        }
        msW = m_waitTimes[0];
#endif
        return true;
#endif
    }
}
/*----------------------------E-N-D---O-F---F-I-L-E---------------------------*/
