/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
    \file

    \brief       Implementation of network interface PAL

    \date        08-03-03 12:12:02

*/
/*----------------------------------------------------------------------------*/
#include <scxcorelib/scxcmn.h>
#include "networkinterface.h"
#include <scxcorelib/scxassert.h>
#include <scxcorelib/scxfile.h>
#include <scxcorelib/stringaid.h>
#include <scxcorelib/scxexception.h>

#if defined(aix)
#include <libperfstat.h>
#endif

#include <sys/types.h>
#include <sys/socket.h>
#include <net/if.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <errno.h>
#include <vector>
#include <unistd.h>

#if defined(sun)
#include <sys/sockio.h>
#endif

using namespace SCXCoreLib;
using namespace std;

namespace {

using namespace SCXSystemLib;
using namespace SCXCoreLib;


#if defined(sun) || defined(linux)
/*----------------------------------------------------------------------------*/
//! Read a network interface name from a stream
//! \param[in]  source  To read from
//! \returns    Network interface name
std::wstring ReadInterfaceName(std::wistream &source) {
    std::wstring text;
    while (SCXStream::IsGood(source) && source.peek() == ' ') {
        source.get();
    }
    while (SCXStream::IsGood(source) && source.peek() != ':') {
        text += source.get();
    }
    source.get();
    return text;
}

#endif

/*----------------------------------------------------------------------------*/
// RAII encapsulation of file descriptor
class FileDescriptor {
public:
    //! Constructor
    //! \param[in]    Descriptor  To be managed
    FileDescriptor(int descriptor) : m_descriptor(descriptor) {
    }

    //! Destructor
    ~FileDescriptor() {
        if (m_descriptor >= 0) {
            close(m_descriptor);
        }
    }

    //! Enable normal use of the descriptor
    operator int() {
        return m_descriptor;
    }

private:
    int m_descriptor;   //!< Native descriptor to be managed
};

/*----------------------------------------------------------------------------*/
//! Convert a socket address to textual format
//! \param[in]  addr To be converted
//! \returns    Textual format
std::wstring ToString(sockaddr &addr) {
    std::wostringstream ipAddress;
    ipAddress << (unsigned char) addr.sa_data[2];
    ipAddress << L"." << (unsigned char) addr.sa_data[3];
    ipAddress << L"." << (unsigned char) addr.sa_data[4];
    ipAddress << L"." << (unsigned char) addr.sa_data[5];
    return ipAddress.str();
}

#if defined(sun)

/*----------------------------------------------------------------------------*/
//! Retrieve the value of an attribute
//! \param[in]    attr     Pointer to structure containing attribute
//! \param[out]   attrId   Identifier of attribute
//! \param[out]   knownAttributesMask  Will contain id of attribute if value is present
//! \returns      Value of attribute if present, undefined otherwise
scxulong ValueOf(const kstat_named *attr, NetworkInterfaceInfo::OptionalAttribute attrId, unsigned &knownAttributesMask) {
    scxulong value = 0;
    if (attr != 0) {
        value = attr->value.ui64;
        knownAttributesMask |= attrId;
    }
    return value;
}

/*----------------------------------------------------------------------------*/
//! Retrieve the "best" value of an attribute
//! \param[in]    attr64   Pointer to structure containing attribute
//! \param[in]    attr     Pointer to structure containing attribute
//! \param[out]   attrId   Identifier of attribute
//! \param[out]   knownAttributesMask  Will contain id of attribute if value is present
//! \returns      Best value of attribute if present, undefined otherwise
scxulong BestValueOf(const kstat_named *attr64, const kstat_named *attr, NetworkInterfaceInfo::OptionalAttribute attrId, unsigned &knownAttributesMask) {
    scxulong value = 0;
    if (attr64 != 0) {
        value = attr64->value.ui64;
        knownAttributesMask |= attrId;
    } else if (attr != 0) {
        value = attr->value.ui64;
        knownAttributesMask |= attrId;
    }
    return value;
}

#endif
}

#if defined(sun)

/*----------------------------------------------------------------------------*/
//! Constructor
KStatAPI::KStatAPI() {
}

/*----------------------------------------------------------------------------*/
//! Initialize a kstat control structure that provides access to the
//! kernel statistics library
//! \returns Pointer to kstat control structure
kstat_ctl_t *KStatAPI::kstat_open() {
    return ::kstat_open();
}

/*----------------------------------------------------------------------------*/
//! Free all resources that were associated with the kstat control structure
void KStatAPI::kstat_close(kstat_ctl_t *kstat) {
    ::kstat_close(kstat);
}

/*----------------------------------------------------------------------------*/
//! Destructor
KStatAPI::~KStatAPI() {
}

/*----------------------------------------------------------------------------*/
//! Search KStat data for an attribute of a certain name
//! \param[in]    header       KStat header in chain
//! \param[in]    name         Name of attribute to search for
//! \returns      Pointer to attribute structure, or NULL if attrbute does not exist
kstat_named_t *KStatAPI::kstat_named_lookup(kstat_t *header, const std::string &name
 ) const {
    std::vector<char> namebuf(name.size() + 1);
    strncpy(&namebuf[0], name.c_str(), namebuf.size());
    return static_cast<kstat_named_t *> (kstat_data_lookup(header, &namebuf[0]));
}

/*----------------------------------------------------------------------------*/
//! Read data associated with a particular header
//! \param[in]  kstat   KStat control structure
//! \param[in]  header  KStat header
//! \param[in]  buf     Pointer to buffer to be populated with data
//! \returns Version of data
kid_t KStatAPI::kstat_read(kstat_ctl_t *kstat, kstat_t *header, void *buf) const {
    return ::kstat_read(kstat, header, buf);
}

/*----------------------------------------------------------------------------*/
//! RAII encapsulation of KStat handle
class KStatHandle {
public:
    //! Constructor
    //! \param[in]   deps    Component external dependencies
    KStatHandle(SCXHandle<NetworkInterfaceDependencies> deps) : m_deps(deps) {
        if (NULL == sm_handle) {
            sm_handle = deps->kstat_open();

            if (sm_handle == 0) {
                throw SCXErrnoException(L"kstat_open", errno, SCXSRCLOCATION);
            }
        }
        SCXASSERT( NULL != sm_handle );
    }

    //! Destructor
    ~KStatHandle() { }

    //! Enable normal use of the handle
    operator kstat_ctl *()  {
        SCXASSERT( NULL != sm_handle );
        return sm_handle;
    }

    //! Enable normal use of the handle
    kstat_ctl *operator->()  {
        SCXASSERT( NULL != sm_handle );
        return sm_handle;
    }

    //! Close the KStat control chain if opened
    //! \param[in]  bJustReset    Flag to just reset handle (needed by test code)
    void CloseControlChain(bool bJustReset) {
        if (NULL != sm_handle) {
            if (!bJustReset) {
                m_deps->kstat_close(sm_handle);
            }
            sm_handle = NULL;
        }
    }

private:
    SCXHandle<NetworkInterfaceDependencies> m_deps; //!< Component external dependencies
    static kstat_ctl *sm_handle;  //!< Native KStat handle
};

kstat_ctl *KStatHandle::sm_handle = NULL;

#endif

namespace SCXSystemLib {

std::vector<std::wstring> NetworkInterfaceInfo::s_validInterfaces;


#if defined(sun)
/*----------------------------------------------------------------------------*/
//! Hook to close the KStat control chain (Necessary to get at KStatHandle class, local to this module)
//! \param[in]     deps             Dependencies to rely on
//! \param[in]     bJustReset       Flag to just reset handle (needed by test code)
void NetworkInterfaceInfo::ResetKStat(SCXHandle<NetworkInterfaceDependencies> deps, bool bJustReset) {
    KStatHandle kstat(deps);
    kstat.CloseControlChain(bJustReset);
}

/*----------------------------------------------------------------------------*/
//! Find all network interfaces using the KStat API
//! \param[out]    interfaces       To be populated
//! \param[in]     deps             Dependencies to rely on
//! \throws        SCXErrnoException   KStat problems
void NetworkInterfaceInfo::FindAllUsingKStat(std::vector<NetworkInterfaceInfo> &interfaces,
                                             SCXHandle<NetworkInterfaceDependencies> deps) {
    KStatHandle kstat(deps);
    for (kstat_t *header = kstat->kc_chain; header != NULL; header = header->ks_next) {
        if (strcmp(header->ks_class, "net") == 0 && header->ks_type == KSTAT_TYPE_NAMED) {
            if (deps->kstat_read(kstat, header, NULL) < 0) {
                throw SCXErrnoException(L"kstat_read", errno, SCXSRCLOCATION);
            }
            kstat_named_t *ipackets = deps->kstat_named_lookup(header, "ipackets");
            kstat_named_t *opackets = deps->kstat_named_lookup(header, "opackets");
            kstat_named_t *ipackets64 = deps->kstat_named_lookup(header, "ipackets64");
            kstat_named_t *opackets64 = deps->kstat_named_lookup(header, "opackets64");
            kstat_named_t *rbytes = deps->kstat_named_lookup(header, "rbytes");
            kstat_named_t *obytes = deps->kstat_named_lookup(header, "obytes");
            kstat_named_t *rbytes64 = deps->kstat_named_lookup(header, "rbytes64");
            kstat_named_t *obytes64 = deps->kstat_named_lookup(header, "obytes64");
            kstat_named_t *ierrors = deps->kstat_named_lookup(header, "ierrors");
            kstat_named_t *oerrors = deps->kstat_named_lookup(header, "oerrors");
            kstat_named_t *collisions = deps->kstat_named_lookup(header, "collisions");
            kstat_named_t *lbufs = deps->kstat_named_lookup(header, "lbufs");
            if ((0 == lbufs // lbufs is not a valid property for network interfaces
                ) && (
                ipackets != NULL || opackets != NULL || ipackets64 != NULL || opackets64 != NULL ||
                rbytes != NULL || obytes != NULL || rbytes64 != NULL || obytes64 != NULL ||
                ierrors != NULL || oerrors != NULL || collisions != NULL))
            {
                NetworkInterfaceInfo instance(deps);
                instance.m_name = StrFromMultibyte(header->ks_name);
                instance.m_packetsSent = BestValueOf(opackets64, opackets, ePacketsSent, instance.m_knownAttributesMask);
                instance.m_packetsReceived = BestValueOf(ipackets64, ipackets, ePacketsReceived, instance.m_knownAttributesMask);
                instance.m_bytesSent = BestValueOf(obytes64, obytes, eBytesSent, instance.m_knownAttributesMask);
                instance.m_bytesReceived = BestValueOf(rbytes64, rbytes, eBytesReceived, instance.m_knownAttributesMask);
                instance.m_errorsSending = ValueOf(oerrors, eErrorsSending, instance.m_knownAttributesMask);
                instance.m_errorsReceiving = ValueOf(ierrors, eErrorsReceiving, instance.m_knownAttributesMask);
                instance.m_collisions = ValueOf(collisions, eCollisions, instance.m_knownAttributesMask);
                interfaces.push_back(instance);
            }
        }
    }
}

#endif

#if defined(linux)
/*----------------------------------------------------------------------------*/
//! Find all network interfaces using the KStat API
//! \param[out]    interfaces          To be populated
//! \param[in]     deps                Dependencies to rely on
//! \throws        SCXErrnoException   KStat problems
void NetworkInterfaceInfo::FindAllInFile(std::vector<NetworkInterfaceInfo> &interfaces,
                                         SCXHandle<NetworkInterfaceDependencies> deps) {
    std::vector<std::wstring> lines;
    SCXStream::NLFs foundNlfs;
    SCXFile::ReadAllLines(deps->GetDynamicInfoFile(), lines, foundNlfs);
    for (size_t nr = 2; nr < lines.size(); nr++) {
        std::wistringstream infostream(lines[nr]);
        infostream.exceptions(std::ios::failbit | std::ios::badbit);
        NetworkInterfaceInfo instance(deps);
        instance.m_name = ReadInterfaceName(infostream);
        scxlong skip;
        infostream >> instance.m_bytesReceived;
        instance.m_knownAttributesMask |= eBytesReceived;
        infostream >> instance.m_packetsReceived;
        instance.m_knownAttributesMask |= ePacketsReceived;
        infostream >> instance.m_errorsReceiving;
        instance.m_knownAttributesMask |= eErrorsReceiving;
        infostream >> skip;
        infostream >> skip;
        infostream >> skip;
        infostream >> skip;
        infostream >> skip;
        infostream >> instance.m_bytesSent;
        instance.m_knownAttributesMask |= eBytesSent;
        infostream >> instance.m_packetsSent;
        instance.m_knownAttributesMask |= ePacketsSent;
        infostream >> instance.m_errorsSending;
        instance.m_knownAttributesMask |= eErrorsSending;
        infostream >> skip;
        infostream >> skip;
        infostream >> instance.m_collisions;
        instance.m_knownAttributesMask |= eCollisions;

        interfaces.push_back(instance);

    }

}
#endif

#if defined(aix)
/*----------------------------------------------------------------------------*/
//! Find all network interfaces using the perfstat API
//! \param[out]    interfaces       To be populated
//! \param[in]     deps             Dependencies to rely on
//! \throws        SCXErrnoException   perfstat problems
void NetworkInterfaceInfo::FindAllUsingPerfStat(std::vector<NetworkInterfaceInfo> &interfaces,
                                             SCXHandle<NetworkInterfaceDependencies> deps) {

    perfstat_id_t first;
    int structsAvailable = deps->perfstat_netinterface(NULL, NULL, sizeof(perfstat_netinterface_t), 0);
    if (structsAvailable < 0) {
        throw SCXErrnoException(L"perfstat_netinterface", errno, SCXSRCLOCATION);
    }
    vector<char> buffer(structsAvailable * sizeof(perfstat_netinterface_t));
    perfstat_netinterface_t *statp = reinterpret_cast<perfstat_netinterface_t *>(&buffer[0]);
    strcpy(first.name, FIRST_NETINTERFACE);
    int structsReturned = deps->perfstat_netinterface(&first, statp, sizeof(perfstat_netinterface_t), structsAvailable);
    if (structsReturned < 0) {
        throw SCXErrnoException(L"perfstat_netinterface", errno, SCXSRCLOCATION);
    }
    for (int nr = 0; nr < structsReturned; nr++) {
        // Currently there is no way to return type of network, our current CIM-model supports ethernet
        if (statp[nr].type == IFT_ETHER) {
            NetworkInterfaceInfo instance(deps);
            instance.m_name = StrFromMultibyte(statp[nr].name);

            instance.m_packetsSent = statp[nr].opackets;
            instance.m_knownAttributesMask |= ePacketsSent;

            instance.m_packetsReceived = statp[nr].ipackets;
            instance.m_knownAttributesMask |= ePacketsReceived;

            instance.m_bytesSent = statp[nr].obytes;
            instance.m_knownAttributesMask |= eBytesSent;

            instance.m_bytesReceived = statp[nr].ibytes;
            instance.m_knownAttributesMask |= eBytesReceived;

            instance.m_errorsSending = statp[nr].oerrors;
            instance.m_knownAttributesMask |= eErrorsSending;

            instance.m_errorsReceiving = statp[nr].ierrors;
            instance.m_knownAttributesMask |= eErrorsReceiving;

            instance.m_collisions = statp[nr].collisions;
            instance.m_knownAttributesMask |= eCollisions;

            interfaces.push_back(instance);
        }
    }
}
#endif

#if defined(hpux)
/*----------------------------------------------------------------------------*/
//! Find all whose name (without index) is baseName
//! \param[in]     baseName            Name except index
//! \param[out]    interfaces          To be populated
//! \param[in]     deps                Dependencies to rely on
void NetworkInterfaceInfo::FindAllNamed(const std::string &baseName,
                         std::vector<NetworkInterfaceInfo> &interfaces,
                         SCXCoreLib::SCXHandle<NetworkInterfaceDependencies> deps) {
    for (unsigned intfNr = 0; ; intfNr++) {
        FileDescriptor fd = socket(AF_INET, SOCK_DGRAM, 0);
        struct ifreq ifr;
        memset(&ifr, 0, sizeof(ifr));
        ostringstream name;
        name << baseName.c_str() << intfNr;
        strcpy(ifr.ifr_name, name.str().c_str());
        if (deps->ioctl(fd, SIOCGIFFLAGS, &ifr) < 0) {
            break;
        }

        NetworkInterfaceInfo instance(deps);
        instance.m_name = StrFromMultibyte(name.str());
        interfaces.push_back(instance);
    }

}
#endif

#if defined(linux)
/*----------------------------------------------------------------------------*/
//! Retrieves the name of the file containing dynamic network interface properties
//! \returns    Path to file
SCXCoreLib::SCXFilePath NetworkInterfaceDependencies::GetDynamicInfoFile() const {
    return L"/proc/net/dev";
}

#endif

/*----------------------------------------------------------------------------*/
//! Perform a variety of control functions on devices
//! \param[in]     fildes    Open file descriptor
//! \param[in]     request   Control function selected
//! \param[in]     ifreqptr  Pointer to buffer to be initialized
int NetworkInterfaceDependencies::ioctl(int fildes, int request, void *ifreqptr) {
    return ::ioctl(fildes, request, ifreqptr);
}

#if defined(aix)
/*----------------------------------------------------------------------------*/
//! Find performance statistics for network interface
//! \param[in]   name    Either FIRST_NETINTERFACE, or the first network interface of interest
//! \param[in]   userbuff  Points to a memory area to be filled with structs
//! \param[in]   sizeof_struct Number of bytes of a struct
//! \param[in]   desiredNumber    Number of structs to fetch
//! \returns Number of structs copied to buffer, if name and userbuff are non null
//! If name or userbuff is NULL no structs are copied and the function returns the number of structs
//! that would have been copied.
int  NetworkInterfaceDependencies::perfstat_netinterface(perfstat_id_t *name, perfstat_netinterface_t *userbuff,
                                                         size_t sizeof_struct, int desiredNumber) {
    return ::perfstat_netinterface(name, userbuff, sizeof_struct, desiredNumber);
}
#endif

/*----------------------------------------------------------------------------*/
//! Make the information correspond to the current state of the system
void NetworkInterfaceInfo::Refresh() {
    vector<NetworkInterfaceInfo> latestInterfaces(FindAll(m_deps));
    for (size_t nr = 0; nr < latestInterfaces.size(); nr++) {
        if (latestInterfaces[nr].GetName() == GetName()) {
            *this = latestInterfaces[nr];
            break;
        }
    }

}

/*----------------------------------------------------------------------------*/
//! Find all network interfaces on the machine
//! \param[in]  deps    What this PAL depends on
//! \returns    Information on the network instances
std::vector<NetworkInterfaceInfo> NetworkInterfaceInfo::FindAll(SCXHandle<NetworkInterfaceDependencies> deps) {
    std::vector<NetworkInterfaceInfo> interfaces;
#if defined(linux)
    FindAllInFile(interfaces, deps);
#elif defined(sun)
    FindAllUsingKStat(interfaces, deps);
#elif defined(hpux)
    // The names are standardized according to
    // ifconfig man page. There is a wi5006
    // to collect the names in a more "proper" way
    FindAllNamed("lan", interfaces, deps);
    FindAllNamed("iptu", interfaces, deps);
    FindAllNamed("ip6tu", interfaces, deps);
    FindAllNamed("lo", interfaces, deps);
#elif defined(aix)
    FindAllUsingPerfStat(interfaces, deps);
#else
#error "Platform not supported"
#endif
    FileDescriptor fd = socket(AF_INET, SOCK_DGRAM, 0);
    struct ifreq ifr;
    memset(&ifr, 0, sizeof(ifr));
    for (size_t nr = 0; nr < interfaces.size(); nr++) {
        NetworkInterfaceInfo &instance = interfaces[nr];
        strcpy(ifr.ifr_name, StrToMultibyte(instance.m_name).c_str());
        if (deps->ioctl(fd, SIOCGIFADDR, &ifr) >= 0) {
            instance.m_ipAddress = ToString(ifr.ifr_addr);
            instance.m_knownAttributesMask |= eIPAddress;
        }
        if (deps->ioctl(fd, SIOCGIFNETMASK, &ifr) >= 0) {
            instance.m_netmask = ToString(ifr.ifr_addr);
            instance.m_knownAttributesMask |= eNetmask;
        }
        if (deps->ioctl(fd, SIOCGIFBRDADDR, &ifr) >= 0) {
            instance.m_broadcastAddress = ToString(ifr.ifr_addr);
            instance.m_knownAttributesMask |= eBroadcastAddress;
        }
        if (deps->ioctl(fd, SIOCGIFFLAGS, &ifr) >= 0) {
            instance.m_up = (ifr.ifr_flags & IFF_UP) != 0;
            instance.m_running = (ifr.ifr_flags & IFF_RUNNING) != 0;
            instance.m_knownAttributesMask |= eUp;
            instance.m_knownAttributesMask |= eRunning;
        }
    }

    std::vector<NetworkInterfaceInfo> resultList;
    for (size_t nr = 0; nr < interfaces.size(); nr++) {
        NetworkInterfaceInfo &instance = interfaces[nr];

        // If this interface is "UP" or "RUNNING", add it to our valid list (if needed)
        if ((instance.IsKnownIfUp() && instance.IsUp()) || (instance.IsKnownIfRunning() && instance.IsRunning())) {
            if (!IsOrWasRunningInterface(instance.GetName()))
            {
                s_validInterfaces.push_back(instance.GetName());
            }
        }

        // Only return the interface if it's in our valid list

        if (IsOrWasRunningInterface(instance.GetName()))
        {
            resultList.push_back(instance);
        }
    }

    return resultList;
}

/*----------------------------------------------------------------------------*/
//! Construct an instance out of known information
//! \param[in]  name    Name that identifies an interface
//! \param[in]  knownAttributesMask Bitmask where bits set indicates existing optional attriutes
//! \param[in]  ipAddress   IP-address assigned to the interface
//! \param[in]  netmask     Netmask indicating which network the interface belongs to
//! \param[in]  broadcastAddress    Broadcast address used by the interface
//! \param[in]  bytesSent           Number of bytes sent from the interface
//! \param[in]  bytesReceived       Number of bytes received on the interface
//! \param[in]  packetsSent         Number of packets sent from the interface
//! \param[in]  packetsReceived     Number of packets received on the interface
//! \param[in]  errorsSending       Number of errors that occurred when sending from the interface
//! \param[in]  errorsReceiving     Number of errors that occurred when receiving on the interface
//! \param[in]  collisions          Number of collisions that occured in communication
//! \param[in]  up                  Is the the interface "up"
//! \param[in]  running             Is the interface "running"
//! \param[in]  deps                Dependencies to rely on when information is refreshed
NetworkInterfaceInfo::NetworkInterfaceInfo(const std::wstring &name, unsigned knownAttributesMask,
        const std::wstring &ipAddress, const std::wstring &netmask, const std::wstring &broadcastAddress,
        scxulong bytesSent, scxulong bytesReceived,
        scxulong packetsSent, scxulong packetsReceived,
        scxulong errorsSending, scxulong errorsReceiving,
        scxulong collisions,
        bool up, bool running,
        SCXCoreLib::SCXHandle<NetworkInterfaceDependencies> deps)
        : m_name(name), m_knownAttributesMask(knownAttributesMask),
          m_ipAddress(ipAddress), m_netmask(netmask), m_broadcastAddress(broadcastAddress),
          m_bytesSent(bytesSent), m_bytesReceived(bytesReceived),
          m_packetsSent(packetsSent), m_packetsReceived(packetsReceived),
          m_errorsSending(errorsSending), m_errorsReceiving(errorsReceiving),
          m_collisions(collisions),
          m_up(up), m_running(running),
          m_deps(deps) {

}


/*----------------------------------------------------------------------------*/
//! Private constructor
//! \param[in]  deps    Dependencies to rely on
NetworkInterfaceInfo::NetworkInterfaceInfo(SCXHandle<NetworkInterfaceDependencies> deps) :
    m_knownAttributesMask(0),
    m_bytesSent(0), m_bytesReceived(0),
    m_packetsSent(0), m_packetsReceived(0),
    m_errorsSending(0), m_errorsReceiving(0),
    m_collisions(0), m_up(false), m_running(false),
    m_deps(deps)
{
}


}


/*----------------------------E-N-D---O-F---F-I-L-E---------------------------*/
