/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
    \file        

    \brief       Implementation of network interface enumeration PAL
    
    \date        08-03-03 12:12:02

    
*/
/*----------------------------------------------------------------------------*/

#include <scxcorelib/scxcmn.h>
#include <scxsystemlib/networkinterfaceenumeration.h>
#include "networkinterface.h"

using namespace std;
using namespace SCXCoreLib;

namespace SCXSystemLib {

/*----------------------------------------------------------------------------*/
//! Constructs an enumeration dependent on the actual system
NetworkInterfaceEnumeration::NetworkInterfaceEnumeration()
        : m_log(SCXLogHandleFactory::GetLogHandle(
                L"scx.core.common.pal.system.networkinterface.networkinterfaceenumeration")),
          m_deps(new NetworkInterfaceDependencies())
{    
}

/*----------------------------------------------------------------------------*/
//! Constructs an enumeration dependent on injected dependencies rather than the actual system.
//! Useful for testing behaviour in a reproducable manner.
NetworkInterfaceEnumeration::NetworkInterfaceEnumeration(SCXHandle<NetworkInterfaceDependencies> deps)
        : m_log(SCXLogHandleFactory::GetLogHandle(
                L"scx.core.common.pal.system.networkinterface.networkinterfaceenumeration")),
          m_deps(deps)
{    
}

/*----------------------------------------------------------------------------*/
//! Destructor
NetworkInterfaceEnumeration::~NetworkInterfaceEnumeration() {
}

/*----------------------------------------------------------------------------*/
//! Implementation of the Init method of the entity framework.
void NetworkInterfaceEnumeration::Init() {
    UpdateEnumeration();
}

/*----------------------------------------------------------------------------*/
/**
   Implementation of the Update method of the entity framework.

   \param updateInstances - indicates whether only the existing instances shall be updated. 
   
   The method refreshes the set of known instances in the enumeration. 
   
   Any newly created instances must have a well-defined state after execution, 
   meaning that instances which update themselves have to init themselves upon 
   creation. 
*/
void NetworkInterfaceEnumeration::Update(bool updateInstances) {
    if (updateInstances) {
        UpdateInstances();        
    } else {
        UpdateEnumeration();
    }
    
}

/*----------------------------------------------------------------------------*/
//! Run the Update() method on all instances in the colletion, including the
//! Total instance if any.
//! \note Optimized implementation that recreates the same result as running update on
//!       each instance, but does not actually do so
void NetworkInterfaceEnumeration::UpdateInstances() {
    vector<NetworkInterfaceInfo> latestInterfaces = NetworkInterfaceInfo::FindAll(m_deps);
    typedef map<wstring, size_t> IndexByStrMap;
    IndexByStrMap latestInterfaceById;

    // Create an index of the latest instances by their id
    for (size_t nr = 0; nr < latestInterfaces.size(); nr++) {
        latestInterfaceById.insert(IndexByStrMap::value_type(NetworkInterfaceInstance(latestInterfaces[nr]).GetId(), nr));
    }    
    
    for (EntityIterator oldIter = Begin(); oldIter != End(); oldIter++) {
        IndexByStrMap::iterator stillExistingInterfacePos = latestInterfaceById.find((*oldIter)->GetId());        
        if (stillExistingInterfacePos != latestInterfaceById.end()) {                                    
            // Update instances that still exists
            (*oldIter)->Update(latestInterfaces[stillExistingInterfacePos->second]);
        }
    }
    
}

/*----------------------------------------------------------------------------*/
//! Make the enumeration correspond to the current state of the system
void NetworkInterfaceEnumeration::UpdateEnumeration() {
    vector<NetworkInterfaceInfo> latestInterfaces = NetworkInterfaceInfo::FindAll(m_deps);
    typedef map<wstring, size_t> IndexByStrMap;
    IndexByStrMap newInterfaceById;

    // Prepare an index of new instances by their id
    for (size_t nr = 0; nr < latestInterfaces.size(); nr++) {
        newInterfaceById.insert(IndexByStrMap::value_type(NetworkInterfaceInstance(latestInterfaces[nr]).GetId(), nr));
    }    
    
    for (size_t nr = Size();  nr > 0; nr--) {
        IndexByStrMap::iterator stillExistingInterfacePos = newInterfaceById.find(GetInstance(nr-1)->GetId());
        if (stillExistingInterfacePos != newInterfaceById.end()) {                                    
            // Update instances that still exists
            GetInstance(nr-1)->Update(latestInterfaces[stillExistingInterfacePos->second]);

            // Remove instances, that are not new, from index of new instances
            newInterfaceById.erase(stillExistingInterfacePos);
        } else { 
            // Remove instances, that no longer exists, from the enumeration
            RemoveInstance(Begin() + nr - 1);
        }
    }
    
    // Add new instances to the enumeration
    for (IndexByStrMap::iterator iter = newInterfaceById.begin(); iter != newInterfaceById.end(); iter++) {
        const NetworkInterfaceInfo &intf = latestInterfaces[iter->second];
        bool knownState = intf.IsKnownIfUp() && intf.IsKnownIfRunning();
        bool acceptedAddress = !intf.IsIPAddressKnown() || intf.GetIPAddress().find(L"127.0.0.") != 0; 
        // Fix according to WI5275, don't add uninteresting interfaces
        if (knownState && acceptedAddress) {
            AddInstance(
                SCXCoreLib::SCXHandle<NetworkInterfaceInstance>(new NetworkInterfaceInstance(latestInterfaces[iter->second])));
        }
    }
}

}

/*----------------------------E-N-D---O-F---F-I-L-E---------------------------*/
