/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
    \file        

    \brief       Implementation of network interface instance PAL
    
    \date        08-03-03 12:12:02

    
*/
/*----------------------------------------------------------------------------*/

#include <scxcorelib/scxcmn.h>
#include <scxsystemlib/networkinterfaceinstance.h>
#include "networkinterface.h"
#include <vector>

using namespace std;

namespace SCXSystemLib {

/*----------------------------------------------------------------------------*/
//! Constructor
//! \param[in]   info     Initial information
NetworkInterfaceInstance::NetworkInterfaceInstance(const NetworkInterfaceInfo &info) 
        : EntityInstance(info.GetName()), m_info(new NetworkInterfaceInfo(info)) {
    
}

/*----------------------------------------------------------------------------*/
//! Destructor
NetworkInterfaceInstance::~NetworkInterfaceInstance() {
}

/*----------------------------------------------------------------------------*/
//! Name of interface
//! \returns Name
std::wstring NetworkInterfaceInstance::GetName() const {
    return m_info->GetName();
}

/*----------------------------------------------------------------------------*/
//!  \copydoc SCXSystemLib::EntityInstance::Update()
void NetworkInterfaceInstance::Update() {
    m_info->Refresh();
}

/*----------------------------------------------------------------------------*/
//! Make the content correspond to given information
//! \param[in]   info    New content
void NetworkInterfaceInstance::Update(const NetworkInterfaceInfo &info) {
    (*m_info) = info; 
}

/*----------------------------------------------------------------------------*/
//! IP Address assigned to interface
//! \param[out] value     Reference to value of property to be initialized
//! \returns    true iff property is supported
bool NetworkInterfaceInstance::GetIPAddress(std::wstring &value) const {
    bool supported = false;
    if (m_info->IsValueKnown(NetworkInterfaceInfo::eIPAddress)) {
        value = m_info->GetIPAddress();
        supported = true;
    }
    return supported;
}

/*----------------------------------------------------------------------------*/
//! Netmask assigned to interface
//! \param[out] value     Reference to value of property to be initialized
//! \returns    true iff property is supported
bool NetworkInterfaceInstance::GetNetmask(std::wstring &value) const {
    bool supported = false;
    if (m_info->IsValueKnown(NetworkInterfaceInfo::eNetmask)) {
        value = m_info->GetNetmask();
        supported = true;
    }
    return supported;
}

/*----------------------------------------------------------------------------*/
//! Broadcast address assigned to interface
//! \param[out] value     Reference to value of property to be initialized
//! \returns    true iff property is supported
bool NetworkInterfaceInstance::GetBroadcastAddress(std::wstring &value) const {
    bool supported = false;
    if (m_info->IsValueKnown(NetworkInterfaceInfo::eBroadcastAddress)) {
        value = m_info->GetBroadcastAddress();
        supported = true;
    }
    return supported;
}

/*----------------------------------------------------------------------------*/
//! Number of bytes received from interface
//! \param[out] value     Reference to value of property to be initialized
//! \returns    true iff property is supported
bool NetworkInterfaceInstance::GetBytesReceived(scxulong &value) const {
    bool supported = false;
    if (m_info->IsValueKnown(NetworkInterfaceInfo::eBytesReceived)) {
        value = m_info->GetBytesReceived();
        supported = true;
    }
    return supported;
}

/*----------------------------------------------------------------------------*/
//! Number of bytes sent to interface
//! \param[out] value     Reference to value of property to be initialized
//! \returns    true iff property is supported
bool NetworkInterfaceInstance::GetBytesSent(scxulong &value) const {
    bool supported = false;
    if (m_info->IsValueKnown(NetworkInterfaceInfo::eBytesSent)) {
        value = m_info->GetBytesSent();
        supported = true;
    }
    return supported;
}
        
/*----------------------------------------------------------------------------*/
//! Number of packets received from interface
//! \param[out] value     Reference to value of property to be initialized
//! \returns    true iff property is supported
bool NetworkInterfaceInstance::GetPacketsReceived(scxulong &value) const {
    bool supported = false;
    if (m_info->IsValueKnown(NetworkInterfaceInfo::ePacketsReceived)) {
        value = m_info->GetPacketsReceived();
        supported = true;
    }
    return supported;
}

/*----------------------------------------------------------------------------*/
//! Number of packets sent to interface
//! \param[out] value     Reference to value of property to be initialized
//! \returns    true iff property is supported
bool NetworkInterfaceInstance::GetPacketsSent(scxulong &value) const {
    bool supported = false;
    if (m_info->IsValueKnown(NetworkInterfaceInfo::ePacketsSent)) {
        value = m_info->GetPacketsSent();
        supported = true;
    }
    return supported;
}

/*----------------------------------------------------------------------------*/
//! Number of errors that have occurred when receiving from interface
//! \param[out] value     Reference to value of property to be initialized
//! \returns    true iff property is supported
bool NetworkInterfaceInstance::GetErrorsReceiving(scxulong &value) const {
    bool supported = false;
    if (m_info->IsValueKnown(NetworkInterfaceInfo::eErrorsReceiving)) {
        value = m_info->GetErrorsReceiving();
        supported = true;
    }
    return supported;
}

/*----------------------------------------------------------------------------*/
//! Number of errors that have occurred when sending to interface
//! \param[out] value     Reference to value of property to be initialized
//! \returns    true iff property is supported
bool NetworkInterfaceInstance::GetErrorsSending(scxulong &value) const {
    bool supported = false;
    if (m_info->IsValueKnown(NetworkInterfaceInfo::eErrorsSending)) {
        value = m_info->GetErrorsSending();
        supported = true;
    }
    return supported;
}

/*----------------------------------------------------------------------------*/
//! Number of collisons that have occurred on interface
//! \param[out] value     Reference to value of property to be initialized
//! \returns    true iff property is supported
bool NetworkInterfaceInstance::GetCollisions(scxulong &value) const {
    bool supported = false;
    if (m_info->IsValueKnown(NetworkInterfaceInfo::eCollisions)) {
        value = m_info->GetCollisions();
        supported = true;
    }
    return supported;
}

/*----------------------------------------------------------------------------*/
//! Is the interface up
//! \returns true iff if up
bool NetworkInterfaceInstance::GetUp(bool &value) const {
    bool supported = false;
    if (m_info->IsValueKnown(NetworkInterfaceInfo::eUp)) {
        value =  m_info->IsUp();
        supported = true;
    }
    return supported;
}


/*----------------------------------------------------------------------------*/
//! Is the interface running
//! \returns true iff if running
bool NetworkInterfaceInstance::GetRunning(bool &value) const {
    bool supported = false;
    if (m_info->IsValueKnown(NetworkInterfaceInfo::eRunning)) {
        value = m_info->IsRunning();
        supported = true;
    }
    return supported;
}

}

/*----------------------------E-N-D---O-F---F-I-L-E---------------------------*/
