/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
    \file        

    \brief       Contains the definition of log file configurator class.

    \date        2008-07-22 08:55:15

*/
/*----------------------------------------------------------------------------*/
#ifndef SCXLOGCONFIGREADER_H
#define SCXLOGCONFIGREADER_H

#include <scxcorelib/scxlog.h>
#include <scxcorelib/scxhandle.h>
#include <scxcorelib/scxthread.h>
#include <scxcorelib/stringaid.h>
#include "scxlogfilebackend.h"
#include "scxlogmediator.h"

#include <list>

namespace SCXCoreLib
{
    /*----------------------------------------------------------------------------*/
    /**
        Implementation of the log configurator reader.
        to make reader generic enough for both config-tool and scx-core, template is used;
        template type ("BaseBackendType") must support the following operations:
        bool IsInitialized()
        bool SetProperty(const std::wstring& key, const std::wstring& value)
        
        // interface to the caller ("ConfigConsumerInterface"):
        SCXHandle< BaseBackendType > Create( const std::wstring& name ) = 0;
        void Add( SCXHandle< BaseBackendType > backend ) = 0;
        bool SetSeverityThreshold( SCXHandle< BaseBackendType > backend, const std::wstring& module, SCXLogSeverity newThreshold) = 0;
            
        };
    */
    template <class BaseBackendType, class ConfigConsumerInterface>
    class SCXLogConfigReader
    {
    public:
        SCXLogConfigReader(){}
        bool ParseConfigFile( const SCXFilePath& configFilePath, ConfigConsumerInterface* pInterface );

    private:
    };

    // helper funcitons
    SCXLogSeverity SCXLogConfigReader_TranslateSeverityString(const std::wstring& severityString);
    std::wstring SCXLogConfigReader_SeverityToString(SCXLogSeverity severity);


    // implementation
    /*----------------------------------------------------------------------------*/
    /**
        Parses the configuration file.
        \returns false if configuration could not be parsed.

        This is an example of what a file may look like:
        FILE (
        PATH: /var/log/scx.log
        MODULE: WARNING
        MODULE: scx.some.module TRACE
        )
    */
    template <class BaseBackendType, class ConfigConsumerInterface>
    bool SCXLogConfigReader<BaseBackendType, ConfigConsumerInterface>::ParseConfigFile( const SCXFilePath& configFilePath, ConfigConsumerInterface* pInterface )
    {
        std::vector<std::wstring> configLines;
        SCXStream::NLFs nlfs;
        SCXFile::ReadAllLinesAsUTF8(configFilePath,
                                    configLines,
                                    nlfs);
        bool validConfig = false;
        for (std::vector<std::wstring>::const_iterator i = configLines.begin();
             i != configLines.end();
             ++i)
        {
            SCXHandle< BaseBackendType > backend = pInterface->Create( *i );
            if (backend != 0)
            {
                for (++i ; i != configLines.end(); ++i)
                {
                    if (L")" == *i)
                    {
                        if (backend->IsInitialized())
                        {
                            pInterface->Add( backend );
                            validConfig = true;
                            break;
                        }
                        else
                        {
                            return false;
                        }
                    }
                    else
                    {
                        std::vector<std::wstring> lineTokens;
                        StrTokenize(*i, lineTokens, L":");
                        if (lineTokens.size() != 2)
                        {
                            continue;
                        }
                        if (L"MODULE" == lineTokens[0])
                        {
                            // This key is for the severity filter.
                            std::vector<std::wstring> severityTokens;
                            StrTokenize(lineTokens[1], severityTokens, L" ");
                            if (severityTokens.size() == 1)
                            {
                                // This should be severity for root module.
                                SCXLogSeverity severity = SCXLogConfigReader_TranslateSeverityString(severityTokens[0]);
                                pInterface->SetSeverityThreshold(backend, L"", severity);
                            }
                            else if (severityTokens.size() == 2)
                            {
                                // should be <module> <severity>
                                SCXLogSeverity severity = SCXLogConfigReader_TranslateSeverityString(severityTokens[1]);
                                pInterface->SetSeverityThreshold(backend, severityTokens[0], severity);
                            }
                        } 
                        else
                        {
                            backend->SetProperty(lineTokens[0], lineTokens[1]);
                        }
                    }
                }

                if ( i == configLines.end() )
                    break;
                
            }
        }

        return validConfig;
    }
    
    
}

#endif /* SCXLOGCONFIGREADER_H */
/*----------------------------E-N-D---O-F---F-I-L-E---------------------------*/
