/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
    \file        

    \brief       Implementation of the SCXLogHandle class.

    \date        07-06-07 14:27:00
    
*/
/*----------------------------------------------------------------------------*/

#include <scxcorelib/scxcmn.h>
#include <scxcorelib/stringaid.h>
#include <scxcorelib/scxdumpstring.h>
#include "scxlogitem.h"

namespace SCXCoreLib
{

    /*----------------------------------------------------------------------------*/
    /**
        Default constructor.
    
        Creates a default SCXLogItem.

    */
    SCXLogItem::SCXLogItem() :
        m_module(L""),
        m_severity(eNotSet),
        m_message(L""),
        m_location(L"", 0),
        m_threadId(0),
        m_timestamp(SCXCalendarTime::CurrentUTC())
    {
    }

    /*----------------------------------------------------------------------------*/
    /**
        Constructor with parameters.
    
        \param[in]  module The string representation of the module the log
                           item belongs to.
        \param[in]  severity The severity of the new log item.
        \param[in]  message The actual log message.
        \param[in]  location Source code location.
        \param[in]  threadId - Thread that caused the log.
             
         Creates a new log item and sets the data members.

    */
    SCXLogItem::SCXLogItem(const std::wstring& module,
                           SCXLogSeverity severity,
                           const std::wstring& message,
                           const SCXCodeLocation& location,
                           SCXThreadId threadId) :
        m_module(module),
        m_severity(severity),
        m_message(message),
        m_location(location),
        m_threadId(threadId),
        m_timestamp(SCXCalendarTime::CurrentUTC())
    {
    }

    /*----------------------------------------------------------------------------*/
    /**
        Copy constructor
    
        \param[in]  o SCXLogItem to copy.
    */
    SCXLogItem::SCXLogItem(const SCXLogItem& o) :
        m_module(o.m_module),
        m_severity(o.m_severity),
        m_message(o.m_message),
        m_location(o.m_location),
        m_threadId(o.m_threadId),
        m_timestamp(o.m_timestamp)
    {
    }

    /*----------------------------------------------------------------------------*/
    /**
        Destructor.
    */
    SCXLogItem::~SCXLogItem()
    {
    }

    /*----------------------------------------------------------------------------*/
    /**
        Assignment operator
    
        \param[in]  o SCXLogItem to copy.
        \returns *this
    */
    SCXLogItem& SCXLogItem::operator=(const SCXLogItem& o)
    {
        m_module = o.m_module;
        m_severity = o.m_severity;
        m_message = o.m_message;
        m_location = o.m_location;
        m_threadId = o.m_threadId;
        m_timestamp = o.m_timestamp;
        return *this;
    }

    /*----------------------------------------------------------------------------*/
    /**
        Returns a string representation of the module this log item belongs to.
    
    */
    const std::wstring& SCXLogItem::GetModule() const
    {
        return m_module;
    }

    /*----------------------------------------------------------------------------*/
    /**
        Returns the severity of this log item.
    
    */
    SCXLogSeverity SCXLogItem::GetSeverity() const
    {
        return m_severity;
    }

    /*----------------------------------------------------------------------------*/
    /**
        Returns the log message as passed by the developer.
    
    */
    const std::wstring& SCXLogItem::GetMessage() const
    {
        return m_message;
    }

    /*----------------------------------------------------------------------------*/
    /**
        Returns the source code location that generated this log item.
    
    */
    const SCXCodeLocation& SCXLogItem::GetLocation() const
    {
        return m_location;
    }

    /*----------------------------------------------------------------------------*/
    /**
        Returns the thread id of the thread that generated this log item.
    
    */
    SCXThreadId SCXLogItem::GetThreadId() const
    {
        return m_threadId;
    }

    /*----------------------------------------------------------------------------*/
    /**
        Returns the timestamp when the item was created.
    
    */
    const SCXCalendarTime& SCXLogItem::GetTimestamp() const
    {
        return m_timestamp;
    }

    /*----------------------------------------------------------------------------*/
    /**
        Dump object as string (for logging).
    
        \retval   String representation of object.
        
    */
    const std::wstring SCXLogItem::DumpString() const
    {
        return SCXDumpStringBuilder("SCXLogItem")
            .Text("module", m_module)
            .Instance("timestamp", m_timestamp)
            .Scalar("severity", m_severity)
            .Text("message", m_message);
    }
        
}
