/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
    \file

    \brief       Implementation for the simple log mediator class.

    \date        2008-07-18 13:01:31

*/
/*----------------------------------------------------------------------------*/

#include "scxlogmediatorsimple.h"
#include "scxlogitem.h"
#include <scxcorelib/scxdumpstring.h>

namespace SCXCoreLib
{
    /*----------------------------------------------------------------------------*/
    /**
        Default constructor.

    */
    SCXLogMediatorSimple::SCXLogMediatorSimple() :
        m_lock(ThreadLockHandleGet())
    {
    }

    /*----------------------------------------------------------------------------*/
    /**
        Constructor with injected thread lock handle.

        \param[in] lock Used to inject a thread lock handle.
    */
    SCXLogMediatorSimple::SCXLogMediatorSimple(const SCXThreadLockHandle& lock) :
        m_lock(lock)
    {
    }

    /*----------------------------------------------------------------------------*/
    /**
        Log a message. The supplied item is distributed to those backends that
        have registered themselves. This entry point is thread safe.

        \param[in] item Log item to add to the log mediator.

        \note This simple implementation is blocking. The calling thread must wait
        for the logging to complete.
    */
    void SCXLogMediatorSimple::LogThisItem(const SCXLogItem& item)
    {
        SCXThreadLock lock(m_lock);
        for (ConsumerSet::iterator i = m_Consumers.begin();
             i != m_Consumers.end();
             ++i)
        {
            (*i)->LogThisItem(item);
        }
    }

    /*----------------------------------------------------------------------------*/
    /**
        Register an SCXLogItemConsumerIf as a new receiver of log messages. It will
        recieve all items that were logged through the LogThisItem interface.

        \param[in] consumer SCXLogItemConsumerIf to register as consumer.
        \returns False if consumer can't be added.
    */
    bool SCXLogMediatorSimple::RegisterConsumer(SCXHandle<SCXLogItemConsumerIf> consumer)
    {
        SCXThreadLock lock(m_lock);
        m_Consumers.insert(consumer);
        return true;
    }

    /*----------------------------------------------------------------------------*/
    /**
        A registered consumer that is no longer interested in receiving SCXLogItems
        can de-register itself through this interface.

        \param[in] consumer SCXLogItemConsumerIf to de-register.
        \returns False if consumer was not previously registered.
    */
    bool SCXLogMediatorSimple::DeRegisterConsumer(SCXHandle<SCXLogItemConsumerIf> consumer)
    {
        SCXThreadLock lock(m_lock);
        return m_Consumers.erase(consumer) > 0;
    }

    /*----------------------------------------------------------------------------*/
    /**
        Get the effective severity for a particular log module.

        \param[in] module Log module to retrieve severity for.
        \returns Effective severity for log module.

    */
    SCXLogSeverity SCXLogMediatorSimple::GetEffectiveSeverity(const std::wstring& module) const
    {
        SCXLogSeverity effectiveSeverity = eSuppress;
        SCXThreadLock lock(m_lock);
        for (ConsumerSet::const_iterator i = m_Consumers.begin();
             i != m_Consumers.end();
             ++i)
        {
            SCXLogSeverity backendSeverity = (*i)->GetEffectiveSeverity(module);
            if (backendSeverity < effectiveSeverity)
            {
                effectiveSeverity = backendSeverity;
            }
            if (effectiveSeverity == eHysterical)
            {
                return eHysterical;
            }
        }
        return effectiveSeverity;
    }

    /*----------------------------------------------------------------------------*/
    /**
       Handle log rotations that have occurred
     */
    void SCXLogMediatorSimple::HandleLogRotate()
    {
        SCXThreadLock lock(m_lock);
        for (ConsumerSet::iterator i = m_Consumers.begin();
             i != m_Consumers.end();
             ++i)
        {
            (*i)->HandleLogRotate();
        }
    }

    /*----------------------------------------------------------------------------*/
    /**
        Dump object as string (for logging).

        \returns The object represented as a string suitable for logging.

    */
    const std::wstring SCXLogMediatorSimple::DumpString() const
    {
        SCXDumpStringBuilder dsb("SCXLogMediatorSimple");
        return dsb;
    }
} /* namespace SCXCoreLib */
/*----------------------------E-N-D---O-F---F-I-L-E---------------------------*/
