/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
    \file        

    \brief       Implementation of the SCXFilePersistDataWriter class.
    
    \date        2008-08-21 15:06:10

*/
/*----------------------------------------------------------------------------*/

#include <scxcorelib/scxcmn.h>
#include <scxcorelib/scxfile.h>
#include <scxcorelib/stringaid.h>
#include "scxfilepersistdatawriter.h"
#include <string>
#include <sstream>

namespace SCXCoreLib
{
    /*----------------------------------------------------------------------------*/
    /**
        Constructor

        \param[in]  path        Path where persisted data should be written.
        \param[in]  version     Version of data stored.
    */
    SCXFilePersistDataWriter::SCXFilePersistDataWriter(const SCXFilePath& path, unsigned int version) :
        SCXPersistDataWriter(version),
        m_StartedGroups(),
        m_Indentation(L"  ")
    {
        m_Stream = SCXFile::OpenFstream(path, std::ios_base::out);
        std::wostringstream os;
        os << L"<?xml version=\"1.0\" encoding='UTF-8' standalone='yes' ?>" << std::endl;
        os << L"<SCXPersistedData Version=\"" << version << L"\">" << std::endl;
        SCXStream::WriteAsUTF8(*m_Stream, os.str());
    }

    /*----------------------------------------------------------------------------*/
    /**
        Destructor
    */
    SCXFilePersistDataWriter::~SCXFilePersistDataWriter()
    {
    }

    /*----------------------------------------------------------------------------*/
    /**
        Mark the start of a new group.

        \param[in]  name     Name of group.
    */
    void SCXFilePersistDataWriter::WriteStartGroup(const std::wstring& name)
    {
        std::wostringstream os;
        os << m_Indentation << L"<Group Name=\"" << EncodeString(name) << L"\">" << std::endl;
        SCXStream::WriteAsUTF8(*m_Stream, os.str());    
        m_StartedGroups.push_front(name);
        m_Indentation.append(L"  ");
    }

    /*----------------------------------------------------------------------------*/
    /**
        Mark the end of the last started group.
        \throws SCXInvalidStateException if no group is currently started.
    */
    void SCXFilePersistDataWriter::WriteEndGroup()
    {
        if (m_StartedGroups.empty())
        {
            throw SCXInvalidStateException(L"No open group when calling WriteEndGroup.", SCXSRCLOCATION);
        }
        m_Indentation.erase(0, 2);
        std::wostringstream os;
        os << m_Indentation << L"</Group>" << std::endl;
        SCXStream::WriteAsUTF8(*m_Stream, os.str());
        m_StartedGroups.pop_front();
    }

    /*----------------------------------------------------------------------------*/
    /**
        Write a new name/value pair.

        \param[in]  name    Name of value.
        \param[in] value    Value.
    */
    void SCXFilePersistDataWriter::WriteValue(const std::wstring& name, const std::wstring& value)
    {
        std::wostringstream os;
        os << m_Indentation  << L"<Value Name=\""   << EncodeString(name)
           << L"\" Value=\"" << EncodeString(value) << L"\"/>" << std::endl;
        SCXStream::WriteAsUTF8(*m_Stream, os.str());
    }

    /*----------------------------------------------------------------------------*/
    /**
        Mark the end of writing data.
        Will also be called from destructor if not called explicitly
        \throws SCXInvalidStateException if all groups have not been ended.
    */
    void SCXFilePersistDataWriter::DoneWriting()
    {
        if ( ! m_StartedGroups.empty())
        {
            throw SCXInvalidStateException(L"Can not call DoneWriting when open groups still exist.", SCXSRCLOCATION);
        }
        if (m_Stream->is_open())
        {
            SCXStream::WriteAsUTF8(*m_Stream, L"</SCXPersistedData>\n");
            m_Stream->close();
        }
    }

    /*----------------------------------------------------------------------------*/
    /**
        Encode a string replacing special characters with their xml counterparts.
        \param[in] in String to encode.
        \returns Encoded string.
    */
    std::wstring SCXFilePersistDataWriter::EncodeString(const std::wstring& in) const
    {
        std::wstring out;
        for (std::wstring::const_iterator i = in.begin(); i != in.end(); ++i)
        {
            switch (*i)
            {
            case L'<':
                out.append(L"&lt;");
                break;
            case L'&':
                out.append(L"&amp;");
                break;
            case L'\'':
                out.append(L"&apos;");
                break;
            case L'\"':
                out.append(L"&quot;");
                break;
            default:
                out.push_back(*i);
                break;
            }
        }
        return out;
    }
}
/*----------------------------E-N-D---O-F---F-I-L-E---------------------------*/
