/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
    \file        

    \brief       Simple file based implementation of the SCXPersistMedia interface
    
    \date        2008-08-21 13:41:29

*/
/*----------------------------------------------------------------------------*/

#include <scxcorelib/scxcmn.h>
#include <scxcorelib/scxpersistence.h>
#include <scxcorelib/scxfilepath.h>
#include <scxcorelib/scxfile.h>
#include <scxcorelib/scxuser.h>
#include "scxfilepersistmedia.h"
#include "scxfilepersistdatareader.h"
#include "scxfilepersistdatawriter.h"

namespace SCXCoreLib {

    /*----------------------------------------------------------------------------*/
    /**
        Default constructor.
    */
    SCXFilePersistMedia::SCXFilePersistMedia() :
        m_BasePath(SCX_L(SCX_STATEDIR) L"/")
    {
        AddUserNameToBasePath();
    }

    /*----------------------------------------------------------------------------*/
    /**
        Destructor.
    */
    SCXFilePersistMedia::~SCXFilePersistMedia()
    {
    }

    /*----------------------------------------------------------------------------*/
    /**
        Create a new data reader and populate it with the data previously written
        with the given name.

        \param[in]  name     Name of data to populate reader with.
        \returns   The new reader.
        \throws an exception if no data previously written with the given name.
    */
    SCXHandle<SCXPersistDataReader> SCXFilePersistMedia::CreateReader(const std::wstring& name)
    {
        try
        {
            return SCXHandle<SCXPersistDataReader>(
                new SCXFilePersistDataReader(NameToFilePath(name)) );
        }
        catch (const SCXFilePathNotFoundException&)
        {
            throw PersistDataNotFoundException(name, SCXSRCLOCATION);
        }
        catch (const SCXUnauthorizedFileSystemAccessException&)
        {
            throw PersistDataNotFoundException(name, SCXSRCLOCATION);
        }
    }

    /*----------------------------------------------------------------------------*/
    /**
        Add name of current user to base path.
    */
    void SCXFilePersistMedia::AddUserNameToBasePath()
    {
        SCXUser user;
        if (!user.IsRoot())
        {
            m_BasePath.AppendDirectory(user.GetName());
        }
    }

    /*----------------------------------------------------------------------------*/
    /**
        Create a new data writer to write data with the given name.
        If data has previously been written with the same name, that data will be
        overwriten.

        \param[in]  name     Name of data to write.
        \param[in]  version  Version of data to write.
        \returns   The new writer.
    */
    SCXHandle<SCXPersistDataWriter> SCXFilePersistMedia::CreateWriter(const std::wstring& name, unsigned int version /* = 0*/)
    {
        try
        {
            return SCXHandle<SCXPersistDataWriter>(
                new SCXFilePersistDataWriter(NameToFilePath(name), version) );
        }
        catch (const SCXFilePathNotFoundException& e1)
        {
            throw PersistMediaNotAvailable(e1.What(), SCXSRCLOCATION);
        }
        catch (const SCXUnauthorizedFileSystemAccessException& e2)
        {
            throw PersistDataNotFoundException(e2.What(), SCXSRCLOCATION);
        }
    }

    /*----------------------------------------------------------------------------*/
    /**
        Remove persisted data with the given name.

        \param[in]  name     Name of data to remove.
        \throws  PersistDataNotFoundException if no data previously written with the given name.
    */
    void SCXFilePersistMedia::UnPersist(const std::wstring& name)
    {
        SCXFileInfo f(NameToFilePath(name));
        if ( ! f.Exists())
        {
            throw PersistDataNotFoundException(name, SCXSRCLOCATION);
        }
        f.Delete();
    }

    /*----------------------------------------------------------------------------*/
    /**
        Get the SCXPersistMedia implementation.
    
        \returns     SCXFilePersistMedia handle
    */
    SCXHandle<SCXPersistMedia> GetPersistMedia()
    {
        return SCXHandle<SCXPersistMedia>(new SCXFilePersistMedia());
    }

    /*----------------------------------------------------------------------------*/
    /**
        Translate a persistence name into a complete file path.
    
        \param[in]   name Persistence name to translate.
        \returns     SCXFilePath representing the name supplied.
    */
    SCXFilePath SCXFilePersistMedia::NameToFilePath(const std::wstring& name) const
    {
        SCXFilePath p(m_BasePath);
        // Replace all '_' with "__"
        std::wstring newname = name;
        for (std::wstring::size_type pos = newname.find(L'_');
             pos != std::wstring::npos;
             pos = newname.find(L'_', pos+2))
        {
            newname.insert(pos, 1, L'_');
        }
        
        // Replace all '/' with "_s"
        for (std::wstring::size_type pos = newname.find(L'/');
             pos != std::wstring::npos;
             pos = newname.find(L'/', pos+2))
        {
            newname[pos] = L's';
            newname.insert(pos, 1, L'_');
        }

        p.Append(newname);
        return p;
    }

    /*----------------------------------------------------------------------------*/
    /**
        This method is currently primarily for testing purposes.
    
        \param[in]   path New base path to use for storing files.
    */
    void SCXFilePersistMedia::SetBasePath(const SCXFilePath& path)
    {
        m_BasePath = path;
    }
}

/*----------------------------E-N-D---O-F---F-I-L-E---------------------------*/
