/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
    \file
 
    \brief     Implementation regarding the SCXException base class
 
    \date      07-05-24 14:30:18
 
    < Optional detailed description of file purpose >  
*/
/*----------------------------------------------------------------------------*/

#include <scxcorelib/scxcmn.h>
#include <scxcorelib/scxexception.h>
#include <scxcorelib/stringaid.h>

using namespace std;

namespace SCXCoreLib {


    /******************************************************************************
     *  
     *   SCXCodeLocation implementation
     *  
     ******************************************************************************/

    /*----------------------------------------------------------------------------*/
    /**
       Returns a formatted string with where an exception occured
    
    */
    std::wstring SCXCodeLocation::Where() const 
    {
        if (m_File.length())
        {
            // Compose e.g. [my_file.cpp:434]
            return L"[" + m_File + L":" + StrFrom(m_Line) + L"]"; 
        }
        else
        {
            return L"[unknown]";
        }
    }
    
    
    /******************************************************************************
     *  
     *  SCXException Implementation 
     *  
     ******************************************************************************/
    
    /*----------------------------------------------------------------------------*/
    /**
       Returns a formatted string with details on an exception.
       
       Normally this does not need to be overridden in subclasses.
    
    */
    wstring SCXException::Where() const 
    { 
        if (m_StackContext.length())
        {
            return m_StackContext + L", thrown from " + m_OriginatingLocation.Where(); 
        }
        else
        {
            return m_OriginatingLocation.Where();
        }
    }
    
    
    /*----------------------------------------------------------------------------*/
    /**
       Add relevant stack context to an exception

       When a function catches an exception and pass it on, this method can be 
       used to add any information relevant to the end user.
       
    */
    void SCXException::AddStackContext(const wstring& context, const SCXCodeLocation& location) 
    {
        wstring thisContext = context;        // The context to add composed here

        if (location.GotInfo())
        {
            // Got line/position info, concatinate
            thisContext += location.Where();
        }

        // Add to existing context in the exception
        if (m_StackContext.length())
        {
            m_StackContext = thisContext + wstring(L"->") + m_StackContext;
        }
        else
        {
            m_StackContext = thisContext;
        }
    }

    void SCXException::AddStackContext(const wstring& context) 
    {
        AddStackContext(context, SCXCodeLocation());
    }

    void SCXException::AddStackContext(const SCXCodeLocation& location) 
    {
        AddStackContext(L"", location);
    }


    /******************************************************************************
     *  
     *  SCXInvalidArgumentException Implementation 
     *  
     ******************************************************************************/
    
    /*----------------------------------------------------------------------------*/
    /* (overload)
       Format reason why the argument was invalid
       
    */
    wstring SCXInvalidArgumentException::What() const 
    { 
        // Example
        // Formal argument 'myArgument' is invalid: Syntax error
        return L"Formal argument '" + m_FormalArg + L"' is invalid: " + m_Reason;
    }


    /******************************************************************************
     *  
     *  SCXNULLPointerException Implementation 
     *  
     ******************************************************************************/
    
    /*----------------------------------------------------------------------------*/
    /* (overload)
       Format which pointer was NULL
       
    */
    wstring SCXNULLPointerException::What() const 
    { 
        // Example:
        // A NULL pointer was supplied in argument 'myPointer'
        return L"A NULL pointer was supplied in argument '" + m_PointerName + L"'";
    }


    /******************************************************************************
     *  
     *  SCXIllegalIndexException Implementation 
     *  
     ******************************************************************************/
    
        

    /******************************************************************************
     *  
     *  SCXNotSupportedException Implementation 
     *  
     ******************************************************************************/
    
    /*----------------------------------------------------------------------------*/
    /* (overload)
       Format details of violation
       
    */
    wstring SCXNotSupportedException::What() const 
    { 
        // Example
        // Enumeration of dead birds not supported
        return m_Functionality + L" not supported";
    }


    /******************************************************************************
     *  
     *  SCXInternalErrorException Implementation 
     *  
     ******************************************************************************/
    
    /*----------------------------------------------------------------------------*/
    /* (overload)
       Format details of violation
       
    */
    wstring SCXInternalErrorException::What() const 
    { 
        // Example
        // Internal Error: expected element not found in list
        return L"Internal Error: " + m_Reason;
    }


    /******************************************************************************
     *  
     *  SCXResourceExhaustedException Implementation 
     *  
     ******************************************************************************/
    
    /*----------------------------------------------------------------------------*/
    /* (overload)
       Format details of violation
       
    */
    wstring SCXResourceExhaustedException::What() const 
    { 
        // Example
        // Failed to allocate resource of type process: too many pids in system
        return L"Failed to allocate resource of type " + m_ResourceType + L": " + m_ResourceDetails;
    }

    /******************************************************************************
     *  
     *  SCXInvalidStateException Implementation 
     *  
     ******************************************************************************/
    
    /*----------------------------------------------------------------------------*/
    /* (overload)
       Format reason why the state was invalid
       
    */
    wstring SCXInvalidStateException::What() const 
    { 
        // Example
        // Invalid state: Can not call method DoOtherStuff before DoStuff is called.
        return L"Invalid state: " + m_Reason;
    }

}

/*----------------------------E-N-D---O-F---F-I-L-E---------------------------*/
