/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
    \file        

    \brief       Implements a simple pattern finder.
    
    \date        2008-01-29 11:17:00

*/
/*----------------------------------------------------------------------------*/

#include <scxcorelib/scxcmn.h>
#include <scxcorelib/stringaid.h>
#include <scxcorelib/scxpatternfinder.h>

namespace SCXCoreLib
{
/*----------------------------------------------------------------------------*/
/**
    Default constructor.
*/
    SCXPatternFinder::SCXPatternFinder()
    {
        m_separators = L" \n\t=<>";
        m_mergeMarkers[L"\""] = L"\"";
        m_mergeMarkers[L"'"] = L"'";
        m_parameterIdentifier = L"%";
    }
     
/*----------------------------------------------------------------------------*/
/**
    Virtual destructor
*/
    SCXPatternFinder::~SCXPatternFinder()
    {

    }

/*----------------------------------------------------------------------------*/
/**
    Register a new pattern.
    
    \param       cookie A caller defined cookie to identify the pattern. This
                        cookie is returned when the pattern is matched.
    \param       pattern The pattern string.
    \throws      SCXInternalErrorException if the cookie has already been used or 
                                           if the pattern cannot be tokenized.
    
    The default patterns use white-space and the three chars =, < and > to seperate 
    tokens. " or ' might be used to create token strings. Parameters starts with %.
    Example: Select * from something where value=%parameter
    
*/
    void SCXPatternFinder::RegisterPattern(const SCXPatternCookie& cookie, const std::wstring& pattern)
    {
        if (m_patterns.find(cookie) != m_patterns.end())
        {
            throw SCXCoreLib::SCXInternalErrorException(L"Cookie already exists", SCXSRCLOCATION);
        }
        std::vector<std::wstring> tokens;
        SCXCoreLib::StrTokenize(pattern, tokens, m_separators, false, true, true);
        
        if ( ! SCXCoreLib::StrMergeTokens(tokens, m_mergeMarkers, L""))
        {
            throw SCXCoreLib::SCXInternalErrorException(L"Cannot tokenize pattern: " + pattern, SCXSRCLOCATION);
        }
        m_patterns[cookie] = tokens;
    }

/*----------------------------------------------------------------------------*/
/**
    Attempts to match the given string against registred patterns.
    
    \param       s The string we try to match agains registered patterns.
    \param[out]  cookie The cookie used when registering a matched pattern.
    \param[out]  matches Any parameters matched are returned as string-string
                         value pairs. \note this parameter is not reset by this method.
    \returns     true if a matching pattern was found, otherwise false.
    
    If there is a registred pattern looking like this:
    Select * from something where value=%parameter
    testing for a match with 
    SELECT * FROM something WHERE value=v
    will result in a successful match since matching is case insensitive.
    The matches parameter will hold a "parameter" -> "v" pair.
    
*/
    bool SCXPatternFinder::Match(const std::wstring& s, SCXPatternCookie& cookie, SCXPatternMatch& matches) const
    {
        std::vector<std::wstring> tokens;
        SCXCoreLib::StrTokenize(s, tokens, m_separators, false, true, true);
        if ( ! SCXCoreLib::StrMergeTokens(tokens, m_mergeMarkers, L""))
        {
            return false;
        }
        for (std::map<SCXPatternCookie, std::vector<std::wstring> >::const_iterator pattern = m_patterns.begin();
             pattern != m_patterns.end(); pattern++)
        {
            if (tokens.size() == pattern->second.size())
            {
                bool ok = true;
                for (std::vector<std::wstring>::const_iterator t = tokens.begin(),
                         p = pattern->second.begin();
                     t != tokens.end() && p != pattern->second.end();
                     t++, p++)
                {
                    if (p->substr(0,m_parameterIdentifier.length()) == m_parameterIdentifier)
                    { // Found parameter
                        matches[p->substr(m_parameterIdentifier.length())] = *t;
                    }
                    else if (0 != SCXCoreLib::StrCompare(*t, *p, true))
                    {
                        ok = false;
                    }
                }
                if (ok)
                {
                    cookie = pattern->first;
                    return true;
                }
            }
        }
        return false;
    }

} /*namespace SCXCoreLib */
/*----------------------------E-N-D---O-F---F-I-L-E---------------------------*/
