/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
    \file        

    \brief       Common include for all SCX files.

    \date        2007-05-16 16:26:00

    This file should always be included before
    any other includes in implementation files.

*/
/*----------------------------------------------------------------------*/
#ifndef SCXCMN_H
#define SCXCMN_H

#include <time.h>

#if defined(SCX_OPTIONS)
# include "options.h"
#else
# define SCX_PREFIX "/opt/microsoft/scx"
# define SCX_LIBDIR "/opt/microsoft/scx/lib"
# define SCX_BINDIR "/opt/microsoft/scx/bin"
# define SCX_CONFDIR "/etc/opt/microsoft/scx/conf"
# define SCX_LOGDIR "/var/opt/microsoft/scx/log"
# define SCX_STATEDIR "/var/opt/microsoft/scx/lib/state"
# define SCX_RUNDIR "/var/opt/microsoft/scx/tmp"
#endif

// Define convenience macro for all UNIX/Linux platforms
// The intent here is to use this for things common across all Linux platforms
// (i.e. <unistd.h> exists on all UNIX/Linux platforms, \n line endings, etc)
#if defined(WIN32)
    // WIN32, most definitely, is not UNIX or Linux-like
#elif defined(aix) || defined(hpux) || defined(linux) || defined(macos) || defined(sun)
#define SCX_UNIX 1
/* Including stdlib.h for convenience since it is used in a lot of places */
#include <stdlib.h>
#else
#error "Platform not supported"
#endif


/** 
    Macro which will suppress the given VisualStudio warning with number n. Scope
    is next statement only. txt should be justification to suppress the warning.
 */
#if defined(WIN32)
#define PREFAST_SUPPRESS(n,txt) __pragma(warning(suppress:n))
#else
#define PREFAST_SUPPRESS(n,txt)
#endif

// Pick up definitions for int64_t and u_int_64_t (or uint64_t)
#if defined(linux) || defined(sun) || defined(hpux) || defined(aix)
#include <sys/types.h>    // int64_t & u_int64_t
#elif defined(macos)
#include <stdint.h>       // int64_t & uint64_t
#endif

#if defined(WIN32)
 
/** scxlong and scxulong types to handle long integers consistent. */
typedef __int64 scxlong;
/** scxlong and scxulong should be 64 bit on all platforms. */
typedef unsigned __int64 scxulong;

#if defined(_DEBUG)
// memory debugging stuff
#include <crtdbg.h>
// all these files have to be included before we play trick with "new" operator below 
// since it may not work with the way they use "new" internaly;
// in case you added new include and get strange compliation error please add this include to this list
#include <string>
#include <fstream>
#include <vector>
#include <xtree>
#include <xlocmon>

#define DEBUG_NEW new(_NORMAL_BLOCK, __FILE__, __LINE__)
#define new DEBUG_NEW
 
#endif

#elif defined(linux)

/** scxlong and scxulong types to handle long integers consistent. */
typedef int64_t scxlong;
/** scxlong and scxulong should be 64 bit on all platforms. */
typedef u_int64_t scxulong;

#elif defined(sun) || defined(hpux) || defined(aix) || defined(macos)

/** scxlong and scxulong types to handle long integers consistent. */
typedef int64_t scxlong;
/** scxlong and scxulong should be 64 bit on all platforms. */
typedef uint64_t scxulong;

#else

#error "scxlong and scxulong not defined for this platform."

#endif

/**
    \page Real numbers in Aurora

    For real numbers use:
    
    float  - for 32 bit real numebers
    double - for 64 bit real numbers

    If we encounter a platform where this is not true, we need to define our own 32 & 64 bit real types

*/

/* Include scxassert.h for convenience since it is used everywhere. */
#include <scxcorelib/scxassert.h>

/* Include scxcompat.h so it can be used without special requirements */
#include <scxcorelib/scxcompat.h>

#if !defined(sun) && !defined(aix) && !defined(PF_DISTRO_SUSE) && !defined(PF_DISTRO_REDHAT)

// this macro disables dynamic_cast, since t causes problems on aix 5.3 - see wi 11149, 11220
#define dynamic_cast sorry_but_dynamic_cast_is_not_allowed_in_core_project_see_wi_11220
#endif

/*
 * This macro prepends the 'L' prefix to string literals. For example,
 * if we define this:
 *
 *     #define X "Hello"
 *
 * Then the following:
 *
 *     SCX_L(X) 
 *
 * expands to:
 *
 *     L"Hello"
 *
 * This eliminates the need for two sets of macros (one the prefix and one without).
 */
#define __SCX_L(X,Y) X##Y
#define SCX_L(X) __SCX_L(L,X)

#endif /* SCXCMN_H */
