/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
    \file        scxdirectoryinfo.h

    \brief       Defines the public interface for the directory PAL.

    \date        07-08-28 14:31:57

*/
/*----------------------------------------------------------------------------*/
#ifndef SCXDIRECTORYINFO_H
#define SCXDIRECTORYINFO_H

#include <scxcorelib/scxfilepath.h>
#include <scxcorelib/scxfilesystem.h>
#include <scxcorelib/scxfile.h>
#include <scxcorelib/scxhandle.h>
#include <vector>

// On windows CreateDirectory is defined to CreateDirectoryW.
#if defined(WIN32)
#undef CreateDirectory
#endif
         
namespace SCXCoreLib
{
    /*----------------------------------------------------------------------------*/
    /**
       Represents different kinds of search options for the directory search methods.
    */
    enum SCXDirectorySearchOption
    {
        eDirSearchOptionNone = 0x000,
        eDirSearchOptionDir  = 0x001,
        eDirSearchOptionFile = 0x002,
        eDirSearchOptionSys  = 0x004,
        eDirSearchOptionAll  = eDirSearchOptionDir|eDirSearchOptionFile|eDirSearchOptionSys
    }; 

    /*----------------------------------------------------------------------------*/
    /**
       Represents a set of search options for the directory search methods.
    */
    typedef scxulong SCXDirectorySearchOptions;

    /*----------------------------------------------------------------------------*/
    /**
        Represents a directory in the filesystem. It exposes metods for listing
        the contents thereof and is the place to place additional methods for
        directory manipulation, should the need arise.
        The directory is represented by its file path through the SCXFilePath
        class. This means that an instance don't need to correspond to an
        existing directory in the file system, but rather more a potential directory.

        This class is in part modelled on the DirectoryInfo class of Microsoft .NET.

        \internal
        This class will inherit from SCXFileSystemInfo when that class is finished.
    */

    class SCXDirectoryInfo : public SCXFileSystemInfo {

        private:
        /** The default constructor is not allowed. */
        SCXDirectoryInfo();             // Intentionally not implemented (never used)

        public:
        /**
            Initializes a new instance of the SCXDirectoryInfo class on the specified path.
            \param dirname Pathname of directory
            \param pStat A stat struct to use when initializing. If zero (default) a new one will be used.

            This constructor does not check if the directory exists.
        */
        SCXDirectoryInfo(const SCXFilePath& dirname, SCXCoreLib::SCXFileSystem::SCXStatStruct* pStat = 0) :
            SCXFileSystemInfo(SCXFileSystem::CreateFullPath(dirname), dirname, pStat)
    {};

        void Refresh();
        void Delete();
        void Delete(bool recursive);

        std::vector<SCXCoreLib::SCXHandle<SCXCoreLib::SCXFileSystemInfo> > GetFileSystemInfos(const SCXDirectorySearchOptions options = SCXCoreLib::eDirSearchOptionAll);
        std::vector<SCXCoreLib::SCXHandle<SCXCoreLib::SCXFileInfo> > GetFiles();
        std::vector<SCXCoreLib::SCXHandle<SCXCoreLib::SCXFileInfo> > GetSysFiles();
        std::vector<SCXCoreLib::SCXHandle<SCXCoreLib::SCXDirectoryInfo> > GetDirectories();
    };


    /*----------------------------------------------------------------------------*/
    /**
     * Utility methods for managing directories. Useful when a single operation
     * is to be performed on a directory
     */
    class SCXDirectory {
    public:
        static void Delete(const SCXFilePath &path, bool recursive = false);
        static void Move(const SCXFilePath& oldPath, const SCXFilePath &newPath);
        static bool Exists(const SCXFilePath&);
        static SCXDirectoryInfo CreateDirectory(const SCXFilePath& path);
#if !defined(DISABLE_WIN_UNSUPPORTED)        
        static SCXDirectoryInfo CreateTempDirectory();
#endif

        static std::vector<SCXCoreLib::SCXFilePath> GetFileSystemEntries(const SCXFilePath &path, const SCXDirectorySearchOptions options = SCXCoreLib::eDirSearchOptionAll);
        static std::vector<SCXCoreLib::SCXFilePath> GetFiles(const SCXFilePath &path);
        static std::vector<SCXCoreLib::SCXFilePath> GetDirectories(const SCXFilePath &path);
        static std::vector<SCXCoreLib::SCXFilePath> GetSysFiles(const SCXFilePath &path);
    private:
        SCXDirectory(); //!< Utility class, prevent instances
    };

}
#endif /* SCXDIRECTORYINFO_H */
/*----------------------------E-N-D---O-F---F-I-L-E---------------------------*/
