/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
    \file        

    \brief       Provide a reference counted pointer handle. 
    
    \date        07-08-06 09:51:00
    
*/
/*----------------------------------------------------------------------------*/
#ifndef SCXHANDLE_H
#define SCXHANDLE_H

#include <scxcorelib/scxcmn.h>
#include <memory>
#include <scxcorelib/scxatomic.h> 

namespace SCXCoreLib
{
    /*----------------------------------------------------------------------------*/
    /**
        Provide a reference counted pointer handle.
    
        Objects being reference counted MUST be allocated using new operator and
        may not be an array. Template class will deallocate object using delete
        once the reference counter reaches zero.

        \note This implementation is \b not thread safe.
    */

    template<class T>
    class SCXHandle
    {
    private:
        T* m_pData;          //!< The actual data.
        scx_atomic_t* m_pCounter; //!< The reference counter for the data.
        bool m_isOwner;      //!< Indicates if the SCXHandle instance is owner of data.

        /*----------------------------------------------------------------------------*/
        /**
            Helper method used to increment the reference counter.
    
        */
        void AddRef()
        {
            SCXASSERT(0 != m_pCounter);
            PREFAST_SUPPRESS(6011,'m_pCounter != NULL by design and verified above')
            SCXASSERT((*m_pCounter) > 0);
            scx_atomic_increment(m_pCounter);
        }

        /*----------------------------------------------------------------------------*/
        /**
            Helper method to decrease the reference counter.
    
            Will deallocate object reference counted if the counter reaches zero.
    
        */
        void Release()
        {
            SCXASSERT(0 != m_pCounter);
            PREFAST_SUPPRESS(6011,'m_pCounter != NULL by design and verified above')
            SCXASSERT((*m_pCounter) > 0);
            SCXASSERT(( ! m_isOwner) || (1 == (*m_pCounter)));
            if ( scx_atomic_decrement_test(m_pCounter) )
            {
                if (0 != m_pData)
                    delete m_pData;
                delete m_pCounter;
            }
        }


    public:
        /*----------------------------------------------------------------------------*/
        /**
            Default construcor, so you can create array of handles, 
            use them as members and use map [] operator
    
        */
        SCXHandle() 
            : m_pData(0)
            , m_pCounter(new scx_atomic_t(1))
            , m_isOwner(false)
        {
            
        }

        /*----------------------------------------------------------------------------*/
        /**
            Construcor to start reference counting an object. 
    
            \param       p - object to reference count.
    
        */
        explicit SCXHandle(T* p) 
            : m_pData(p)
            , m_pCounter(new scx_atomic_t(1))
            , m_isOwner(false)
        {
            
        }

        /*----------------------------------------------------------------------------*/
        /**
            Copy constructor.
    
            \param       h - handle to copy.
    
        */
        SCXHandle(const SCXHandle& h)
            : m_pData(h.m_pData)
            , m_pCounter(h.m_pCounter)
            , m_isOwner(false)
        {
            AddRef();
        }
 
        /*----------------------------------------------------------------------------*/
        /**
            Constructor
    
            \param    pData       Object to reference count.
            \param    pCounter    Reference counter 
        */
        SCXHandle(T* pData, scx_atomic_t* pCounter)
            : m_pData(pData)
            , m_pCounter(pCounter)
            , m_isOwner(false)
        {
            AddRef();
        }

        /*----------------------------------------------------------------------------*/
        /**
         * Allow the same kind of implicit type conversion that is allowed for pointers.
         * A handle to a subclass may be assigned to a handle to a superclass, in the
         * same way that a pointer to a subclass may be assigned to a pointer to superclass
         */
        template<class T2> operator SCXHandle<T2>()
        {
            return SCXHandle<T2>(m_pData, m_pCounter); 
        }

        /*----------------------------------------------------------------------------*/
        /**
         * Allow the same kind of implicit type conversion that is allowed for pointers.
         * A handle to a subclass may be assigned to a handle to a superclass, in the
         * same way that a pointer to a subclass may be assigned to a pointer to superclass
         */
        template<class T2> operator SCXHandle<T2>() const
        {
            return SCXHandle<T2>(m_pData, m_pCounter); 
        }

        /*----------------------------------------------------------------------------*/
        /**
            Virtual destructor.
    
        */
        virtual ~SCXHandle()
        {
            Release();
        }

        /*----------------------------------------------------------------------------*/
        /**
         * Dereferencing operator
         * \returns Reference to reference counted object
         */
        T& operator*() const {
            return *m_pData;
        }       
        
        /*----------------------------------------------------------------------------*/
        /**
            Member access operator to access reference counted object members.
    
            \returns     Pointer to reference counted object.
    
        */
        T* operator->() const
        {
            return m_pData;
        }

        /*----------------------------------------------------------------------------*/
        /**
            Comparison with pointer.
    
            \returns     true if pointer is equal to m_pData.
    
        */
        bool operator == (const T* p ) const {
            return m_pData == p;
        }

        /*----------------------------------------------------------------------------*/
        /**
            Comparison with pointer.
    
            \returns     true if pointer is not equal to m_pData.
    
        */
        bool operator != (const T* p ) const {
            return m_pData != p;
        }

        /*----------------------------------------------------------------------------*/
        /**
            Comparison with other handle
    
            \returns     true if handles point to the same object.
    
        */
        bool operator == (const SCXHandle& other) const {
            return m_pData == other.m_pData;
        }

        /*----------------------------------------------------------------------------*/
        /**
            Comparison with other handle
    
            \returns     false if handles point to the same object.
    
        */
        bool operator != (const SCXHandle& other) const {
            return m_pData != other.m_pData;
        }

        /*----------------------------------------------------------------------------*/
        /**
            Assignment operator.
    
            \param       other - object to replace reference with.
            \returns     reference to this.
    
            Replaces the currently reference counted object with that of the argument.
    
        */
        SCXHandle& operator=(const SCXHandle& other)
        {
            if (m_pCounter == other.m_pCounter)
            { // Using counter to determine if handle is same object since data may be NULL.
                return *this;
            }
            Release();
            m_pData = other.m_pData;
            m_pCounter = other.m_pCounter;
            m_isOwner = false;
            AddRef();
            return *this;
        }

        /*----------------------------------------------------------------------------*/
        /**
            Assignment operator.
    
            \param       p - pointer to the new object
            \returns     reference to this.
    
            Replaces the currently reference counted object with that of the argument.
    
        */
        SCXHandle& operator=(T* p)
        {
            SetData( p );
            return *this;
        }
        
        /*----------------------------------------------------------------------------*/
        /**
            Method to retrieve the object pointer.
    
            \returns     Pointer to reference counted object.
    
        */
        T* GetData() const
        {
            return m_pData;
        }

        /*----------------------------------------------------------------------------*/
        /**
            Replace the reference counted pointer with a new pointer.
    
            \param       p - new pointer to reference count.
    
        */
        void SetData(T* p)
        {
            if (p == m_pData)
            {
                return;
            }

            m_isOwner = false;
             
            if (scx_atomic_decrement_test(m_pCounter))
            {
                if (0 != m_pData)
                {
                    delete m_pData;
                }

                m_pData = p;
                *m_pCounter = 1;
                return;
            }

            m_pData = p;
            m_pCounter = new scx_atomic_t(1);
        }
    
        /*----------------------------------------------------------------------------*/
        /**
            Set ownership of data.

            This method is used to set a certain instance of SCXHandle as owner of the
            data. The purpose is to add the possibility to assert if ref counter is not
            zero when the owning SCXHandle goes out of scope. makes it easier to catch
            the scenario when a reference is kept at an unknown location.

        */
        void SetOwner()
        {
            m_isOwner = true;
        }
    }; /* SCXHandle */

} /* namespace SCXCoreLib */


/*----------------------------------------------------------------------------*/
/**
    Equality operator for ordinary pointer and SCXHandle
*/
template<class T> inline bool operator == ( const T* p, const SCXCoreLib::SCXHandle<T>& ptr ){
    return p == ptr.GetData();
}


/*----------------------------------------------------------------------------*/
/**
   Equality operator for NULL pointer and handle
*/
template<class T> inline bool operator == ( void* p, const SCXCoreLib::SCXHandle<T>& ptr ){
    // NULL comparison
    SCXASSERT( p == 0 );
    p = p;  // to disable warning
    return 0 == ptr.GetData();
}

/*----------------------------------------------------------------------------*/
/**
    Inequality operator for ordinary pointer and SCXHandle
*/
template<class T> inline bool operator != ( const T* p, const SCXCoreLib::SCXHandle<T>& ptr ){
    return p != ptr.GetData();
}

/*----------------------------------------------------------------------------*/
/**
   Inequality operator for NULL pointer and handle
*/
template<class T> inline bool operator != ( void* p, const SCXCoreLib::SCXHandle<T>& ptr ){
    // NULL comparison
    SCXASSERT( p == 0 );
    p = p;  // to disable warning
    return 0 != ptr.GetData();
}


#endif /* SCXHANDLE_H */
/*----------------------------E-N-D---O-F---F-I-L-E---------------------------*/
