/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
    \file        

    \brief       Contains the public interface of the Log policy.

    \date        2008-08-05 14:08:54

*/
/*----------------------------------------------------------------------------*/
#ifndef SCXLOGPOLICY_H
#define SCXLOGPOLICY_H

#include <scxcorelib/scxlog.h>
#include <scxcorelib/scxfilepath.h>
#if defined(SCX_UNIX)
#include <scxcorelib/scxuser.h>
#endif

namespace SCXCoreLib
{
    /*----------------------------------------------------------------------------*/
    /**
        Defines the interface for the log policy.
        This is also the default implementation of the log policy. If another
        policy is needed, then inherit from this policy overriding any methods
        that need to be changed.
    */
    class SCXLogPolicy
    {
    public:
        /**
            Virtual destructor.
        */
        virtual ~SCXLogPolicy() {}

        /**
            Get the path of the log config file.
            \returns the path of the log config file.
        */
        virtual SCXFilePath GetConfigFileName() const
        {
#if defined(WIN32)
            return SCXFilePath(L"C:\\scxlog.conf");
#elif defined(SCX_UNIX)
            return SCXFilePath(SCX_L(SCX_CONFDIR) L"/scxlog.conf");
#endif
        }

        /**
            If no config is specified, then log output will be written
            to the file specified by this method.
            \returns Path to the default log file.
        */
        virtual SCXFilePath GetDefaultLogFileName() const
        {
#if defined(WIN32)
            return SCXFilePath(L"C:\\scx.log");
#elif defined(SCX_UNIX)
            SCXUser user;
            SCXFilePath filepath(SCX_L(SCX_LOGDIR) L"/scx.log");
            if (!user.IsRoot())
            {
                filepath.AppendDirectory(user.GetName());
            }
            return filepath;
#endif
        }

        /**
            Get the default severity threshold.
            \returns Default severity threshold.
        */
        virtual SCXLogSeverity GetDefaultSeverityThreshold() const
        {
            return eInfo;
        }
    };
}

/**
    This is the method that the core library logging framework recieves its
    implementation of the log policy.

    If you are happy with the default log policy, then a default implementation
    of the log policy factory is provided in the header file
    scxdefaultlogpolicyfactory.h. You need to include that file in one of your
    source files. (As is done in logpolicy.cpp in the metaprovider directory
    for the SCXCoreLib dynamic library for example)
    
    If, on the other hand, you would like to use another log policy than the
    default one, override the policy object above and provide your own
    implementation of the CustomLogPolicyFactory function.
    An example of this can be found in the testrunner.
*/
SCXCoreLib::SCXHandle<SCXCoreLib::SCXLogPolicy> CustomLogPolicyFactory();

#endif /* SCXLOGPOLICY_H */
/*----------------------------E-N-D---O-F---F-I-L-E---------------------------*/
