/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
   /file
   
   /brief      Provides information about the host.
   
   /date       1-17-2008
   
   Provides information about a machine.
   
*/

#if ! defined(SCXMACHINEINFO_H)
#define SCXMACHINEINFO_H

#include <string>


// prototype to avoid including <sys/utsname>
struct utsname;

/*
 * DNS may or may not be used for lookup logic (depending on if we trust the DNS servers)
 *
 * We've gone back and fourth on this issue, and ultimately this will be decided by what our
 * customers want. Today, we don't know what that is.
 *
 * #define SCXMACHINEINFO_USE_DNS_LOOKUPS 1
 *
 * will control this. If defined, DNS lookups will be used. If undefined, they will never be
 * used.
 */

#define SCXMACHINEINFO_USE_DNS_LOOKUPS 1


namespace SCXCoreLib 
{
    //! Data source
    enum eMachineInfoSource {
        eNone = 0,        //!< Source Unknown
        eGethostname,     //!< Source gethostname()
        eEtcHosts,        //!< Source from /etc/hosts
        eEtcResolvConf,   //!< Source from /etc/resolv.conf
        eUname,           //!< Source from uname()
        eDns,             //!< Source from DNS: gethostbyname()
        eSourceMax        //!< Value past the end
    };


    /*------------------------------------------------------------------------*/
    /**
       This class implements methods to get the domain name and hostname.

       This is an internal class, and should not be called directly.
       Instead, use the "MachineInfo" class defined below.

       Note that the MachineInfoInternal class can be somewhat lengthy to run
       (reading several files and performing DNS lookup operations).  As a
       result, caching is relevant for speed, and is handled automatically by
       the MachineInfo public interface.
    */
    class MachineInfoInternal {
    protected:
        eMachineInfoSource m_hostnameSource;   //!< Source location for the hostname
        std::wstring m_hostname;               //!< hostname
        eMachineInfoSource m_domainnameSource; //!< Source location for the domain name
        std::wstring m_domainname;             //!< domain name
        int m_nativeBitSize;                   //!< Native bit size on machine
#if defined(aix)
        bool m_isInWPAR;                       //!< Are we in a Workload Partition?
#endif

        eMachineInfoSource m_resolveDomain;    //!< Resolve domain name first via 'files' or 'dns'
        bool m_fResolveBoth;                   //!< Resolve using both 'files' AND 'dns'?

    public:
        MachineInfoInternal();
        ~MachineInfoInternal();
        void Update();

        // Methods to get the host and domain names.
        std::wstring GetHostname() const;
        std::wstring GetDomainname() const;
        std::wstring GetHostDomainname() const;
        eMachineInfoSource GetHostnameSource() const;
        eMachineInfoSource GetDomainnameSource() const;
        int GetNativeBitSize() const;
#if defined(aix)
        bool isInWPAR() const;
#endif

        // Debug Methods
        std::wstring DumpString() const;
        std::wstring DumpSourceString(eMachineInfoSource e) const;

    protected:
        // Methods to set the host and domain names.
        void GetHostVia_uname();
        void GetHostVia_gethostname();
        void GetHostVia_gethostbyname(const std::wstring & name);
        void GetHostVia_EtcHosts(const std::string & name);
        void GetHostVia_ResolvConf(const std::string & name);
        void ParseFile_nsswitchConf(const std::string & name);
        int DetermineNativeBitSize();

        // Utility methods
        void Normalize();

    private:
        // Do not allow copying
        MachineInfoInternal(const MachineInfoInternal &);             //!< Intentionally not implemented.
        MachineInfoInternal & operator=(const MachineInfoInternal &); //!< Intentionally not implemented.

        // Utility methods
        size_t GetNameBufSize() const;
        std::wstring GetHostByName(const std::wstring & name) const;

    public:
        friend class SCXMachineInfoTest;
    };// end class MachineInfoInternal


    /*------------------------------------------------------------------------*/
    /**
       This class implements methods to get the domain name and hostname, as
       well as other information (32-bit/64-bit, etc).

       To use this class, simply instanciate it and look up what you need.
       For example:

       MachineInfo mi;
       wstring hostname = mi.GetHostname();
       wstring domainname = mi.GetDomainname();

       Note that if you want both the hostname and domain name, you should
       use the class for that:

       wstring fqdn_hostname = mi.GetHostDomainname();


       This information is cached.  Users should not be concerned of any
       performance implications with this class.
    */
    class MachineInfo {
    public:
        MachineInfo() { };
        ~MachineInfo() { };

        /*
         * Function Update() isn't currently exposed for the following reasons:
         * 1) Nobody needs it today, and
         * 2) If someone needs it, it may introduce multithreading issues
         *
         * For now, we don't implement that.  We can revisit later if needed.
         * At this time, the global MachineInfo instance is built at process startup.
         *
         *
         * void Update() { m_mi.Update(); }
         */

        // Methods to get the host and domain names.
        /** \copydoc SCXCoreLib::MachineInfoInternal::GetHostname */
        std::wstring GetHostname() const { return m_mi.GetHostname(); }
        /** \copydoc SCXCoreLib::MachineInfoInternal::GetDomainname */
        std::wstring GetDomainname() const { return m_mi.GetDomainname(); }
        /** \copydoc SCXCoreLib::MachineInfoInternal::GetHostDomainname */
        std::wstring GetHostDomainname() const { return m_mi.GetHostDomainname(); }
        /** \copydoc SCXCoreLib::MachineInfoInternal::GetHostnameSource */
        eMachineInfoSource GetHostnameSource() const { return m_mi.GetHostnameSource(); }
        /** \copydoc SCXCoreLib::MachineInfoInternal::GetDomainnameSource */
        eMachineInfoSource GetDomainnameSource() const { return m_mi.GetDomainnameSource(); }
        /** \copydoc SCXCoreLib::MachineInfoInternal::GetNativeBitSize */
        int GetNativeBitSize() const { return m_mi.GetNativeBitSize(); }
#if defined(aix)
        /** \copydoc SCXCoreLib::MachineInfoInternal::isInWPAR */
        bool isInWPAR() const { return m_mi.isInWPAR(); }
#endif

        // Debug Methods
        /** \copydoc SCXCoreLib::MachineInfoInternal::DumpString */
        std::wstring DumpString() const { return m_mi.DumpString(); }
        /** \copydoc SCXCoreLib::MachineInfoInternal::DumpSourceString */
        std::wstring DumpSourceString(eMachineInfoSource e) const { return m_mi.DumpSourceString(e); }

    private:
        static MachineInfoInternal m_mi;              //!< Static member (initialized once per run)

        // Do not allow copying
        MachineInfo(const MachineInfo &);             //!< Intentionally left unimplemented.
        MachineInfo & operator=(const MachineInfo &); //!< Intentionally left unimplemented.
    };// end class MachineInfo
}// end namespace SCXCoreLib

#endif /* SCXMACHINEINFO_H */

/*--------------------------E-N-D---O-F---F-I-L-E----------------------------*/
