/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
    \file        scxprocess.h

    \brief       Defined the process PAL.
    
    \date        2007-10-12 15:43:00
    
*/
/*----------------------------------------------------------------------------*/
#ifndef SCXPROCESS_H
#define SCXPROCESS_H

#include <scxcorelib/scxcmn.h>
#include <scxcorelib/scxexception.h>
#include <scxcorelib/scxfilepath.h>

#if defined(WIN32)
#include <windows.h>
#elif defined(linux) || defined(sun) || defined(hpux)
#include <unistd.h>
#endif
 
#include <iostream>
#include <vector>
#include <string>
#include <sys/types.h>

#if defined(sun) || defined(hpux)
#include <sys/wait.h>
#endif

namespace SCXCoreLib
{
    /** Definition of a process id. */
#if defined(WIN32)
    typedef DWORD SCXProcessId;
#elif defined(SCX_UNIX)
    typedef pid_t SCXProcessId;
#else
#error Not implemented on this platform 
#endif


    /*----------------------------------------------------------------------------*/
    /**
     * Process did not run to completion, it terminated prematurally and there was
     * not return code
     */
    class SCXInterruptedProcessException : public SCXException
    {
    public:
        //! Constructor
        //! \param[in]  location    Where the exception occurred
        SCXInterruptedProcessException(const SCXCodeLocation& location) 
           : SCXException(location)  {};
                                                         
        virtual std::wstring What() const { return L"Process interrupted"; }
    
    };

    /*----------------------------------------------------------------------------*/
    /**
        Represents a reference to a process. 
    
        Provide a reference to process instances.
    
    */
    class SCXProcess
    {
    public:
        static SCXProcessId GetCurrentProcessID();
            static std::vector<std::wstring> SplitCommand(const std::wstring &command);

#if defined(SCX_UNIX)

        static int Run(const std::wstring &command, std::istream &mystdin, std::ostream &mystdout, std::ostream &mystderr, 
                       unsigned timeout = 0, const SCXFilePath& cwd = SCXFilePath(), const SCXFilePath& chrootPath = SCXFilePath());
        static int Run(const std::vector<std::wstring> &myargv, std::istream &mystdin, std::ostream &mystdout, std::ostream &mystderr, 
                       unsigned timeout = 0, const SCXFilePath& cwd = SCXFilePath(), const SCXFilePath& chrootPath = SCXFilePath());
        
        SCXProcess(const std::vector<std::wstring> myargv, const SCXFilePath& cwd = SCXFilePath(), const SCXFilePath& chrootPath = SCXFilePath());
        void FetchInput(std::istream &mystdin);
        bool FetchOutput(std::ostream &mystdout, std::ostream &mystderr);
        int WaitForReturn();
        int WaitForReturn(std::istream &mystdin, std::ostream &mystdout, std::ostream &mystderr); 
        ~SCXProcess();
        void Kill();
        
    private:        
        //!< Index of file descriptors for a pipe
        enum Direction {
            R,           //!< in
            W            //!< out
        };

        int m_inForChild[2];                  //!< Pipe for stdin of the process
        int m_outForChild[2];                 //!< Pipe for stdput of the process
        int m_errForChild[2];                 //!< Pipe for stderr of the process
        std::vector<char *> m_cargv;          //!< Process arguments as null terminated char arrays
        std::vector<char> m_stdinChars;       //!< Buffer when reading from stdin
        std::vector<char> m_buffer;           //!< Buffer when writing to stdout and stderr
        int m_stdinCharCount;                 //!< Number of buffered stdin characters
        SCXProcessId m_pid;                          //!< Pid (process ID) of process
#endif

        };

} /* namespace SCXCoreLib */

#endif /* FILENAME_H */
/*----------------------------E-N-D---O-F---F-I-L-E---------------------------*/
