/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
    \file        

    \brief       Contains the definition of the Singleton Template.

    \date        2007-06-19 08:50:28

*/
/*----------------------------------------------------------------------------*/
#ifndef SCXSINGLETON_H
#define SCXSINGLETON_H

#include <scxcorelib/scxexception.h>
#include <scxcorelib/scxthreadlock.h>
#include <scxcorelib/scxhandle.h>

namespace SCXCoreLib
{
    /*----------------------------------------------------------------------------*/
    /**
        Provides singleton functionality.
    
        \date        <2007-06-19 08:50:28>
        
        Use this class when a singleton is needed.

        To make class A a singleton, let it inherit from SCXSingleton<A>.
        Make Singleton<A> a friend of A, and make the constructors private.

        \ex
        \code
        class A : public SCXSingleton<A>
        {
            friend class SCXSingleton<A>;
                public:
                    std::wstring DumpString() const;
        private:
            A();
        };
        \endcode
    */
    template<class T> class SCXSingleton
    {
    public:
        /*----------------------------------------------------------------------------*/
        /**
            Returns a reference to the singleton instance.
           
            If the instance is not yet created, it first creates it.
           
        */
        static T& Instance()
        {
            if (0 == s_lockHandle.GetData())
            {
                throw SCXInternalErrorException(L"Tried to get a singleton instance before static initialization was completed.", SCXSRCLOCATION);
            }

            SCXThreadLock lock(*s_lockHandle);
            
            if (0 == s_instance.GetData())
            {
                s_instance = new T();
            }
            return *s_instance;
        }

    protected:
        /** Only instantiated by subclassing. */
        SCXSingleton() {}
        /** Only instantiated by subclassing. */
        SCXSingleton(const SCXSingleton&) {}
        /** Only copied by subclassing. */
        SCXSingleton& operator=(const SCXSingleton&) { return *this; }

    private:
        static SCXCoreLib::SCXHandle<T> s_instance; //!< Handle to the singleton instance.
        static SCXCoreLib::SCXHandle< SCXThreadLockHandle > s_lockHandle; //!< Used for locking at creation time.
    };

    template<class T> SCXCoreLib::SCXHandle<T> SCXSingleton<T>::s_instance(0);
    template<class T> SCXCoreLib::SCXHandle<SCXThreadLockHandle> SCXSingleton<T>::s_lockHandle( new SCXThreadLockHandle(ThreadLockHandleGet()) );
}

#endif /* SCXSINGLETON_H */
/*----------------------------E-N-D---O-F---F-I-L-E---------------------------*/
