/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
    \file

    \brief     A collection of string functions assisting the wstring class

    \date      07-05-14 12:00:00

    String helper functions, support for working with std::wstring
*/
/*----------------------------------------------------------------------------*/
#ifndef STRINGAID_H
#define STRINGAID_H

#include <string>
#include <vector>
#include <scxcorelib/scxexception.h>
#include <map>
 

namespace SCXCoreLib
{
    
    std::wstring StrFromUTF8(const std::string& str);
    std::string StrToUTF8(const std::wstring& str);

    std::wstring DumpString(const std::exception &e);
    
    std::wstring StrFromMultibyte(const std::string& str);
    std::string StrToMultibyte(const std::wstring& str);

    std::wstring StrTrimL(const std::wstring& str);
    std::wstring StrTrimR(const std::wstring& str);
    std::wstring StrTrim(const std::wstring& str);

    std::wstring StrStripL(const std::wstring& str, const std::wstring& what);
    std::wstring StrStripR(const std::wstring& str, const std::wstring& what);
    std::wstring StrStrip(const std::wstring& str, const std::wstring& what);

    // These templates are defined below in this file. Usable for at least all integral types.
    template <typename T> std::wstring StrAppend(const std::wstring& str, T i);
    template <typename T> std::wstring StrFrom(T v);

    unsigned int StrToUInt(const std::wstring& str);
    double StrToDouble(const std::wstring& str);
    scxlong StrToLong(const std::wstring& str);
    scxulong StrToULong(const std::wstring& str);

    std::wstring StrToUpper(const std::wstring& str);
    std::wstring StrToLower(const std::wstring& str);

    int StrCompare(const std::wstring& str1, const std::wstring& str2, bool ci=false);

    void StrTokenize(const std::wstring& str, std::vector<std::wstring>& tokens, const std::wstring& delimiters = L" \n", bool trim=true, bool emptyTokens=false, bool keepDelimiters = false);
    void StrTokenizeStr(const std::wstring& str, std::vector<std::wstring>& tokens, const std::wstring& delimiter, bool trim=true, bool emptyTokens=false);
    bool StrMergeTokens(std::vector<std::wstring>& tokens, const std::map<std::wstring,std::wstring>& mergePairs, const std::wstring& glue);

    bool StrIsPrefix(const std::wstring& str, const std::wstring& prefix, bool ci=false);

    /** Exception for failed multibyte conversion*/
    class SCXStringConversionException : public SCXException {
    public:
        //! Ctor
        SCXStringConversionException(const SCXCodeLocation& l)
            : SCXException(l)
        {};

        std::wstring What() const { return L"Multibyte string conversion failed"; }
    };

    /*----------------------------------------------------------------------------*/
    /**
        Append an integral datatype to a string

        \param  str    String to append to
        \param  i      Integral datatype to append
        \returns       The appended string

        \date          2007-10-02 14:02:00

        \remarks This template started its life as a number of separate functions
        that took an explicitly typed second arguments. That eventually clashed with
        how size_t was defined on separated platforms. You may think that it is ugly
        to include the definition in the header file and that we should use export
        instead, the so called separation model. Well, that's not possible, export
        is not supported by the compiler we use. What about explicit instantiation
        then? That would have some very bad consequences in shape of wild includes
        and dependencies. This is because for every type we use, there must be very
        explicit instantiation. Since we feed types from pegasus into this function
        that will be very bad.
    */
    template <typename T>
        std::wstring StrAppend(const std::wstring& str, T i)
    {
        std::wstringstream ss;
        ss << str << i;
        return ss.str();
    }

    /*----------------------------------------------------------------------------*/
    /**
        Convert an integral datatype to a string

        \param    v     Integral datatype to convert
        \returns        The integral datatype as a string

        \date           2007-10-02 14:02:00

    */
    template <typename T>
        std::wstring StrFrom(T v)
    {
        std::wstringstream ss;
        ss << v;
        return ss.str();
    }
}


#endif /* STRINGAID_H */
/*----------------------------E-N-D---O-F---F-I-L-E---------------------------*/
