/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
   \file

   \brief     Definition of SCXProviderLib::SCXProperty

   \date      07-07-24 16:18:48


*/
/*----------------------------------------------------------------------------*/
#ifndef SCXPROPERTY_H
#define SCXPROPERTY_H

#include <string>
#include <vector>
#include <scxcorelib/scxtime.h>

namespace SCXProviderLib
{
    /*----------------------------------------------------------------------------*/
    /**
       Representation of a property.

       Keep name/value pair with type for value. The class behaves like an union
       in that instances can have one data item only at any time.

    */
    class SCXProperty
    {
    public:
        //! Internal enum indicating the type of current value.
        enum SCXType
        {
            SCXStringType,   //!< Carrying wstring data
            SCXIntType,      //!< Carrying integer
            SCXUIntType,     //!< Carrying unsigned int
            SCXULongType,    //!< Carrying scxulong
            SCXBoolType,     //!< Carrying bool
            SCXTimeType,     //!< Carrying scxtime
            SCXFloatType,    //!< Carrying float
            SCXDoubleType,   //!< Carrying double
            SCXUShortType,   //!< Carrying unsigned short
            SCXUCharType,    //!< Carrying unsigned char
            SCXSShortType,   //!< Carrying signed short
            SCXArrayType     //!< Carrying array data
        };

        // Subclasses use these to set properties
        SCXProperty(std::wstring name, const std::wstring& value);
        SCXProperty(std::wstring name, const wchar_t *value);
        SCXProperty(std::wstring name, int value);
        SCXProperty(std::wstring name, signed short value);
        SCXProperty(std::wstring name, unsigned int value);
        SCXProperty(std::wstring name, unsigned short value);
        SCXProperty(std::wstring name, unsigned char value);
        SCXProperty(std::wstring name, scxulong value);
        SCXProperty(std::wstring name, bool value);
        SCXProperty(std::wstring name, const SCXCoreLib::SCXCalendarTime& value);
        SCXProperty(std::wstring name, float value);
        SCXProperty(std::wstring name, double value);
        SCXProperty(std::wstring name, const std::vector<SCXProperty>& value);
        SCXProperty();

        // The CMPI base class uses these to retrieve property information
        const std::wstring& GetName() const;
        void SetName(std::wstring name);

        const std::wstring& GetStrValue() const;
        int             GetIntValue() const;
        unsigned int    GetUIntValue() const;
        signed short    GetSShortValue() const;
        unsigned short  GetUShortValue() const;
        unsigned char   GetUCharValue() const;
        scxulong        GetULongValue() const;
        bool            GetBoolValue() const;
        const SCXCoreLib::SCXCalendarTime& GetTimeValue() const;
        float           GetFloatValue() const;
        double          GetDoubleValue() const;
        const std::vector<SCXProperty>&    GetVectorValue() const;

        // The CMPI base class uses these to set properties before handing over to
        // subclass implementation
        void SetValue(const std::wstring& value);
        void SetValue(const wchar_t *value);
        void SetValue(int value);
        void SetValue(signed short value);
        void SetValue(unsigned int value);
        void SetValue(unsigned short value);
        void SetValue(unsigned char value);
        void SetValue(scxulong value);
        void SetValue(bool value);
        void SetValue(const SCXCoreLib::SCXCalendarTime& value);
        void SetValue(float value);
        void SetValue(double value);
        void SetValue(const std::vector<SCXProperty>& value);

        SCXType GetType() const;

        bool operator==(const SCXProperty& other) const;

        // For tracing etc.
        static std::wstring DumpTypeString(SCXType t);
        std::wstring DumpString() const;

    private:
        void Init(std::wstring name, SCXType type);

    private:
        //! Property name
        std::wstring m_name;

        // Depending on type exactly one of the following is set
        // When adding a type, be sure to remember to update the Init method.
        std::wstring   m_svalue;   //!< wstring
        int            m_ivalue;   //!< int
        signed short   m_ssvalue;  //!< sshort
        unsigned int   m_uivalue;  //!< uint
        unsigned short m_usvalue;  //!< ushort
        unsigned char  m_ucvalue;  //!< uchar
        scxulong       m_ulvalue;  //!< scxulong
        bool           m_bvalue;   //!< bool
        SCXCoreLib::SCXCalendarTime m_tvalue;   //!< scxcalendartime
        float          m_fvalue;   //!< float
        double         m_dvalue;   //!< double
        std::vector<SCXProperty>   m_vvalue;   //!< array

        //! Indicates which of the data fields that have a valid data
        SCXType      m_type;
    };

}

#endif /* SCXPROPERTY_H */
/*----------------------------E-N-D---O-F---F-I-L-E---------------------------*/
