/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
   \file

   \brief       Enumeration of CPU:s

   \date        07-05-21 12:00:00

   \date        08-05-29 12:28:00
*/
/*----------------------------------------------------------------------------*/
#ifndef CPUENUMERATION_H
#define CPUENUMERATION_H

#include <vector>

#include <scxsystemlib/entityenumeration.h>
#include <scxsystemlib/cpuinstance.h>
#include <scxcorelib/scxlog.h>
#include <scxcorelib/scxthread.h>
#include <scxcorelib/scxthreadlock.h>

#if defined(sun)

#include <scxsystemlib/scxkstat.h>
#include <sys/types.h>
#include <sys/processor.h>

#elif defined(hpux)

#include <sys/mp.h>
#include <sys/pstat.h>

#elif defined(aix)

#include <sys/systemcfg.h>

#endif

namespace SCXSystemLib
{

    /** Time between each sample in seconds. */
    const int CPU_SECONDS_PER_SAMPLE = 60;

    /*----------------------------------------------------------------------------*/
    /**
       Class representing all external dependencies from the CPU PAL.

    */
    class CPUPALDependencies
    {
    public:
        virtual SCXCoreLib::SCXHandle<std::wistream> OpenStatFile() const;
        virtual long sysconf(int name) const;
#if defined(sun)
        virtual const SCXCoreLib::SCXHandle<SCXKstat> CreateKstat(const int cpuid) const;
        virtual int p_online(processorid_t processorid, int flag) const;
#elif defined(hpux)
        virtual int pstat_getprocessor(struct pst_processor *buf,
                                       size_t elemsize,
                                       size_t elemcount,
                                       int index) const;
        virtual int pstat_getdynamic(struct pst_dynamic *buf,
                                     size_t elemsize,
                                     size_t elemcount,
                                     int index) const;
#elif defined(aix)
        virtual int perfstat_cpu_total(perfstat_id_t *name,
                                       perfstat_cpu_total_t* buf,
                                       int bufsz,
                                       int number) const;

        virtual int perfstat_cpu(perfstat_id_t *name,
                                 perfstat_cpu_t* buf,
                                 int bugsz,
                                 int number) const;
#endif
        virtual ~CPUPALDependencies() {};
    };

    /*----------------------------------------------------------------------------*/
    /**
       Class that represents a colletion of CPU:s.

       PAL Holding collection of CPU:s.

    */
    class CPUEnumeration : public EntityEnumeration<CPUInstance>
    {
    public:
        explicit CPUEnumeration(SCXCoreLib::SCXHandle<CPUPALDependencies> = SCXCoreLib::SCXHandle<CPUPALDependencies>(new CPUPALDependencies()) );
        ~CPUEnumeration();
        virtual void Init();
        virtual void Update(bool updateInstances=true);
        virtual void CleanUp();
        void SampleData();

    private:
        SCXCoreLib::SCXHandle<CPUPALDependencies> m_deps; //!< Collects external dependencies of this class.
        SCXCoreLib::SCXLogHandle m_log;         //!< Log handle.
        SCXCoreLib::SCXThreadLockHandle m_lock; //!< Handles locking in the cpu enumeration.

        SCXCoreLib::SCXHandle<SCXCoreLib::SCXThread> m_dataAquisitionThread; //!< Thread pointer.
        static void DataAquisitionThreadBody(SCXCoreLib::SCXThreadParamHandle& param);
        bool IsCPUEnabled(const int cpuid);

#if defined(sun) || defined(hpux)
        /*----------------------------------------------------------------------------*/
        /**
           Utility class for gathering all relevant cpu counters for all platforms
           and exposing them through a common interface.
        */
        class CPUStatHelper
        {
            friend class CPUEnumeration;
        public:
            CPUStatHelper(unsigned int cpuid, SCXCoreLib::SCXHandle<CPUPALDependencies> deps);
            ~CPUStatHelper();
            void Update();
        protected:
            scxulong User;      //!< User ticks
            scxulong System;    //!< System ticks
            scxulong Idle;      //!< Idle ticks
            scxulong IOWait;    //!< IO Wait ticks
            scxulong Nice;      //!< Nice ticks
            scxulong Irq;       //!< IRQ Ticks
            scxulong SoftIrq;   //!< Soft IRC ticks (Dpc).
            scxulong Total;     //!< Total ticks.
        private:
            /*----------------------------------------------------------------------------*/
            /**
               Private constructor.
            */
            CPUStatHelper();
            void Init();

            SCXCoreLib::SCXLogHandle m_log;     //!< Log handle.
            unsigned int m_cpuid;               //!< The cpuid for which the counters where read.
            SCXCoreLib::SCXHandle<CPUPALDependencies> m_deps; //!< Collects external dependencies of this class.


#if defined(sun)
            SCXCoreLib::SCXHandle<SCXKstat> m_kstat;       //!< Kstat instance to read cpu counters from for Sun.
            scxulong GetValue(const std::wstring & statistic);
#elif defined(hpux)
            struct pst_processor m_pst_processor; //!< CPU data
#else
#error "Not implemented for this platform"
#endif
        };

#elif defined(aix)

        std::vector<perfstat_cpu_t> m_dataarea; //!< Holds output from perfstat_cpu()
        perfstat_cpu_total_t m_dataarea_total;  //!< Holds output from perfstat_cpu_total()
        perfstat_id_t m_cpuid;                  //!< First CPU ID in perfstat_cpu() queries

#endif
    };

}

#endif /* CPUENUMERATION_H */
/*----------------------------E-N-D---O-F---F-I-L-E---------------------------*/
