/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
   \file

   \brief       PAL representation of a CPU

   \date        07-05-21 12:00:00

   \date        08-05-28 14:43:00
*/
/*----------------------------------------------------------------------------*/
#ifndef CPUINSTANCE_H
#define CPUINSTANCE_H

#include <string>

#if defined(aix)
#include <libperfstat.h>
#endif

#include <scxsystemlib/entityinstance.h>
#include <scxsystemlib/datasampler.h>
#include <scxcorelib/scxlog.h>

namespace SCXSystemLib
{
    /** Number of samples collected in the datasampler for CPU. */
    const int MAX_CPUINSTANCE_DATASAMPER_SAMPLES = 6;

    /** Datasampler for CPU information. */
#if defined(aix)
    typedef DataSampler<u_longlong_t, MAX_CPUINSTANCE_DATASAMPER_SAMPLES> CPUInstanceDataSampler;
#else
    typedef DataSampler<scxulong, MAX_CPUINSTANCE_DATASAMPER_SAMPLES> CPUInstanceDataSampler;
#endif
    /*----------------------------------------------------------------------------*/
    /**
       Class that represents a colletion of instances.

       Concrete implementation of an instance of a CPU

    */
    class CPUInstance : public EntityInstance
    {
        friend class CPUEnumeration;

    public:

        CPUInstance(unsigned int procNumber, bool isTotal = false);
        virtual ~CPUInstance();

        const std::wstring& GetProcName() const;
        unsigned int GetProcNumber() const;

        virtual void Update();

        // Return values indicate whether the implementation for this platform
        // supports the value or not.
        bool GetProcessorTime(scxulong& processorTime) const ;
        bool GetIdleTime(scxulong& idleTime) const;
        bool GetUserTime(scxulong& userTime) const;
        bool GetNiceTime(scxulong& niceTime) const;
        bool GetPrivilegedTime(scxulong& privilegedTime) const;
        bool GetIowaitTime(scxulong& iowaitTime) const;
        bool GetInterruptTime(scxulong& interruptTime) const;
        bool GetDpcTime(scxulong& dpcTime) const;
        bool GetQueueLength(scxulong& queueLength) const;

#if defined(aix)
        void UpdateDataSampler(perfstat_cpu_t *raw);
        void UpdateDataSampler(perfstat_cpu_total_t *raw);
#endif

        scxulong GetUserLastTick() const;
        scxulong GetIdleLastTick() const;
        scxulong GetNiceLastTick() const;
        scxulong GetPrivilegedLastTick() const;
        scxulong GetIowaitLastTick() const;
        scxulong GetInterruptLastTick() const;
        scxulong GetSWInterruptLastTick() const;
        scxulong GetTotalLastTick() const;

    private:
        scxulong GetPercentageSafe(const scxulong tic_delta,
				   const scxulong tot_delta,
				   const bool inverse = false) const;

    private:

        SCXCoreLib::SCXLogHandle m_log;  //!< Log handle

        std::wstring m_procName;         //!< Processor name
        unsigned int m_procNumber;       //!< Processor number

        scxulong m_processorTime;        //!< Processor time.
        scxulong m_idleTime;             //!< Processor idle time.
        scxulong m_userTime;             //!< Processor user time.
        scxulong m_niceTime;             //!< Processor nice time.
        scxulong m_privilegedTime;       //!< Processor privileged time.
        scxulong m_iowaitTime;           //!< Processor io wait time.
        scxulong m_interruptTime;        //!< Processor interrupt time.
        scxulong m_dpcTime;              //!< Processor dpc time.
        scxulong m_queueLength;          //!< Processor queue length.

        // NB: Not all of these are used on every platform
        CPUInstanceDataSampler m_UserCPU_tics;       //!< Data sampler for user time.
        CPUInstanceDataSampler m_NiceCPU_tics;       //!< Data sampler for nice time.
        CPUInstanceDataSampler m_SystemCPUTime_tics; //!< Data sampler for system time.
        CPUInstanceDataSampler m_IdleCPU_tics;       //!< Data sampler for idle time.
        CPUInstanceDataSampler m_IOWaitTime_tics;    //!< Data sampler for IO wait time.
        CPUInstanceDataSampler m_IRQTime_tics;       //!< Data sampler for IRQ time.
        CPUInstanceDataSampler m_SoftIRQTime_tics;   //!< Data sampler for soft IRQ time
        CPUInstanceDataSampler m_Total_tics;         //!< Data sampler for total time.
    };

}

#endif /* CPUINSTANCE_H */
/*----------------------------E-N-D---O-F---F-I-L-E---------------------------*/
