/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
    \file

    \brief      Contains the definition of the DataSampler template class.
    

    \date       2007-07-10 13:57:48

*/
/*----------------------------------------------------------------------------*/
#ifndef DATASAMPLER_H
#define DATASAMPLER_H

#include <scxcorelib/scxthreadlock.h>
#include <deque>

namespace SCXSystemLib  
{
    /*----------------------------------------------------------------------------*/
    /**
        Template Class that represents a series of measurements of a particular
        value over time. 
        
        \param T Sample type. This type has to be of a numerical type, or
                 more speciffically, it has to support the operato - and
                 must be devidable by an integer.
        \param maxSamples Maximum number of samples represented.
        
        Could for example be used to collect statistics about how
        a counter value changes over time.

    */
    template<class T, int maxSamples> class DataSampler
    {
    public:
        /*----------------------------------------------------------------------------*/
        /**
            Constructor.
            
        */
        DataSampler() : m_lock(SCXCoreLib::ThreadLockHandleGet())
        {
        }

        /*----------------------------------------------------------------------------*/
        /**
            Add a new sample.
        
            \param  sample New sample.
            
        */
        void AddSample(T sample)
        {
            SCXCoreLib::SCXThreadLock lock(m_lock);
            m_samples.push_front(sample);
            if ( m_samples.size() > maxSamples)
            {
                m_samples.pop_back();
            }
        }

        /*----------------------------------------------------------------------------*/
        /**
            Check if the latest value added is smaller then an earlier value
        
            \param  samples Number of samples to go back.
            
            \returns  true if smaller, else false
            
        */
        bool HasWrapped(size_t samples)
        {
            if (m_samples.size() < 2)
            {
                return false;
            }
            SCXCoreLib::SCXThreadLock lock(m_lock);
            size_t index = samples - 1;
            if (samples > m_samples.size())
            {
                index = m_samples.size() - 1;
            }
            return m_samples[0] < m_samples[index];
        }

        /*----------------------------------------------------------------------------*/
        /**
           Get the average value of all samples.

           \returns  average value of all samples.

        */
        template <class V> V GetAverage() const 
        {
            V sum = 0;
            if (m_samples.size() == 0)
            {
                return sum;
            }

            SCXCoreLib::SCXThreadLock lock(m_lock);
            for (typename std::deque<T>::const_iterator it = m_samples.begin(); it != m_samples.end(); ++it) 
            {
                sum += static_cast<V>(*it);
            } 
            return sum / static_cast<V>(m_samples.size());
        }

        /*----------------------------------------------------------------------------*/
        /**
            Get the average change in value in the latest samples.
        
            \param  samples Number of samples to go back.
            
            \returns  Average change in value in the last samples.
            
            If the number of collected samples is less than the samples parameter, a
            best effort average is returned using all the samples collected.
            
        */
        T GetAverageDelta(size_t samples) const
        {
            return GetAverageDeltaFactored(samples, 1);
        }

        /*----------------------------------------------------------------------------*/
        /**
            Get the average change in value in the latest samples factored by a given factor.
    
            \param       samples number of samples to go back.
            \param       factor factor to increase the average
            \returns     average change in value in the last samples factored by the factor.
    
            \date        07-08-22 15:10:00
    
            Used to get better values for integer averages which should be factored.
    
        */
        T GetAverageDeltaFactored(size_t samples, T factor) const
        {
            if (samples < 2 || m_samples.size() < 2 || 0 == factor)
            {
                // Too few samples to produce a valid output (or zero factor).
                return T();
            }
            SCXCoreLib::SCXThreadLock lock(m_lock);
            size_t index = samples - 1;
            if (samples > m_samples.size())
            {
                index = m_samples.size() - 1;
            }
            return ((m_samples[0] - m_samples[index])*factor) / static_cast<T>(index);
        }

        /*----------------------------------------------------------------------------*/
        /**
            Get the change in value in the latest samples.
    
            \param       samples Number of samples to go back.
            \returns     change in value in the last samples factored by the factor.
    
            \date        07-08-22 15:10:00
    
        */
        T GetDelta(size_t samples) const
        {
            if (samples < 2 || m_samples.size() < 2)
            {
          
                // Too few samples to produce a valid output.
                return T();
            }
            SCXCoreLib::SCXThreadLock lock(m_lock);
            size_t index = samples - 1;
            if (samples > m_samples.size())
            {
                index = m_samples.size() - 1;
            }
            return m_samples[0] - m_samples[index];
        }

        /*----------------------------------------------------------------------------*/
        /**
            Get a specific sample value.
    
            \param       index sample index to retrieve.
            \returns     The sample value ate given index.
            \throws      SCXIllegalIndexExceptionUInt if index is larger than sample history.
    
            \date        07-08-22 15:10:00
    
        */
        T operator [] (size_t index) const
        {
            SCXCoreLib::SCXThreadLock lock(m_lock);
            if (index >= m_samples.size())
            {
                throw SCXCoreLib::SCXIllegalIndexException<size_t>(L"index", index, SCXSRCLOCATION);
            }
            return m_samples[index];
        }

        /*----------------------------------------------------------------------------*/
        /**
            Erase all samples.
    
            \date        07-08-22 15:10:00
    
        */
        void Clear()
        {
            SCXCoreLib::SCXThreadLock lock(m_lock);
            m_samples.clear();
        }

        /*----------------------------------------------------------------------------*/
        /**
            Retrieve the number of samples.
        
            \returns      Number of samples saved.
            
        */
        size_t GetNumberOfSamples() const
        {
            SCXCoreLib::SCXThreadLock lock(m_lock);
            return m_samples.size();
        }

    private:
        SCXCoreLib::SCXThreadLockHandle m_lock;  //!< Makes the datasampler thread safe.
        std::deque<T> m_samples;                 //!< Contains the samples.
    };
}

#endif /* DATASAMPLER_H */
/*----------------------------E-N-D---O-F---F-I-L-E---------------------------*/
