/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
    \file

    \brief       PAL representation of Operating System
    \date        08-03-04 14:36:00

*/
/*----------------------------------------------------------------------------*/
#ifndef OSINSTANCE_H
#define OSINSTANCE_H

#include <scxcorelib/scxlog.h>
#include <scxcorelib/scxtime.h>
#include <scxsystemlib/entityinstance.h>
#include <string>

// Definitions for uname()
#include <sys/utsname.h>

#if defined(hpux)
#include <sys/pstat.h>
#endif


namespace SCXSystemLib
{
    using SCXCoreLib::SCXCalendarTime;

    /*----------------------------------------------------------------------------*/
    /**
       Class that represents the common set of OS parameters.

       This class only implements the total instance and has no
       collection threrad.

       These are the type mappings used in the property methods.
       typedef unsigned short uint16;
       typedef unsigned int uint32;
       typedef scxulong uint64;
       typedef signed short sint16;
       typedef bool boolean;
       typedef SCXCalendarTime datetime;
    */
    class OSInstance : public EntityInstance
    {
        friend class ProcessEnumeration;

        static const wchar_t *moduleIdentifier;         //!< Shared module string

    public:
        OSInstance();
        virtual ~OSInstance();

        virtual void Update();
        virtual void CleanUp();

        virtual const std::wstring DumpString() const;

        /* Properties of CIM_ManagedElement */
        bool GetCaption(std::wstring& cap) const;
        bool GetDescription(std::wstring& desc) const;

        /* Properties of CIM_ManagedSystemElement */
        bool GetName(std::wstring& name) const;

        /* Properties of CIM_OperatingSystem */
        bool GetOSType(unsigned short& ost) const;
        bool GetOtherTypeDescription(std::wstring& otd) const;
        bool GetVersion(std::wstring& ver) const;
        bool GetLastBootUpTime(SCXCalendarTime& lbut) const;
        bool GetLocalDateTime(SCXCalendarTime& ldt) const;
        bool GetCurrentTimeZone(signed short& ctz) const;
        bool GetNumberOfLicensedUsers(unsigned int& nolu) const;
        bool GetNumberOfUsers(unsigned int& numberOfUsers) const;
        bool GetMaxNumberOfProcesses(unsigned int& mp) const;
        bool GetMaxProcessMemorySize(scxulong& mpms) const;
        bool GetMaxProcessesPerUser(unsigned int& mppu) const;

        /* Properties of PG_OperatingSystem */
        bool GetSystemUpTime(scxulong& sut) const;

    private:
        SCXCoreLib::SCXLogHandle m_log;         //!< Log handle.

        struct utsname m_unameInfo;             //!< Saves the output from uname()
        bool m_unameIsValid;                    //!< Is m_unameInfo valid now?

#if defined(linux)
        void SetBootTime(void);
        void PrecomputeMaxProcesses(void);
        void IdentifyDistribution(void);

        unsigned int m_MaxProcesses;            //!< Maximum number of process contexts.
        double m_upsec;                         //!< Uptime in seconds
        std::wstring m_platform;                     //!< The current platform and version
#endif

        SCXCalendarTime m_system_boot;          //!< Time of system boot
        bool m_system_boot_isValid;             //!< Is m_system_boot valid now?
        SCXCalendarTime m_now;                  //!< Current time on system

#if defined(hpux)
        void SetBootTime(void);

        struct pst_static m_psts;               //!< Holds output from pstat_getstatic
        struct pst_dynamic m_pstd;              //!< Holds output from pstat_getdynamic
        bool m_psts_isValid;                    //!< Is m_psts valid now?
        bool m_pstd_isValid;                    //!< Is m_pstd valid now?
#endif

        /**
           A constant returned by the GetOSType call.
           This comes from the Pegasus code. Source of the original definition
           is unknown.
        */
        enum OSTYPE {
            /* Useless documentation to make doxygen happy */
            Unknown,            //!< Unknown
            Other,              //!< Other
            MACOS,              //!< MACOS
            ATTUNIX,            //!< ATTUNIX
            DGUX,               //!< DGUX
            DECNT,              //!< DECNT
            Digital_Unix,       //!< Digital_Unix
            OpenVMS,            //!< OpenVMS
            HP_UX,              //!< UX
            AIX,                //!< AIX
            MVS,                //!< MVS
            OS400,              //!< OS400
            OS2,                //!< OS2
            JavaVM,             //!< JavaVM
            MSDOS,              //!< MSDOS
            WIN3x,              //!< WIN3x
            WIN95,              //!< WIN95
            WIN98,              //!< WIN98
            WINNT,              //!< WINNT
            WINCE,              //!< WINCE
            NCR3000,            //!< NCR3000
            NetWare,            //!< NetWare
            OSF,                //!< OSF
            DCOS,               //!< DCOS
            Reliant_UNIX,       //!< Reliant UNIX
            SCO_UnixWare,       //!< SCO UnixWare
            SCO_OpenServer,     //!< SCO OpenServer
            Sequent,            //!< Sequent
            IRIX,               //!< IRIX
            Solaris,            //!< Solaris
            SunOS,              //!< SunOS
            U6000,              //!< U6000
            ASERIES,            //!< ASERIES
            TandemNSK,          //!< TandemNSK
            TandemNT,           //!< TandemNT
            BS2000,             //!< BS2000
            LINUX,              //!< LINUX
            Lynx,               //!< Lynx
            XENIX,              //!< XENIX
            VM_ESA,             //!< VM/ESA
            Interactive_UNIX,   //!< Interactive UNIX
            BSDUNIX,            //!< BSDUNIX
            FreeBSD,            //!< FreeBSD
            NetBSD,             //!< NetBSD
            GNU_Hurd,           //!< Hurd
            OS9,                //!< OS9
            MACH_Kernel,        //!< Mach Kernel
            Inferno,            //!< Inferno
            QNX,                //!< QNX
            EPOC,               //!< EPOC
            IxWorks,            //!< IxWorks
            VxWorks,            //!< VxWorks
            MiNT,               //!< MiNT
            BeOS,               //!< BeOS
            HP_MPE,             //!< HP MPE
            NextStep,           //!< NextStep
            PalmPilot,          //!< PalmPilot
            Rhapsody,           //!< Rhapsody
            Windows_2000,       //!< Windows 2000
            Dedicated,          //!< Dedicated
            OS_390,             //!< OS 390
            VSE,                //!< VSE
            TPF,                //!< TPF
            Windows_Me,         //!< Me
            Open_UNIX,          //!< Open UNIX
            OpenBDS,            //!< OpenBDS
            NotApplicable,      //!< NotApplicable
            Windows_XP,         //!< Windows XP
            zOS,                //!< zOS
            Windows_2003,       //!< Windows 2003
            Windows_2003_64     //!< Windows_2003_64
        };

    };
}
#endif /* OSINSTANCE_H */
/*----------------------------E-N-D---O-F---F-I-L-E---------------------------*/
