/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
    \file

    \brief          Enumeration of Process Items
    \date           07-10-29 15:24:00
    
*/
/*----------------------------------------------------------------------------*/
#ifndef PROCESSENUMERATION_H
#define PROCESSENUMERATION_H

#include <map>

#include <scxsystemlib/entityenumeration.h>
#include <scxsystemlib/processinstance.h>
#include <scxcorelib/scxlog.h>
#include <scxcorelib/scxthread.h>
#include <scxcorelib/scxhandle.h>
 
namespace SCXSystemLib
{
    /** Time between each sample in seconds. */
    const int PROCESS_SECONDS_PER_SAMPLE = 60;

    /** Type of live process map. One pid corresponds to one process. */
    typedef std::map<scxpid_t, SCXCoreLib::SCXHandle<ProcessInstance> > ProcMap;

    /*----------------------------------------------------------------------------*/
    /**
        Class that represents a collection of Process:s.
        
        PAL Holding collection of Process:s.
    */
    class ProcessEnumeration : public EntityEnumeration<ProcessInstance>
    {
    public:
        static const wchar_t *moduleIdentifier;         //!< Module identifier

        ProcessEnumeration();
        ~ProcessEnumeration();
        virtual void Init();
        virtual void Update(bool updateInstances=true);
        virtual void CleanUp();

        size_t Size() const;

        const SCXCoreLib::SCXThreadLockHandle& GetLockHandle() const;
        void UpdateNoLock(SCXCoreLib::SCXThreadLock& lck, bool updateInstances=true);

        /* This one is public for testing purposes */
        void SampleData();

        SCXCoreLib::SCXHandle<ProcessInstance> Find(scxpid_t pid);
        std::vector<SCXCoreLib::SCXHandle<ProcessInstance> > Find(const std::wstring& name);
        static bool SendSignalByName(const std::wstring& name, int sig);
        static bool GetNumberOfProcesses(unsigned int& numberOfProcesses);

    private:
        SCXCoreLib::SCXLogHandle m_log;                         //!< Handle to log file 
        SCXCoreLib::SCXThreadLockHandle m_lock; //!< Handles locking in the process enumeration.

        SCXCoreLib::SCXHandle<SCXCoreLib::SCXThread> m_dataAquisitionThread; //!< Thread pointer.
        static void DataAquisitionThreadBody(SCXCoreLib::SCXThreadParamHandle& param);

        /** Map of active processes */
        ProcMap m_procs;

        int m_EnumErrorCount;    //!< Number of consecutive enumeration attempts with errors.
        int m_EnumGoodCount;     //!< Number of consecutive enumeration attempts without errors.
        SCXCoreLib::SCXLogSeverity m_EnumLogLevel;  //!< Log level to use when logging execption during instance update
    };

}

#endif /* PROCESSENUMERATION_H */
/*----------------------------E-N-D---O-F---F-I-L-E---------------------------*/
