/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
    \file        

    \brief       Instances of Process Items 
    
    \date        07-10-29 15:27:00

    PAL representation of a Process instance
    
*/
/*----------------------------------------------------------------------------*/
#ifndef PROCESSINSTANCE_H
#define PROCESSINSTANCE_H

#if defined(sun)
#include <procfs.h>
#endif

#if defined(hpux)
#include <sys/pstat.h>
#endif

#if defined(aix)

#ifdef _DEBUG
// workaround to make build going; proper fix to be provided later (wi 9584)
#define SCX_UNDEF_DEBUG
#undef _DEBUG
#endif

#include <sys/procfs.h>
#include <procinfo.h>

#ifdef SCX_UNDEF_DEBUG
#undef _DEBUG
#define _DEBUG
#undef SCX_UNDEF_DEBUG
#endif

#endif

#include <string>

#include <scxsystemlib/entityinstance.h>
#include <scxsystemlib/datasampler.h>
#include <scxcorelib/stringaid.h>
#include <scxcorelib/scxlog.h>
#include <scxcorelib/scxtime.h>

namespace SCXSystemLib  
{
#ifdef linux
    /** 
        Linux specific process information.
        This is the contents of the /proc/<PROCESS#>/stat file. The scan string
        and types of the fields are taken from the proc(5) manpage.
    */
    struct LinuxProcStat {
        int pid;                //!< %d  1
        char comm[30];          //!< %s
        char state;             //!< %c
        int ppid;               //!< %d
        int pgrp;               //!< %d  5
        int session;            //!< %d
        int tty_nr;             //!< %d
        int tpgid;              //!< %d
        unsigned long flags;    //!< %lu
        unsigned long minflt;   //!< %lu 10
        unsigned long cminflt;  //!< %lu
        unsigned long majflt;   //!< %lu
        unsigned long cmajflt;  //!< %lu
        unsigned long utime;    //!< %lu
        unsigned long stime;    //!< %lu 15
        long cutime;            //!< %ld
        long cstime;            //!< %ld
        long priority;          //!< %ld
        long nice;              //!< %ld
        // dummy at this position, not read;    //!< %ld 20
        long itrealvalue;       //!< %ld
        unsigned long starttime;//!< %lu
        unsigned long vsize;    //!< %lu
        long rss;               //!< %ld
        unsigned long rlim;     //!< %lu 25
        unsigned long startcode;//!< %lu
        unsigned long endcode;  //!< %lu
        unsigned long startstack;//!< %lu
        unsigned long kstkesp;  //!< %lu
        unsigned long kstkeip;  //!< %lu 30
        unsigned long signal;   //!< %lu
        unsigned long blocked;  //!< %lu
        unsigned long sigignore;//!< %lu
        unsigned long sigcatch; //!< %lu
        unsigned long wchan;    //!< %lu 35
        unsigned long nswap;    //!< %lu
        unsigned long cnswap;   //!< %lu
        int exit_signal;        //!< %d
        int processor;          //!< %d
        unsigned long rt_priority;//!< %lu 40 (Since 2.5.19)
        unsigned long policy;   //!< %lu    (Since 2.5.19)

    private:
        static const int procstat_len = 40; //!< Number of fields not counting dummy

        /** The format string that should be supplied to fscanf() to read procstat_fields */
        static const char *scanstring;  /* = 
        "%d %s %c %d %d %d %d %d %lu %lu " // 1 to 10
        "%lu %lu %lu %lu %lu %ld %ld %ld %ld %*ld " // 11 to 20
        "%ld %lu %lu %ld %lu %lu %lu %lu %lu %lu " // 21 to 30
        "%lu %lu %lu %lu %lu %lu %lu %d %d %lu %lu"; // 31 to 41
                                       */
    public:
        bool ReadStatFile(FILE *filePointer, const char* filename);
    };

    /** Holds Linux memory statistics */
    struct LinuxProcStatM {

        unsigned long size;     //!< total program size 
        unsigned long resident; //!< resident set size
        unsigned long share;    //!< shared pages
        unsigned long text;     //!< text (code)
        unsigned long lib;      //!< library
        unsigned long data;     //!< data/stack

    private:
        static const int procstat_len = 6; //!< Number of fields

        /** The format string that should be supplied to fscanf() to read procstat_fields */
        static const char *scanstring;  /* = "%lu %lu %lu %lu %lu %lu" */
    public:
        bool ReadStatMFile(FILE *filePointer, const char* filename);
    };

#endif /* Linux */

    /** Implements subtraction for system type struct timeval.
        \param tv1 A timeval
        \param tv2 Another timeval
        \returns Difference between tv1 and tv2
        This is present so that the DataSampler class can work correctly.
    */
    inline struct timeval operator-(const struct timeval& tv1, const struct timeval& tv2)
    {
        struct timeval tmp;
        if (tv2.tv_usec > tv1.tv_usec) {
            tmp.tv_usec = tv1.tv_usec + 1000000 - tv2.tv_usec;
            tmp.tv_sec = tv1.tv_sec - tv2.tv_sec - 1;           
        } else {
            tmp.tv_usec = tv1.tv_usec - tv2.tv_usec;
            tmp.tv_sec = tv1.tv_sec - tv2.tv_sec;
        }
        return tmp;
    }

#if defined(sun)
    /** Datatype used for time in /proc is redefined into a custom type. */
    typedef timestruc_t scx_timestruc_t;
#elif defined(aix)
    /** Datatype used for time in /proc is redefined into a custom type. */
    typedef pr_timestruc64_t scx_timestruc_t;
#endif

#if defined(sun) || defined(aix)
    /** Implements subtraction for the type scx_timestruc_t.
        \param ts1 A scx_timestruc_t
        \param ts2 Another scx_timestruc_t
        \returns Difference between ts1 and ts2
        This is present so that the DataSampler class can work correctly.
    */
    inline scx_timestruc_t operator-(const scx_timestruc_t& ts1, const scx_timestruc_t& ts2)
    {
        scx_timestruc_t tmp;
        if (ts2.tv_nsec > ts1.tv_nsec) {
            tmp.tv_nsec = ts1.tv_nsec + 1000000000UL - ts2.tv_nsec;
            tmp.tv_sec = ts1.tv_sec - ts2.tv_sec - 1;
        } else {
            tmp.tv_nsec = ts1.tv_nsec - ts2.tv_nsec;
            tmp.tv_sec = ts1.tv_sec - ts2.tv_sec;
        }
        return tmp;
    }


    /** Implements addition for the type scx_timestruc_t.
        \param ts1 A scx_timestruc_t
        \param ts2 Another scx_timestruc_t
        \returns Sum of ts1 and ts2
    */
    inline scx_timestruc_t operator+(const scx_timestruc_t& ts1, const scx_timestruc_t& ts2)
    {
        scx_timestruc_t tmp;
        tmp.tv_nsec = ts1.tv_nsec + ts2.tv_nsec;
        tmp.tv_sec = ts1.tv_sec + ts2.tv_sec;

        if (tmp.tv_nsec > 1000000000UL) {
            tmp.tv_nsec -= 1000000000UL;
            tmp.tv_sec += 1;
        }
        return tmp;
    }
#endif

    /*----------------------------------------------------------------------------*/

    typedef scxulong scxpid_t;  //!< Internal type of process id

    /** Number of samples collected in the datasampler for CPU. */
    const int MAX_PROCESSINSTANCE_DATASAMPER_SAMPLES = 6;

    /** Datasampler for CPU information. */
    typedef DataSampler<scxulong, MAX_PROCESSINSTANCE_DATASAMPER_SAMPLES> ScxULongDataSampler_t;
    /** Datasampler for time stored as a struct timeval */
    typedef DataSampler<struct timeval, MAX_PROCESSINSTANCE_DATASAMPER_SAMPLES> TvDataSampler_t;
#if defined(sun) || defined(aix)
    /** Datasampler for time stored as a scx_timestruc_t (which is specific for solaris&aix) */
    typedef DataSampler<scx_timestruc_t, MAX_PROCESSINSTANCE_DATASAMPER_SAMPLES> TsDataSampler_t;
#endif

    /** Max length of filename in /proc filesystem. */
    const size_t PROCPATH_LEN = 30;
    
    /*----------------------------------------------------------------------------*/

    /**
        Class that represents an instances of a unix process.
        
        Concrete implementation of an instance of a Process entity
    */
    class ProcessInstance : public EntityInstance
    {
        friend class ProcessEnumeration;

        static const wchar_t *moduleIdentifier;         //!< Shared module string

#if defined(linux) || defined(sun) || defined(aix)
    private:
        ProcessInstance(scxpid_t pid, const char* basename);
        bool UpdateInstance(const char* basename, bool initial);
#endif

#if defined(linux)
        void SetBootTime(void);
#endif

#if defined(hpux)
    private:
        ProcessInstance(scxpid_t pid, struct pst_status *pstatus);
        bool UpdateInstance(struct pst_status *pstatus, bool initial);
#endif  

    public:
        /** Gets the process ID which this instance represents. */
        scxpid_t getpid() { return m_pid; }

        virtual ~ProcessInstance();

        static bool m_inhibitAccessViolationCheck;

        /* Properties in SCX_UnixProcess */
        bool GetPID(scxulong& x) const;
        bool GetName(std::string&) const;
        bool GetPriority(unsigned int& prio) const;
        bool GetExecutionState(unsigned short& state) const;
        bool GetCreationDate(SCXCoreLib::SCXCalendarTime& cre) const;
        bool GetTerminationDate(SCXCoreLib::SCXCalendarTime& term) const;
        bool GetParentProcessID(int& pid) const;
        bool GetRealUserID(scxulong& uid) const;
        bool GetProcessGroupID(scxulong& pgid) const;
        bool GetProcessNiceValue(unsigned int& nice) const;

        /* Properties in SCX_UnixProcess, Phase 2 */
        bool GetOtherExecutionDescription(std::wstring& description) const;
        bool GetKernelModeTime(scxulong& kmt) const;
        bool GetUserModeTime(scxulong& umt) const;
        bool GetWorkingSetSize(scxulong& wss) const;
        bool GetProcessSessionID(scxulong& sid) const;
        bool GetProcessTTY(std::string& tty) const;
        bool GetModulePath(std::string& modpath) const;
        bool GetParameters(std::vector<std::string>& params) const;
        bool GetProcessWaitingForEvent(std::string& event) const;

        /* Properties in SCX_UnixProcessStatisticalInformation */
        bool GetCPUTime(unsigned int& cpu) const;
        bool GetBlockWritesPerSecond(scxulong &bws) const;
        bool GetBlockReadsPerSecond(scxulong &bwr) const;
        bool GetBlockTransfersPerSecond(scxulong &bts) const;
        bool GetPercentUserTime(scxulong &put) const;
        bool GetPercentPrivilegedTime(scxulong &ppt) const;
        bool GetUsedMemory(scxulong &um) const;
        bool GetPercentUsedMemory(scxulong &) const;
        bool GetPagesReadPerSec(scxulong &prs) const;

        /* Properties in SCX_UnixProcessStatisticalInformation, Phase 2 */
        bool GetRealText(scxulong &rt) const;
        bool GetRealData(scxulong &rd) const;
        bool GetRealStack(scxulong &rs) const;
        bool GetVirtualText(scxulong &vt) const;
        bool GetVirtualData(scxulong &vd) const;
        bool GetVirtualStack(scxulong &vs) const;
        bool GetVirtualMemoryMappedFileSize(scxulong &vmmfs) const;
        bool GetVirtualSharedMemory(scxulong &vsm) const;
        bool GetCpuTimeDeadChildren(scxulong &ctdc) const;
        bool GetSystemTimeDeadChildren(scxulong &stdc) const;

        /* Utility stuff */
        bool SendSignal(int signl) const;

        std::wstring DumpString(void);
    private:
        /** Tests if this instance was detected when scanning live processes. */ 
        bool WasFound() { bool found = m_found; m_found = false; return found; }
        void UpdateDataSampler(struct timeval& realtime); 
        void UpdateTimedValues(void);
        void CheckRootAccess(void) const;

        SCXCoreLib::SCXLogHandle m_log;         //!< Log handle.
        scxpid_t m_pid;                         //!< Process ID of this instance
        bool m_found;                           //!< Found during iteration
        bool m_accessViolationEncountered;      //!< Flag that we've had problems with access
        struct timeval m_timeOfDeath;           //!< When did process die

#if defined(linux)
        char m_procStatName[PROCPATH_LEN];      //!< Name of /proc/#/stat file
        char m_procStatMName[PROCPATH_LEN];     //!< Name of /proc/#/statm file
        uid_t     m_uid;                        //!< User ID of owner 
        gid_t     m_gid;                        //!< Group ID of owner 
        LinuxProcStat m;                        //!< Linux specific process information
        LinuxProcStatM n;                       //!< Linux specific process information
        static SCXCoreLib::SCXCalendarTime m_system_boot; //!< Time of system boot 
        unsigned int m_jiffies_per_second;              //!< Time base for PC Linux
        static const unsigned int m_pageSize = 4;       //!< Page size in KB on Linux

        TvDataSampler_t       m_RealTime_tics;          //!< Data sampler for real time.
        ScxULongDataSampler_t m_UserTime_tics;          //!< Data sampler for user time.
        ScxULongDataSampler_t m_SystemTime_tics;        //!< Data sampler for system time.
        ScxULongDataSampler_t m_HardPageFaults_tics;    //!< Data sampler for hard page faults.

        /* These are updated when UpdateTimedValues() is run. */
        struct timeval m_delta_RealTime;                //!< Elapsed real time at update
        scxulong m_delta_UserTime;                      //!< Consumed user time at update
        scxulong m_delta_SystemTime;                    //!< Consumed system time at update
        scxulong m_delta_HardPageFaults;                //!< Executed page faults at update

        scxulong ComputeItemsPerSecond(scxulong delta_item,             // Defined below
                                       const struct timeval& elapsedTime) const;
        unsigned int ComputePercentageOfTime(scxulong consumedTime,     // Defined below
                                             const struct timeval& elapsedTime) const;
#endif

#if defined(sun)
        char m_procPsinfoName[PROCPATH_LEN];    //!< Name of /proc/#/psinfo file
        char m_procStatusName[PROCPATH_LEN];    //!< Name of /proc/#/status file (protected)
        char m_procUsageName[PROCPATH_LEN];     //!< Name of /proc/#/usage file

        bool m_logged64BitError;                //!< Has a warning log about problems reading 64-bit process info been issued

        psinfo_t m_psinfo;                      //!< Solaris specific process information
        pstatus_t m_pstat;                      //!< Solaris specific process information
        prusage_t m_puse;                       //!< Solaris specific process information
        unsigned long m_clocksPerSecond;        //!< System clock ticks per second

        TvDataSampler_t m_RealTime_tics;                //!< Data sampler for real time.
        TsDataSampler_t m_UserTime_tics;                //!< Data sampler for user time.
        TsDataSampler_t m_SystemTime_tics;              //!< Data sampler for system time.
        ScxULongDataSampler_t m_BlockOut_tics;          //!< Data sampler for written blocks.
        ScxULongDataSampler_t m_BlockInp_tics;          //!< Data sampler for read blocks.
        ScxULongDataSampler_t m_HardPageFaults_tics;    //!< Data sampler for hard page faults.

        /* These are updated when UpdateTimedValues() is run. */
        struct timeval m_delta_RealTime;                //!< Elapsed real time at update
        scx_timestruc_t m_delta_UserTime;               //!< Consumed user time at update
        scx_timestruc_t m_delta_SystemTime;             //!< Consumed system time at update
        scxulong m_delta_BlockOut;                      //!< Elapsed block outputs at update
        scxulong m_delta_BlockInp;                      //!< Elapsed block inputs at update
        scxulong m_delta_HardPageFaults;                //!< Executed page faults at update

        scxulong ComputeItemsPerSecond(scxulong delta_item,             // Defined below
                                       const timeval& elapsedTime) const;
        unsigned int ComputePercentageOfTime(const scx_timestruc_t& consumedTime,//  Below
                                             const struct timeval& elapsedTime) const;
#endif

#if defined(hpux)
        struct pst_status m_pstatus;            //!< HP/UX specific process information
        static const unsigned int m_pageSize = 4; //!< Page size in KB on HP/UX

        TvDataSampler_t       m_RealTime_tics;          //!< Data sampler for real time.  
        ScxULongDataSampler_t m_UserTime_tics;          //!< Data sampler for user time.
        ScxULongDataSampler_t m_SystemTime_tics;        //!< Data sampler for system time.
        //ScxULongDataSampler_t m_CPUTime_tics;           //!< Data sampler for cpu time.
        //ScxULongDataSampler_t m_CPUTime_total_tics;     //!< Data sampler for process life time.
        ScxULongDataSampler_t m_BlockOut_tics;          //!< Data sampler for written blocks.
        ScxULongDataSampler_t m_BlockInp_tics;          //!< Data sampler for read blocks.
        ScxULongDataSampler_t m_HardPageFaults_tics;    //!< Data sampler for hard page faults.

        /* These are updated when UpdateTimedValues() is run. */
        struct timeval m_delta_RealTime;                //!< Elapsed real time at update
        scxulong m_delta_UserTime;                      //!< Consumed user time at update
        scxulong m_delta_SystemTime;                    //!< Consumed system time at update

        //scxulong m_delta_CPUTime;
        //scxulong m_delta_CPUTime_total;

        scxulong m_delta_BlockOut;                      //!< Elapsed block outputs at update
        scxulong m_delta_BlockInp;                      //!< Elapsed block inputs at update
        scxulong m_delta_HardPageFaults;                //!< Executed page faults at update

        scxulong ComputeItemsPerSecond(scxulong delta_item,             // Defined below
                                       const struct timeval& elapsedTime) const;
        unsigned int ComputePercentageOfTime(scxulong consumedTime,     // Defined below
                                             const struct timeval& elapsedTime) const;
#endif

#if defined(aix)
        char m_procPsinfoName[PROCPATH_LEN];    //!< Name of /proc/#/psinfo file
        char m_procStatusName[PROCPATH_LEN];    //!< Name of /proc/#/status file (protected)

        struct procentry64 m_procentry;         //!< AIX specific process information
        psinfo_t m_psinfo;                      //!< AIX specific process information
        pstatus_t m_pstat;                      //!< AIX specific process information
        unsigned long m_clocksPerSecond;        //!< System clock ticks per second

        TvDataSampler_t m_RealTime_tics;                //!< Data sampler for real time.
        TsDataSampler_t m_UserTime_tics;                //!< Data sampler for user time.
        TsDataSampler_t m_SystemTime_tics;              //!< Data sampler for system time.

        /* These are updated when UpdateTimedValues() is run. */
        struct timeval m_delta_RealTime;                //!< Elapsed real time at update
        scx_timestruc_t m_delta_UserTime;              //!< Consumed user time at update
        scx_timestruc_t m_delta_SystemTime;            //!< Consumed system time at update

        scxulong ComputeItemsPerSecond(scxulong delta_item,             // Defined below
                                       const timeval& elapsedTime) const;
        unsigned int ComputePercentageOfTime(const scx_timestruc_t& consumedTime,// Inline
                                             const struct timeval& elapsedTime) const;
#endif

    };

    /**
       Constructs a identity string for debug printouts
     
       This is exclusively meant for debug output. It will output pid and short name
       for process.
     */
    inline std::wstring ProcessInstance::DumpString(void)
    {
#if defined(linux)
        std::string name(m.comm);
        long pid = m.pid;
#elif defined(sun)
        std::string name(m_psinfo.pr_fname);
        long pid = m_psinfo.pr_pid;
#elif defined(hpux)
        std::string name(m_pstatus.pst_ucomm);
        long pid = m_pstatus.pst_pid;
#elif defined(aix)
        std::string name(m_psinfo.pr_fname);
        long pid = m_psinfo.pr_pid;
#else
        #error "Not supported"
#endif
        std::stringstream ss;
        if (static_cast<long>(m_pid) != pid) {
            ss << "<" << m_pid << ":" << pid << ">" << name;
        } else {
            ss << "<" << m_pid << ">" << name;
        }
        return SCXCoreLib::StrFromMultibyte(ss.str());
    }

#if defined(linux)
    /**
       Computes a parameter for which we have a delta of some kind of item
       and a time delta.

       \param delta_item Delta of usage
       \param elapsedTime Delta of time
       \returns A measure of items per second

       \note If we ever need this in something else that scxulong then 
       make this a template.
    */
    inline scxulong 
    ProcessInstance::ComputeItemsPerSecond(scxulong delta_item, 
                                           const struct timeval& elapsedTime) const
    {
        // Convert elapsed time to milliseconds
        scxulong el = 1000 * elapsedTime.tv_sec + elapsedTime.tv_usec / 1000;
        if (el == 0) return 0;  // Avoid divide by zero

        return 1000 * delta_item / el;
    }


    /**
       Computes the percentage of measure of time in relation to another measure of time.
       
       \param consumedTime Consumed time
       \param elapsedTime Elapsed time
       \returns A percentage number on how consumed time relates to elapsed time

       \note Delta time on Linux is in jiffies. Like it or not. 
    */
    inline unsigned int 
    ProcessInstance::ComputePercentageOfTime(scxulong consumedTime, 
                                             const struct timeval& elapsedTime) const
    {
        // Convert both to milliseconds to get a resonable resolution WO overflow
        unsigned long el = 1000 * elapsedTime.tv_sec + elapsedTime.tv_usec / 1000;
        if (el == 0) { return 0; } // Avoid divide by zero
        scxulong co = 1000 * consumedTime / m_jiffies_per_second;
        return static_cast<unsigned int>(100 * co / el);
    }

#endif /* linux */

#if defined(sun) || defined(aix)
    /**
       Computes a parameter for which we have a delta of some kind of item
       and a time delta.

       \param delta_item Delta of usage
       \param elapsedTime Delta of time
       \returns A measure of items per second

       \note If we ever need this in something else that scxulong then 
       make this a template.
    */
    inline scxulong 
    ProcessInstance::ComputeItemsPerSecond(scxulong delta_item, 
                                           const timeval& elapsedTime) const
    {
        // Convert elapsed time to milliseconds
        scxulong el = 1000 * elapsedTime.tv_sec + elapsedTime.tv_usec / 1000;
        if (el == 0) return 0;  // Avoid divide by zero

        return 1000 * delta_item / el;
    }
    

    /**
       Computes the percentage of measure of time in relation to another measure of time.
       
       \param consumedTime Consumed time
       \param elapsedTime Elapsed time
       \returns A percentage number on how consumed time relates to elapsed time
    */
    inline unsigned int 
    ProcessInstance::ComputePercentageOfTime(const scx_timestruc_t& consumedTime, 
                                             const struct timeval& elapsedTime) const
    {
        if (elapsedTime.tv_sec == 0 && elapsedTime.tv_usec == 0) { return 0; }
        // We convert both values to floating point to get around rounding problems
        double el = elapsedTime.tv_sec + elapsedTime.tv_usec / 1000000.0L;
        if (!(el > 0)) { return 0; } // Avoid floating exceptions
        double co = consumedTime.tv_sec + consumedTime.tv_nsec / 1000000000.0L;
        return static_cast<unsigned int>(100.0L * co / el);     
    }

#endif /* sun || aix */
    
#if defined(hpux)
    /**
       Computes a parameter for which we have a delta of some kind of item
       and a time delta.

       \param delta_item Delta of usage
       \param elapsedTime Delta of time
       \returns A measure of items per second

       \note If we ever need this in something else that scxulong then 
       make this a template.
    */
    inline scxulong 
    ProcessInstance::ComputeItemsPerSecond(scxulong delta_item, 
                                           const struct timeval& elapsedTime) const
    {
        // Convert elapsed time to milliseconds
        scxulong el = 1000 * elapsedTime.tv_sec + elapsedTime.tv_usec / 1000;
        if (el == 0) return 0;  // Avoid divide by zero

        return 1000 * delta_item / el;
    }

    /**
       Computes the percentage of measure of time in relation to another measure of time.
       
       \param consumedTime Consumed time
       \param elapsedTime Elapsed time
       \returns A percentage number on how consumed time relates to elapsed time
    */
    inline unsigned int 
    ProcessInstance::ComputePercentageOfTime(scxulong consumedTime, 
                                             const struct timeval& elapsedTime) const
    {
        // Convert both to milliseconds to get a resonable resolution WO overflow
        unsigned long el = 1000 * elapsedTime.tv_sec + elapsedTime.tv_usec / 1000;
        if (el == 0) { return 0; } // Avoid divide by zero

        unsigned long co = 1000 * consumedTime;
        return static_cast<unsigned int>(100 * co / el);
    }
#endif /* hpux */
}

#endif /* PROCESSINSTANCE_H */
/*----------------------------E-N-D---O-F---F-I-L-E---------------------------*/
