/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
    \file        

    \brief       Defines the statistical disk information instance PAL common
                 for physical and logical disks.
    
    \date        2008-04-28 15:20:00
    
*/
/*----------------------------------------------------------------------------*/
#ifndef STATISTICALDISKINSTANCE_H
#define STATISTICALDISKINSTANCE_H

#include <scxsystemlib/entityinstance.h>
#include <scxcorelib/scxlog.h>
#include <scxsystemlib/datasampler.h>
#include <scxsystemlib/diskdepend.h>
#include <scxcorelib/scxhandle.h>

namespace SCXSystemLib
{
    /** Number of samples collected in the datasampler for Disk. */
    const int MAX_DISKINSTANCE_DATASAMPER_SAMPLES = 6;  // Sampling once every minute.

    /** Time between each sample in seconds. */
    const int DISK_SECONDS_PER_SAMPLE = 60;

    /** Datasampler for disk information. */
    typedef DataSampler<scxulong, MAX_DISKINSTANCE_DATASAMPER_SAMPLES> DiskInstanceDataSampler;

    /*----------------------------------------------------------------------------*/
    /**
        Represents a single statistical disk instance. This class holds common parts 
        and is intended for sub-classing.
    */
    class StatisticalDiskInstance : public EntityInstance
    {
    public:
        StatisticalDiskInstance(SCXCoreLib::SCXHandle<DiskDepend> deps, bool isTotal = false);

        void Reset();

        bool GetDiskDeviceID(std::wstring& value) const;
        bool GetDiskName(std::wstring& value) const;
        virtual bool GetReadsPerSecond(scxulong& value) const;
        virtual bool GetWritesPerSecond(scxulong& value) const;
        virtual bool GetTransfersPerSecond(scxulong& value) const;
        virtual bool GetBytesPerSecond(scxulong& read, scxulong& write) const;
        virtual bool GetBytesPerSecondTotal(scxulong& total) const;
        virtual bool GetIOPercentage(scxulong& read, scxulong& write) const;
        virtual bool GetIOPercentageTotal(scxulong& total) const;
        virtual bool GetIOTimes(double& read, double& write) const;
        virtual bool GetIOTimesTotal(double& total) const;
        virtual bool GetDiskQueueLength(double& value) const;
        virtual bool GetDiskSize(scxulong& mbUsed, scxulong& mbFree) const;
        virtual bool GetBlockSize(scxulong& blockSize) const;
        
        virtual bool GetHealthState(bool& healthy) const;
        
        virtual const std::wstring DumpString() const;
        virtual void Update();

        /*----------------------------------------------------------------------------*/
        /**
           Sample data for instance.
        */
        virtual void Sample() = 0;

        /*----------------------------------------------------------------------------*/
        /**
           Retrieve the last recorded sample values. Typically used in test methods and not by provider.
    
           \param       numR - output parameter where number of reads since boot is stored.
           \param       numW - output parameter where number of writes since boot is stored.
           \param       bytesR - output parameter where number of read bytes since boot is stored.
           \param       bytesW - output parameter where number of written bytes since boot is stored.
           \param       msR - output parameter where number of ms reading since boot is stored.
           \param       msW - output parameter where number of ms writing since boot is stored.
           \returns     true if value was set, otherwise false.
           
           \note On solaris/hpux msR and msW is not read/write but rather run/wait.
           \note On hpux physical disks numR/bytesR are number of/bytes transfered and numW/bytesW is always zero.
        */
        virtual bool GetLastMetrics(scxulong& numR, scxulong& numW, scxulong& bytesR, scxulong& bytesW, scxulong& msR, scxulong& msW) const = 0;

        scxlong FindDiskInfoByID(scxlong id);
        scxlong FindLVInfoByID(scxlong id);
    protected:    
        SCXCoreLib::SCXHandle<DiskDepend> m_deps;//!< StaticDiskDepend object
        SCXCoreLib::SCXLogHandle m_log;         //!< Log handle
        bool m_online;             //!< Tells if disk is still connected.
        std::wstring m_device;     //!< Device name
        std::wstring m_mountPoint; //!< Mount point
        std::wstring m_fsType;     //!< FS type
        std::vector<std::wstring> m_samplerDevices; //!< Devices to sample data from.

        scxulong m_sectorSize;     //!< Sector size

        scxulong m_readsPerSec;    //!< Disk reads per second
        scxulong m_writesPerSec;   //!< Disk writes per second
        scxulong m_transfersPerSec;//!< Disk transfers per second 
        scxulong m_rBytesPerSec;   //!< Bytes read per second
        scxulong m_wBytesPerSec;   //!< Bytes written per second
        scxulong m_tBytesPerSec;   //!< Bytes transfered per second
        scxulong m_rPercentage;    //!< Read percentage of total time
        scxulong m_wPercentage;    //!< Write percentage of total time
        scxulong m_tPercentage;    //!< Read and write percentage of total time
        scxulong m_rTime;          //!< Read time
        scxulong m_wTime;          //!< Write time
        scxulong m_tTime;          //!< Transfer time
        scxulong m_runTime;        //!< Run time
        scxulong m_waitTime;       //!< Wait time
        double m_secPerRead;       //!< Seconds per read
        double m_secPerWrite;      //!< Seconds per write
        double m_secPerTransfer;   //!< Seconds per transfer
        scxulong m_mbUsed;         //!< MB used
        scxulong m_mbFree;         //!< MB free
        scxulong m_blockSize;      //!< Disk block size
        double m_qLength;          //!< Average disk queue length

        DiskInstanceDataSampler m_reads;     //!< Data sampler for reads
        DiskInstanceDataSampler m_writes;    //!< Data sampler for writes
        DiskInstanceDataSampler m_transfers; //!< Data sampler for transfers
        DiskInstanceDataSampler m_tBytes;    //!< Data sampler for bytes transfered
        DiskInstanceDataSampler m_rBytes;    //!< Data sampler for bytes read
        DiskInstanceDataSampler m_wBytes;    //!< Data sampler for bytes written
        DiskInstanceDataSampler m_waitTimes; //!< Data sampler for wait times
        DiskInstanceDataSampler m_tTimes;    //!< Data sampler for total times        
        DiskInstanceDataSampler m_rTimes;    //!< Data sampler for read times
        DiskInstanceDataSampler m_wTimes;    //!< Data sampler for write times
        DiskInstanceDataSampler m_runTimes;  //!< Data sampler for run times
        DiskInstanceDataSampler m_timeStamp; //!< Data sampler for time stamps
        DiskInstanceDataSampler m_qLengths;  //!< Data sampler for queue lengths
    };

}

#endif /* STATISTICALLOGICALDISKINSTANCE_H */
/*----------------------------E-N-D---O-F---F-I-L-E---------------------------*/
