/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
    \file        

    \brief       Defines the statistical disk information enumeration PAL for logical disks.
    
    \date        2008-04-28 15:20:00
    
*/
/*----------------------------------------------------------------------------*/
#ifndef STATISTICALLOGICALDISKENUMERATION_H
#define STATISTICALLOGICALDISKENUMERATION_H

#include <scxsystemlib/entityenumeration.h>
#include <scxsystemlib/statisticallogicaldiskinstance.h>
#include <scxcorelib/scxlog.h>
#include <scxcorelib/scxthread.h>
#include <scxcorelib/scxhandle.h>
#include <scxsystemlib/diskdepend.h>
#include <map>

namespace SCXSystemLib
{
    /*----------------------------------------------------------------------------*/
    /** 
         Represents a set of discovered logical disks and their statistical data 
         on a system.
    
         Will start a sample thread to sample disk statistics when initiated.
    */
    class StatisticalLogicalDiskEnumeration : public EntityEnumeration<StatisticalLogicalDiskInstance>
    {
    public:
        StatisticalLogicalDiskEnumeration(SCXCoreLib::SCXHandle<DiskDepend> deps);
        virtual ~StatisticalLogicalDiskEnumeration();

        SCXCoreLib::SCXHandle<StatisticalLogicalDiskInstance> FindDiskByDevice(const std::wstring& device, bool includeSamplerDevice = false);

        virtual void Init();
        virtual void CleanUp();
        virtual void Update(bool updateInstances);
        virtual void UpdateInstances();
        void InitInstances();
        void SampleDisks();

        // provide class-specific implementation to add locking
        bool RemoveInstanceById(const EntityInstanceId& id);
        
        virtual const std::wstring DumpString() const;

        static void DiskSampler(SCXCoreLib::SCXThreadParamHandle& param);
    private:
        SCXCoreLib::SCXLogHandle m_log;         //!< Log handle
        SCXCoreLib::SCXHandle<DiskDepend> m_deps; //!< Dependencies object
        SCXCoreLib::SCXHandle<SCXCoreLib::SCXThread> m_sampler;       //!< Data sampler.
        SCXCoreLib::SCXThreadLockHandle m_lock; //!< Handles locking in the disk enumeration.
        std::map<std::wstring,scxulong> m_pathToRdev; //!< Cache for path to rdev values.

        void FindLogicalDisks();
        
        void UpdatePathToRdev(const std::wstring& dir);
    };

    /*----------------------------------------------------------------------------*/
    /**
        Parameters for the disk sampler thread keeping all DiskInstances up to date.
    */
    class StatisticalLogicalDiskSamplerParam : public SCXCoreLib::SCXThreadParam
    {
    public:
        /*----------------------------------------------------------------------------*/
        /**
           Constructor
        */
        StatisticalLogicalDiskSamplerParam()
            : m_diskEnum(NULL)
        {}

        StatisticalLogicalDiskEnumeration* m_diskEnum;  //!< Pointer to the disk enumeration associated with the thread.
    };
}

#endif /* STATISTICALLOGICALDISKENUMERATION_H */
/*----------------------------E-N-D---O-F---F-I-L-E---------------------------*/
