/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
    \file
 
    \brief     LogFile provider header file
 
    \date      2008-0-08 09:35:36
 
*/
/*----------------------------------------------------------------------------*/
#ifndef LOGFILEPROVIDER_H
#define LOGFILEPROVIDER_H

#include <sys/stat.h>

#include <string>
#include <vector>
#include <istream>

#include <scxproviderlib/cmpibase.h>
#include <scxcorelib/scxlog.h>
#include <scxcorelib/scxpersistence.h>
#include <scxcorelib/scxpatternfinder.h>

namespace SCXCore
{
    
    /*----------------------------------------------------------------------------*/
    /**
       LogFile provider
   
       Concrete instance of the CMPI BaseProvider delivering CIM
       information about content in log files on current host. 

       A provider-specific thread lock will be held at each call to 
       the Do* methods, so this implementation class does not need 
       to worry about that. 

    */
    class LogFileProvider : public SCXProviderLib::BaseProvider
    {
    public:
        /**
            Persistable representation of a log file with a current position.
            The persistence key is the logfile path together with the qid.
        */
        class LogFilePositionRecord
        {
        public:
            LogFilePositionRecord(const SCXCoreLib::SCXFilePath& logfile,
                                  const std::wstring& qid,
                                  SCXCoreLib::SCXHandle<SCXCoreLib::SCXPersistMedia> persistMedia = SCXCoreLib::GetPersistMedia());
            const SCXCoreLib::SCXFilePath& GetLogFile() const;
            std::streamoff GetPos() const;
            void SetPos(std::streamoff pos);
            scxulong GetStatStIno() const;
            void SetStatStIno(scxulong st_ino);
            scxulong GetStatStSize() const;
            void SetStatStSize(scxulong st_size);
            
            void Persist();
            bool Recover();
            bool UnPersist();
            
        private:
            SCXCoreLib::SCXHandle<SCXCoreLib::SCXPersistMedia> m_PersistMedia; //!< Handle to persistence framework.
            const SCXCoreLib::SCXFilePath m_LogFile; //!< Log file path.
            std::wstring m_IdString;   //!< Persistence id string created from file name and qid.
            
            std::streamoff m_Pos;     //!< file end pos
            scxulong m_StIno;            //!< st_ino field of a stat struct.
            scxulong m_StSize;         //!< st_size field of a stat struct.
        };

        /**
            Class with responsibility of maintaining opening a log file at the correct
            position depending on information in a LogFilePositionRecord.
        */
        class LogFileStreamPositioner
        {
        public:
            LogFileStreamPositioner(const SCXCoreLib::SCXFilePath& logfile,
                                    const std::wstring& qid,
                                    SCXCoreLib::SCXHandle<SCXCoreLib::SCXPersistMedia> persistMedia = SCXCoreLib::GetPersistMedia());
            SCXCoreLib::SCXHandle<std::wfstream> GetStream();
            void PersistState();
            
        private:
            SCXCoreLib::SCXHandle<LogFilePositionRecord> m_Record; //!< Handle to record with persistable data.
            SCXCoreLib::SCXHandle<std::wfstream> m_Stream; //!< Handle to currently open stream.
            SCXCoreLib::SCXLogHandle m_log; //!< Handle to log framework.
            
            bool IsFileNew() const;
            void UpdateStatData();
        };

        LogFileProvider();
        ~LogFileProvider();

        virtual const std::wstring DumpString() const;

    protected:
        //! The set of CIM classes this provider supports
        enum SupportedCimClasses {
            eSCX_LogFile,          //!< LogFile
            eSCX_LogFileRecord     //!< LogFileRecord
        };

        /** CIM methods supported */
        enum SupportedCimMethods {
            eGetMatchedRowsMethod    //!< GetMatchedRowsMethod
        };

        // Overrides from the base class with relevant implementations
        virtual void DoInit();
        virtual void DoCleanup();
        virtual void DoEnumInstanceNames(const SCXProviderLib::SCXCallContext& callContext, 
                                         SCXProviderLib::SCXInstanceCollection &instances);
        virtual void DoEnumInstances(const SCXProviderLib::SCXCallContext& callContext, 
                                     SCXProviderLib::SCXInstanceCollection &instances);
        virtual void DoExecQuery(const SCXProviderLib::SCXCallContext& callContext, SCXProviderLib::SCXInstanceCollection& instances,
                                 std::wstring query, std::wstring language);
        virtual void DoInvokeMethod(const SCXProviderLib::SCXCallContext& callContext,
                                    const std::wstring& methodname, const SCXProviderLib::SCXArgs& args,
                                    SCXProviderLib::SCXArgs& outargs, SCXProviderLib::SCXProperty& result);

        void SetPersistMedia(SCXCoreLib::SCXHandle<SCXCoreLib::SCXPersistMedia> persistMedia);

    private:

        /**
            Internal representation of a log file.
        */
        class SCXLogFile
        {
        public:
            std::wstring name;       //!< file name
            std::streamoff pos;      //!< file end pos
            struct stat64 statinfo;  //!< file information
            bool statvalid;          //!< statinfo valid flag
        };

        std::wstring GetFileName(const std::wstring& query);
        SCXLogFile* GetLogFile(const std::wstring& filename);
        bool CheckFileWrap(const struct stat64& oldstatinfo, const struct stat64& newstatinfo);

        std::vector<SCXLogFile> m_files;   //!< log files

        SCXCoreLib::SCXHandle<SCXCoreLib::SCXPersistMedia> m_persistMedia; //!< Persist media to use
        SCXCoreLib::SCXPatternFinder m_cqlPatterns; //!< Supported cql patterns finder.
        static const SCXCoreLib::SCXPatternFinder::SCXPatternCookie s_patternID; //!< Supported pattern identifier.
        static const std::wstring s_pattern; //!< The actual pattern supported
        static const std::wstring s_patternParameter; //!< name of parameter in pattern.
    };
}

#endif /* LOGFILEPROVIDER_H */
/*----------------------------E-N-D---O-F---F-I-L-E---------------------------*/
