/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
    \file

    \brief     Main implementation file for Meta Provider

    \date      2008-02-01 09:35:36


*/
/*----------------------------------------------------------------------------*/

#include <errno.h>

#include <scxcorelib/scxcmn.h>

#include <fstream>
#include <sstream>

#include <scxcorelib/scxmachineinfo.h>
#include <scxcorelib/scxlog.h>
#include <scxcorelib/stringaid.h>
#include <scxcorelib/scxstream.h>
#include "metaprovider.h"

#include "startuplog.h"

#include "buildversion.h"

#if defined(sun)
#include <sys/systeminfo.h>
#include <scxcorelib/scxlocale.h>
#endif

using namespace SCXProviderLib;
using namespace SCXCoreLib;
using namespace std;

/* This is an anonymous namespace that holds utility classes that are local. */

namespace {

#if defined(linux)
    /**
       Extracts the distribution specific OS name.

       \param[in]  platformString String to extract info from
       \returns    The name of the operating system

       This is a Linux-specific function that picks the name without version
       and os info from a distribution specific file.
    */

    wstring ExtractOSName(const wstring& platformString)
    {
        wstring caption(platformString);

        // We assume caption somewhere in the end has a release number containing
        // a digit. The OSName is then probably all "words" before then "release
        // number word".
        wstring::size_type n = caption.find_first_of(L"0123456789");
        if (n != wstring::npos)
          {
            n = caption.substr(0,n).find_last_of(L" \t\n\t");
            caption = caption.substr(0,n);
          }
        // On redhat the word "release" may also be part of the caption.
        // Remove that part also on all platforms to be sure.
        n = caption.find(L"release");
        if (n != wstring::npos)
          {
            n = caption.substr(0,n).find_last_of(L" \t\n\t");
            caption = caption.substr(0,n);
          }

        return StrTrim(caption);
    }

    /*----------------------------------------------------------------------------*/
#endif /* linux */

}

namespace SCXCore {

    /** Installation information file name */
    static const string installInfoFileName = SCX_CONFDIR "/installinfo.txt";

    /*----------------------------------------------------------------------------*/
    /**
       Provide CMPI interface for this class

       The class implementation (concrete class) is MetaProvider and the name of the
       provider in CIM registration terms is SCX_MetaProvider.

    */
    SCXProviderDef(MetaProvider, SCX_MetaProvider)

    /*----------------------------------------------------------------------------*/
    /**
       Default constructor

       The Singleton thread lock will be held during this call.
       Reading from install time insformation file and parsing release date since this
       is information that should not change.

    */
    MetaProvider::MetaProvider() :
            BaseProvider(L"scx.core.providers.metaprovider"), m_buildTimeOK(false),
            m_readInstallInfoFile(false), m_osVersion(L""), m_osName(L""), m_osAlias(L"")
    {
                LogStartup();
        SCX_LOGTRACE(m_log, L"MetaProvider constructor");

        ReadInstallInfoFile();
        GetUname();
        GetReleaseDate();
        GetOSNameAndVersion();
    }

    /*----------------------------------------------------------------------------*/
    /**
       Destructor
    */
    MetaProvider::~MetaProvider()
    {
        // Do not log here since when this destructor is called the objects neccesary for logging might no longer be alive
    }

    /*----------------------------------------------------------------------------*/
    /**
       Registration of supported capabilities

       Callback from the BaseProvider in which the provider registers the supported
       classes and methods. The registrations should match the contents of the
       MOF files exactly.

    */
    void MetaProvider::DoInit()
    {
        SCX_LOGTRACE(m_log, L"MetaProvider::DoInit");
        m_ProviderCapabilities.RegisterCimClass(eSCX_Agent, L"SCX_Agent");
    }


    /*----------------------------------------------------------------------------*/
    /**
        Provide a way for pal layer to do cleanup. Stop all threads etc.
    */
    void MetaProvider::DoCleanup()
    {
        SCX_LOGTRACE(m_log, L"MetaProvider::DoCleanup");

        m_ProviderCapabilities.Clear();
    }

    /*----------------------------------------------------------------------------*/
    /**
       Read installation information from file

       Called from constructor since this reads in information that should not change
    */
    void MetaProvider::ReadInstallInfoFile()
    {
        wifstream infofile(installInfoFileName.c_str());

        m_readInstallInfoFile = false;

        if (SCXStream::IsGood(infofile))
        {
            vector<wstring> lines;
            SCXStream::NLFs nlfs;

            // Read all lines from install info file
            // First line should be install date on ISO8601 format
            // Second line should be install version string
            // Example:
            // 2008-03-17T17:28:32.0Z
            // 1.0.1-70
            SCXStream::ReadAllLines(infofile, lines, nlfs);
            if (lines.size() == 2)
            {
                SCX_LOGTRACE(m_log, StrAppend(L"Read time from installinfo file: ", lines[0]));
                SCX_LOGTRACE(m_log, StrAppend(L"Read install version from installinfo file: ", lines[1]));

                m_installVersion = lines[1];
                try
                {
                    m_installTime = SCXCalendarTime::FromISO8601(lines[0]);
                    m_readInstallInfoFile = true;
                }
                catch (SCXCoreLib::SCXException &e)
                {
                    SCX_LOGERROR(m_log, StrAppend(StrAppend(StrAppend(L"Failed to convert install time string to SCXCalenderTime: ", lines[0]), L" - "), e.What()));
                }
            }
            else
            {
                SCX_LOGERROR(m_log, StrAppend(L"Wrong number of rows in install info file. Expected 2, got: ", lines.size()));
            }
        }
        else
        {
            SCX_LOGERROR(m_log, L"Failed to open installinfo file");
        }
    }

    /*----------------------------------------------------------------------------*/
    /**
       Reads the uname structure.

       This structure has various stuff the we may or may not use on all platforms.
    */
    void MetaProvider::GetUname()
    {
        m_unameIsValid = !(uname(&m_unameInfo) < 0);
        if (!m_unameIsValid) {
            SCX_LOGERROR(m_log, StrAppend(L"Could not do uname(). errno = ", errno));
        }
    }

    /*----------------------------------------------------------------------------*/
    /**
       Convert build date string to SCXCalendarTime

    */
    void MetaProvider::GetReleaseDate()
    {
        m_buildTimeOK = false;

        wstring buildDate(SCX_BUILDVERSION_DATE);

        if (buildDate.length() == 8)
        {
            wstring buildYear = buildDate.substr(0, 4);
            wstring buildMonth = buildDate.substr(4, 2);
            wstring buildDay = buildDate.substr(6, 2);

            try
            {
                m_buildTime = SCXCalendarTime(StrToUInt(buildYear),StrToUInt(buildMonth),StrToUInt(buildDay));
                m_buildTimeOK = true;
                SCX_LOGTRACE(m_log, StrAppend(L"Build time: ", buildDate));
            }
            catch (SCXCoreLib::SCXException& e)
            {
                SCX_LOGERROR(m_log, StrAppend(L"Failed to convert build time string to SCXCalenderTime: ", buildDate));
            }
        }
        else
        {
            SCX_LOGWARNING(m_log, StrAppend(L"Build time string is not correct length: ", buildDate));
        }
    }

    /*----------------------------------------------------------------------------*/
    /**
       Find out operating system name and version

    */
    void MetaProvider::GetOSNameAndVersion()
    {
        m_osVersion = L"";
        m_osName = L"Unknown";

#if defined(hpux) || defined(sun)

        if (m_unameIsValid)
        {
            m_osName = StrFromMultibyte(m_unameInfo.sysname);
            m_osVersion = StrFromMultibyte(m_unameInfo.release);
        }

#if defined(hpux)
        m_osAlias = L"HPUX";
#elif defined(sun)
        m_osAlias = L"Solaris";
#endif

#elif defined(aix)

        if (m_unameIsValid)
        {
            m_osName = StrFromMultibyte(m_unameInfo.sysname);

            // To get "5.3" we must read "5" and "3" from different fields.
            string ver(m_unameInfo.version);
            ver.append(".");
            ver.append(m_unameInfo.release);
            m_osVersion = StrFromMultibyte(ver);
        }
        m_osAlias = L"AIX";

#elif defined(linux)

        vector<wstring> lines;
        SCXStream::NLFs nlfs;

#if defined(PF_DISTRO_SUSE)

        static const string relFileName = "/etc/SuSE-release";
        wifstream relfile(relFileName.c_str());
        wstring version(L"");
        wstring patchlevel(L"");

        SCXStream::ReadAllLines(relfile, lines, nlfs);

        if (!lines.empty()) {
            m_osName = ExtractOSName(lines[0]);
        }

        // File contains one or more lines looking like this:
        // SUSE Linux Enterprise Server 10 (i586)
        // VERSION = 10
        // PATCHLEVEL = 1
        for (size_t i = 0; i<lines.size(); i++)
        {
            if (StrIsPrefix(StrTrim(lines[i]), L"VERSION", true))
            {
                wstring::size_type n = lines[i].find_first_of(L"=");
                if (n != wstring::npos)
                {
                    version = StrTrim(lines[i].substr(n+1));
                }
            }
            else if (StrIsPrefix(StrTrim(lines[i]), L"PATCHLEVEL", true))
            {
                wstring::size_type n = lines[i].find_first_of(L"=");
                if (n != wstring::npos)
                {
                    patchlevel = StrTrim(lines[i].substr(n+1));
                }
            }
        }

        if (version.length() > 0)
        {
            m_osVersion = version;

            if (patchlevel.length() > 0)
            {
                m_osVersion = version.append(L".").append(patchlevel);
            }
        }
        m_osAlias = L"SLES";

#elif defined(PF_DISTRO_REDHAT)

        static const string relFileName = "/etc/redhat-release";
        wifstream relfile(relFileName.c_str());

        SCXStream::ReadAllLines(relfile, lines, nlfs);

        if (!lines.empty()) {
            m_osName = ExtractOSName(lines[0]);
        }

        // File should contain one line that looks like this:
        // Red Hat Enterprise Linux Server release 5.1 (Tikanga)
        if (lines.size() > 0)
        {
            wstring::size_type n = lines[0].find_first_of(L"0123456789");
            if (n != wstring::npos)
            {
                wstring::size_type n2 = lines[0].substr(n).find_first_of(L" \t\n\t");
                m_osVersion = StrTrim(lines[0].substr(n,n2));
            }
        }
        m_osAlias = L"RHEL";

#else
#error "Linux Platform not supported"
#endif

#else
#error "Platform not supported"
#endif

    }

    /*----------------------------------------------------------------------------*/
    /**
       Set all keys in the SCXInstance

       \param[out]  inst  Instance to add keys to

    */
    void MetaProvider::AddKeys(SCXInstance &inst) // private
    {
        SCXProperty name_prop(L"Name", L"scx");
        inst.AddKey(name_prop);
    }

    /*----------------------------------------------------------------------------*/
    /**
       Set all properties in the SCXInstance

       \param[in]  inst  Instance to populate

    */
    void MetaProvider::AddProperties(SCXInstance &inst) // private
    {
        wstringstream ss;

        ss << SCX_BUILDVERSION_MAJOR << L"." << SCX_BUILDVERSION_MINOR << L"." << SCX_BUILDVERSION_PATCH << L"-" << SCX_BUILDVERSION_BUILDNR;

        SCXProperty version_prop(L"VersionString", ss.str());
        inst.AddProperty(version_prop);

        SCXProperty major_prop(L"MajorVersion", static_cast<unsigned short>(SCX_BUILDVERSION_MAJOR));
        inst.AddProperty(major_prop);

        SCXProperty minor_prop(L"MinorVersion", static_cast<unsigned short>(SCX_BUILDVERSION_MINOR));
        inst.AddProperty(minor_prop);

        SCXProperty patch_prop(L"RevisionNumber", static_cast<unsigned short>(SCX_BUILDVERSION_PATCH));
        inst.AddProperty(patch_prop);

        SCXProperty build_prop(L"BuildNumber", static_cast<unsigned short>(SCX_BUILDVERSION_BUILDNR));
        inst.AddProperty(build_prop);

        SCXProperty status_prop(L"Description", StrAppend(StrAppend(SCX_BUILDVERSION_STATUS, " - "), SCX_BUILDVERSION_DATE));
        inst.AddProperty(status_prop);

        if (m_readInstallInfoFile)
        {
            SCXProperty installversion_prop(L"KitVersionString", m_installVersion);
            inst.AddProperty(installversion_prop);

            // provide standard property as "date-time"
            SCXProperty installdate_prop(L"InstallDate", m_installTime);
            inst.AddProperty(installdate_prop);
        }

        if (m_buildTimeOK)
        {
            SCXProperty builddate_prop(L"BuildDate", m_buildTime.ToExtendedISO8601());
            inst.AddProperty(builddate_prop);
        }

        try {
            MachineInfo mi;
            SCXProperty hostname_prop(L"Hostname", mi.GetHostDomainname());
            inst.AddProperty(hostname_prop);
        } catch (SCXException& e) {
            SCX_LOGWARNING(m_log, StrAppend(
                               StrAppend(L"Can't read host/domainname because ", e.What()),
                               e.Where()));
        }

        SCXProperty os_prop(L"OSName", m_osName);
        inst.AddProperty(os_prop);

        SCXProperty osver_prop(L"OSVersion", m_osVersion);
        inst.AddProperty(osver_prop);

        SCXProperty alias_prop(L"OSAlias", m_osAlias);
        inst.AddProperty(alias_prop);

#if defined(hpux)

#if defined(hppa)
        SCXProperty arch_prop(L"Architecture", L"PA-Risc");
#else
        SCXProperty arch_prop(L"Architecture", L"IA64");
#endif
        inst.AddProperty(arch_prop);

        SCXProperty ostype_prop(L"OSType", L"HPUX");
        inst.AddProperty(ostype_prop);

#elif defined(linux)

        int bitSize = 0;
        try
        {
            MachineInfo mi;
            bitSize = mi.GetNativeBitSize();
        }
        catch (SCXCoreLib::SCXException &e)
        {
            SCX_LOGERROR(m_log, StrAppend( StrAppend(L"Failure in MachineInfo::GetNativeBitSize: ", e.What()), e.Where()));
        }

        if (32 == bitSize)
        {
            SCXProperty arch_prop(L"Architecture", L"x86");
            inst.AddProperty(arch_prop);
        }
        else if (64 == bitSize) {
            SCXProperty arch_prop(L"Architecture", L"x64");
            inst.AddProperty(arch_prop);
        }
        else {
            SCXProperty arch_prop(L"Architecture", L"Unknown");
            inst.AddProperty(arch_prop);
        }

        SCXProperty ostype_prop(L"OSType", L"Linux");
        inst.AddProperty(ostype_prop);

#elif defined(sun)

#if defined(sparc)
        SCXProperty arch_prop(L"Architecture", L"SPARC");
#else
        SCXProperty arch_prop(L"Architecture", L"x86");
#endif
        inst.AddProperty(arch_prop);

        SCXProperty ostype_prop(L"OSType", L"Solaris");
        inst.AddProperty(ostype_prop);

#elif defined(aix)
        SCXProperty arch_prop(L"Architecture", L"powerpc");     // This is what uname -p says
        SCXProperty ostype_prop(L"OSType", L"AIX");
        inst.AddProperty(arch_prop);
        inst.AddProperty(ostype_prop);
#else
#error "Platform not supported"
#endif

#if defined(linux) || defined(hpux)
        if (m_unameIsValid)
        {
            SCXProperty uname_prop(L"UnameArchitecture", StrFromMultibyte(m_unameInfo.machine));
            inst.AddProperty(uname_prop);
        }
#elif defined(sun)
        char buf[256];
        if (0 < sysinfo(SI_ARCHITECTURE, buf, sizeof(buf)))
        {
            SCXProperty uname_prop(L"UnameArchitecture", StrFromMultibyte(buf));
            inst.AddProperty(uname_prop);
        }
#elif defined(aix)
        // Can't find a generic way to get this property on AIX. This is however the
        // only one supported right now.
        SCXProperty uname_prop(L"UnameArchitecture", L"powerpc");
        inst.AddProperty(uname_prop);
#endif

        SCXProperty loglevel_prop(L"MinActiveLogSeverityThreshold",
                                  SCXCoreLib::SCXLogHandleFactory::GetLogConfigurator()->GetMinActiveSeverityThreshold());
        inst.AddProperty(loglevel_prop);

    }

    /*----------------------------------------------------------------------------*/
    /**
       Enumerate instance names

       \param[in]   callContext Context details of this request
       \param[out]  instances   Collection of instances with key properties

    */
    void MetaProvider::DoEnumInstanceNames(const SCXCallContext& /*callContext*/,
                                           SCXInstanceCollection& instances)
    {
        SCX_LOGTRACE(m_log, L"MetaProvider DoEnumInstanceNames");

        SCXInstance inst;

        AddKeys(inst);

        instances.AddInstance(inst);
    }

    /*----------------------------------------------------------------------------*/
    /**
       Enumerate instances

       \param[in]     callContext                 Details of the client request
       \param[out]    instances                   Collection of instances

    */
    void MetaProvider::DoEnumInstances(const SCXCallContext& /*callContext*/,
                                       SCXInstanceCollection& instances)
    {
        SCX_LOGTRACE(m_log, L"MetaProvider DoEnumInstances");

        SCXInstance inst;

        AddKeys(inst);
        AddProperties(inst);

        instances.AddInstance(inst);
    }

    /*----------------------------------------------------------------------------*/
    /**
       Get an instance

       \param[in]   callContext Context of the original request, indicating which instance to retrieve.
       \param[out]  instance    The returned instance

    */
    void MetaProvider::DoGetInstance(const SCXProviderLib::SCXCallContext& callContext, SCXProviderLib::SCXInstance& instance)
    {
        SCX_LOGTRACE(m_log, L"MetaProvider DoGetInstance");
        const SCXInstance& keys = callContext.GetObjectPath();
        ValidateKeyValue(L"Name", keys, L"scx");
        AddKeys(instance);
        AddProperties(instance);
    }

    /*----------------------------------------------------------------------------*/
    /**
        Dump object as string (for logging).

        \returns       The object represented as a string suitable for logging.

    */
    const std::wstring MetaProvider::DumpString() const
    {
        return L"MetaProvider";
    }



        /*----------------------------------------------------------------------------*/
        /**
           Log the startup message once regardless of how many times called
    
           This function will be called from all provider ctors except RunAs to 
           provide one initial log, regardless of how many times called.

        */
        void LogStartup(void) 
        {
                static bool bLoggedInitMessage = false;
                if (!bLoggedInitMessage) 
                {
                        SCXCoreLib::SCXLogHandle log = SCXLogHandleFactory::GetLogHandle(L"scx.core.providers");
                        SCX_LOGINFO(log, L"SCX Provider Module loaded");

#if defined(sun)

                        // Log any errors encountered during SCXLocaleContext initialization.
                        for (size_t i = 0; i < SCXLocaleContext::GetErrors().size(); i++)
                        {
                            SCX_LOGWARNING(log, SCXLocaleContext::GetErrors()[i]);
                        }

#endif /* defined(sun) */

                        bLoggedInitMessage = true;
                }
        }

}

/*----------------------------E-N-D---O-F---F-I-L-E---------------------------*/

