/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
    \file

    \brief     Meta provider header file

    \date      2008-02-01 09:35:36

*/
/*----------------------------------------------------------------------------*/
#ifndef METAPROVIDER_H
#define METAPROVIDER_H

#include <string>

// Definitions for uname()
#include <sys/utsname.h>

#include <scxproviderlib/cmpibase.h>
#include <scxcorelib/scxlog.h>
#include <scxcorelib/scxtime.h>

namespace SCXCore
{

    /*----------------------------------------------------------------------------*/
    /**
       Meta provider

       Concrete instance of the CMPI BaseProvider delivering CIM
       information about the scx installation on current host.

       A provider-specific thread lock will be held at each call to
       the Do* methods, so this implementation class does not need
       to worry about that.

    */
    class MetaProvider : public SCXProviderLib::BaseProvider
    {
    public:
        MetaProvider();
        ~MetaProvider();

        virtual const std::wstring DumpString() const;

    protected:
        //! The set of CIM classes this provider supports
        enum SupportedCimClasses {
            eSCX_Agent   //!< Agent
        };

        // Overrides from the base class with relevant implementations
        virtual void DoInit();
        virtual void DoCleanup();
        virtual void DoEnumInstanceNames(const SCXProviderLib::SCXCallContext& callContext,
                                         SCXProviderLib::SCXInstanceCollection &instances);
        virtual void DoEnumInstances(const SCXProviderLib::SCXCallContext& callContext,
                                     SCXProviderLib::SCXInstanceCollection &instances);
        virtual void DoGetInstance(const SCXProviderLib::SCXCallContext& callContext,
                                   SCXProviderLib::SCXInstance& instance);

    private:
        void AddKeys(SCXProviderLib::SCXInstance& inst);
        void AddProperties(SCXProviderLib::SCXInstance& inst);

        void ReadInstallInfoFile();
        void GetUname();
        void GetReleaseDate();
        void GetOSNameAndVersion();

        //! Set to true if build time could be parsed OK
        bool m_buildTimeOK;
        //! build time set at compile time
        SCXCoreLib::SCXCalendarTime m_buildTime;
        //! Set to true if all info could be read from the install info file
        bool m_readInstallInfoFile;
        //! Install time read from install info file
        SCXCoreLib::SCXCalendarTime m_installTime;
        //! Install version read from install info file
        std::wstring m_installVersion;
        //! OS version
        std::wstring m_osVersion;
        //! OS name
        std::wstring m_osName;
        //! OS Alias
        std::wstring m_osAlias;
        //! Saves the output from uname()
        struct utsname m_unameInfo;
        //! Is m_unameInfo valid now?
        bool m_unameIsValid;

    };
}

#endif /* METAPROVIDER_H */
/*----------------------------E-N-D---O-F---F-I-L-E---------------------------*/
