/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
   \file

   \brief       Implementation of OS Provider

   \date        08-03-07 10:55:00

*/
/*----------------------------------------------------------------------------*/

#include <scxcorelib/scxcmn.h>
#include <scxcorelib/scxexception.h>
#include <scxcorelib/scxlog.h>
#include <scxcorelib/scxmachineinfo.h>
#include <scxcorelib/scxmath.h>

#include <scxproviderlib/scxprovidercapabilities.h>

#include "osprovider.h"

#include "../meta_provider/startuplog.h"

#include <scxsystemlib/osenumeration.h>
#include <scxsystemlib/osinstance.h>
#include <scxsystemlib/processenumeration.h>

using namespace SCXProviderLib;
using namespace SCXSystemLib;
using namespace SCXCoreLib;

namespace SCXCore {

    /*----------------------------------------------------------------------------*/
    /**
       Provide CMPI interface for this class

       The class implementation (concrete class) is OSProvider and the name of the
       provider in CIM registration terms is SCX_OSProvider.

    */
    SCXProviderDef(OSProvider, SCX_OSProvider)

    /*----------------------------------------------------------------------------*/
    /**
       Default constructor

       The Singleton thread lock will be held during this call.

    */
    OSProvider::OSProvider() :
        BaseProvider(L"scx.core.providers.osprovider"), m_osEnum(NULL), m_memEnum(NULL)
    {
                LogStartup();

        SCX_LOGTRACE(m_log, L"OSProvider constructor");
    }

    /*----------------------------------------------------------------------------*/
    /**
       Destructor
    */
    OSProvider::~OSProvider()
    {
        // Do not log here since when this destructor is called the objects neccesary for logging might no longer be alive
    }

    /*----------------------------------------------------------------------------*/
    /**
       Registration of supported capabilities

       Callback from the BaseProvider in which the provider registers the supported
       classes and methods. The registrations should match the contents of the
       MOF files exactly.

    */
    void OSProvider::DoInit()
    {
        SCX_LOGTRACE(m_log, L"OSProvider::DoInit");

        if (m_osEnum != NULL)
        {
            SCXASSERTFAIL(L"DoInit() called multiple times without a call to DoCleanup() between");
            DoCleanup();
        }

        SCXASSERT(NULL == m_memEnum);

        m_ProviderCapabilities.RegisterCimClass(eSCX_OperatingSystem,
                                                s_cOSCreationClassName);
        m_osEnum = new OSEnumeration();
        m_osEnum->Init();

        // We need the memory provider for some stuff as well
        m_memEnum = new MemoryEnumeration();
        m_memEnum->Init();
    }

    /*----------------------------------------------------------------------------*/
    /**
       Provide a way for pal layer to do cleanup. Stop all threads etc.
    */
    void OSProvider::DoCleanup()
    {
        SCX_LOGTRACE(m_log, L"OSProvider::DoCleanup");

        m_ProviderCapabilities.Clear();

        if (m_osEnum != NULL)
        {
            m_osEnum->CleanUp();
            m_osEnum = NULL;
        }

        if (m_memEnum != NULL)
        {
            m_memEnum->CleanUp();
            m_memEnum = NULL;
        }
    }

    /*----------------------------------------------------------------------------*/
    /**
       Set the keys of the SCXInstance object from the information in the entity instance

       \param[in]       osinst  Internal instance representation to get data from
       \param[out]      inst   Collection to add instance to

       \throws          SCXInvalidArgumentException  The instance can not be converted to a os instance

       This method contains knowledge on which are the key fields for the class.
       The key properties are defined in the MOF file.

    */
    void OSProvider::AddKeys(SCXCoreLib::SCXHandle<OSInstance> osinst, SCXInstance &inst) const
    {
        SCX_LOGTRACE(m_log, L"OSProvider AddKeys()");

        if (osinst == NULL)
        {
            throw SCXInvalidArgumentException(L"einst", L"Not a OSInstance", SCXSRCLOCATION);
        }

        // Set all the properties which are key properties to the SCXInstance
        wstring Awstring;

        if (osinst->GetName(Awstring)){
            SCXProperty prop(L"Name", Awstring);
            inst.AddKey(prop);
        }

        SCXProperty csccn_prop(L"CSCreationClassName", s_cCSCreationClassName);
        inst.AddKey(csccn_prop);

        try {
            MachineInfo mi;
            SCXProperty prop(L"CSName", mi.GetHostDomainname());
            inst.AddKey(prop);
        } catch (SCXException& e) {
            SCX_LOGWARNING(m_log, StrAppend(
                               StrAppend(L"Can't read host/domainname because ", e.What()),
                               e.Where()));
        }

        SCXProperty ccn_prop(L"CreationClassName", s_cOSCreationClassName);
        inst.AddKey(ccn_prop);
    }

    /*----------------------------------------------------------------------------*/
    /**
       Set all properties of the SCXInstance from the information in the EntityInstance

       \param[in]       osinst  Internal instance representation to get data from
       \param[in]       meminst Internal memory instance representation to get data from
       \param[out]      inst    Instance to populate

       throws           SCXInvalidArgumentException The instance can not be converted to a os instance

       This method knows how to map the values of the internal instance representation
       to the CMPI class definition.

    */
    void OSProvider::AddProperties(
        SCXCoreLib::SCXHandle<OSInstance> osinst,
        SCXCoreLib::SCXHandle<MemoryInstance> meminst,
        SCXInstance &inst) const
    {
        if (osinst == NULL) {
            throw SCXInvalidArgumentException(L"osinst", L"Not an OSInstance", SCXSRCLOCATION);
        }

        SCX_LOGTRACE(m_log, L"OSProvider::AddPropeties()");

        SCXCalendarTime ASCXCalendarTime;
        scxulong Ascxulong, Ascxulong1;
        unsigned short Aunsignedshort;
        vector<string> Avector;
        vector<unsigned short> Aushortvector;
        wstring Awstring;
        signed short Ashort;
        unsigned int Auint;

        /*===================================================================================*/
        /* Properties of CIM_ManagedElement                                                  */
        /*===================================================================================*/

        if (osinst->GetCaption(Awstring)){
            SCXProperty prop(L"Caption", Awstring);
            inst.AddProperty(prop);
        }
        if (osinst->GetDescription(Awstring)){
            SCXProperty prop(L"Description", Awstring);
            inst.AddProperty(prop);
        }

        /*===================================================================================*/
        /* Properties of CIM_ManagedSystemElement                                            */
        /*===================================================================================*/

        // We don't support the following because there's no way to retrieve on any platforms:
        //      InstallDate
        //      Status
        //      OperationalStatus
        //      StatusDescriptions
        //      HealthState

        /*===================================================================================*/
        /* Properties of CIM_OperatingSystem                                                 */
        /*===================================================================================*/

        // We don't support the following because there's no way to retrieve on any platforms:
        //      EnabledState
        //      OtherEnabledState
        //      RequestedState
        //      EnabledDefault
        //      TimeOfLastStateChange
        //      OverwritePolicy
        //      Distributed

        /* CSCreationClassName is a key property and thus set in AddKeys */
        /* CSName is a key property and thus set in AddKeys */
        /* CreationClassName is a key property and thus set in AddKeys */

        if (osinst->GetOSType(Aunsignedshort)){
            SCXProperty prop(L"OSType", Aunsignedshort);
            inst.AddProperty(prop);
        }
        if (osinst->GetOtherTypeDescription(Awstring)){
            SCXProperty prop(L"OtherTypeDescription", Awstring);
            inst.AddProperty(prop);
        }
        if (osinst->GetVersion(Awstring)){
            SCXProperty prop(L"Version", Awstring);
            inst.AddProperty(prop);
        }
        if (osinst->GetLastBootUpTime(ASCXCalendarTime)){
            SCXProperty prop(L"LastBootUpTime", ASCXCalendarTime);
            inst.AddProperty(prop);
        }
        if (osinst->GetLocalDateTime(ASCXCalendarTime)){
            SCXProperty prop(L"LocalDateTime", ASCXCalendarTime);
            inst.AddProperty(prop);
        }
        if (osinst->GetCurrentTimeZone(Ashort)){
            SCXProperty prop(L"CurrentTimeZone", Ashort);
            inst.AddProperty(prop);
        }
        if (osinst->GetNumberOfLicensedUsers(Auint)){
            SCXProperty prop(L"NumberOfLicensedUsers", Auint);
            inst.AddProperty(prop);
        }
        if (osinst->GetNumberOfUsers(Auint)){
            SCXProperty prop(L"NumberOfUsers", Auint);
            inst.AddProperty(prop);
        }
        if (ProcessEnumeration::GetNumberOfProcesses(Auint)){
            SCXProperty prop(L"NumberOfProcesses", Auint);
            inst.AddProperty(prop);
        }
        if (osinst->GetMaxNumberOfProcesses(Auint)){
            SCXProperty prop(L"MaxNumberOfProcesses", Auint);
            inst.AddProperty(prop);
        }
        if (meminst->GetTotalSwap(Ascxulong)) {
            // TotalSwapSpaceSize comes in MB - we need KBytes
            SCXProperty prop(L"TotalSwapSpaceSize", Ascxulong * 1024);
            inst.AddProperty(prop);
        }
        if (meminst->GetTotalPhysicalMemory(Ascxulong) && meminst->GetTotalSwap(Ascxulong1)) {
            // Both TotalPhysicalMemory and TotalSwap come in MB - we need KBytes
            SCXProperty prop(L"TotalVirtualMemorySize", Ascxulong*1024 + Ascxulong1*1024);
            inst.AddProperty(prop);
        }
        if (meminst->GetAvailableMemory(Ascxulong)) {
            Ascxulong *= 1024; // Convert GetAvailableMemory from MB to KBytes

            if (meminst->GetAvailableSwap(Ascxulong1)) {
                // GetAvailableSwap comes in MB - we need KBytes
                SCXProperty prop(L"FreeVirtualMemory", Ascxulong + Ascxulong1*1024);
                inst.AddProperty(prop);
            }

            SCXProperty prop(L"FreePhysicalMemory", Ascxulong);
            inst.AddProperty(prop);
        }
        if (meminst->GetTotalPhysicalMemory(Ascxulong)) {
            // GetTotalPhysicalMemory comes in MB - we need KBytes
            SCXProperty prop(L"TotalVisibleMemorySize", Ascxulong * 1024);
            inst.AddProperty(prop);
        }
        if (meminst->GetTotalSwap(Ascxulong)) {
            // GetTotalSwap comes in MB - we need KBytes
            SCXProperty prop(L"SizeStoredInPagingFiles", Ascxulong * 1024);
            inst.AddProperty(prop);
        }
        if (meminst->GetAvailableSwap(Ascxulong)) {
            // GetAvailableSwap comes in MB - we need KBytes
            SCXProperty prop(L"FreeSpaceInPagingFiles", Ascxulong * 1024);
            inst.AddProperty(prop);
        }
        if (osinst->GetMaxProcessMemorySize(Ascxulong)){
            SCXProperty prop(L"MaxProcessMemorySize", Ascxulong);
            inst.AddProperty(prop);
        }
        if (osinst->GetMaxProcessesPerUser(Auint)){
            SCXProperty prop(L"MaxProcessesPerUser", Auint);
            inst.AddProperty(prop);
        }

        /*===================================================================================*/
        /* Properties of SCX_OperatingSystem (Taken from PG_OperatingSystem)                 */
        /*===================================================================================*/

        {
            MachineInfo mi;
            std::wostringstream bitText;
            bitText << mi.GetNativeBitSize() << L" bit";

            SCXProperty prop(L"OperatingSystemCapability", bitText.str());
            inst.AddProperty(prop);
        }
        if (osinst->GetSystemUpTime(Ascxulong)){
            SCXProperty prop(L"SystemUpTime", Ascxulong);
            inst.AddProperty(prop);
        }
    }

    /*----------------------------------------------------------------------------*/
    /**
       Enumerate instance names

       \param[in]   callContext Context details of this request
       \param[out]  names       Collection of instances with key properties

    */
    void OSProvider::DoEnumInstanceNames(const SCXCallContext& /* callContext */,
                                         SCXInstanceCollection &names)
    {
        SCX_LOGTRACE(m_log, L"OSProvider DoEnumInstanceNames");

        // There is only one instance.
        if (m_osEnum->GetTotalInstance() != 0)
        {
            SCXInstance inst;
            AddKeys(m_osEnum->GetTotalInstance(), inst);
            names.AddInstance(inst);
        }
    }

    /*----------------------------------------------------------------------------*/
    /**
       Enumerate instances

       \param[in]     callContext                 Details of the client request
       \param[out]    instances                   Collection of instances

       \throws        SCXInternalErrorException   If instances in list are not CPUInstance

    */
    void OSProvider::DoEnumInstances(const SCXCallContext& /* callContext */,
                                     SCXInstanceCollection &instances)
    {
        SCX_LOGTRACE(m_log, L"OSProvider DoEnumInstances");

        // Refresh the collection
        m_osEnum->Update();
        m_memEnum->Update();

        // There should be only one instance.
        if (m_osEnum->GetTotalInstance() != 0)
        {
            SCXInstance inst;
            AddKeys(m_osEnum->GetTotalInstance(), inst);
            AddProperties(m_osEnum->GetTotalInstance(), m_memEnum->GetTotalInstance(), inst);
            instances.AddInstance(inst);
        }
    }

    /*----------------------------------------------------------------------------*/
    /**
       Get an instance

       \param[in]   callContext Context of the original request, indicating which instance to retrieve.
       \param[out]  instance    The returned instance

       \throws     SCXInvalidArgumentException  If no Name property in keys
       \throws     SCXInternalErrorException    If instances in list are not CPUInstance
    */
    void OSProvider::DoGetInstance(const SCXCallContext& callContext, SCXInstance& instance)
    {
        SCX_LOGTRACE(m_log, L"OSProvider::DoGetInstance()");

        // Refresh the collection
        m_osEnum->Update();
        m_memEnum->Update();

        const SCXProperty& nameprop = GetKeyRef(L"Name", callContext.GetObjectPath());

        if (0 == nameprop.GetStrValue().size())
        {
            throw SCXInvalidArgumentException(L"callContext", L"Empty Name property found", SCXSRCLOCATION);
        }

        // There should be only one instance.
        if (m_osEnum->GetTotalInstance() != 0)
        {
            AddKeys(m_osEnum->GetTotalInstance(), instance);
            AddProperties(m_osEnum->GetTotalInstance(), m_memEnum->GetTotalInstance(), instance);
        }
    }

    /*----------------------------------------------------------------------------*/
    /**
       Dump object as string (for logging).

       \returns       The object represented as a string suitable for logging.

    */
    const std::wstring OSProvider::DumpString() const
    {
        return L"OSProvider";
    }

}

/*----------------------------E-N-D---O-F---F-I-L-E---------------------------*/

