/*
**==============================================================================
**
** Microsoft Public License (Ms-PL)
** 
** This license governs use of the accompanying software. If you use the 
** software, you accept this license. If you do not accept the license, do not 
** use the software.
**
** 1. Definitions
** The terms "reproduce," "reproduction," "derivative works," and 
** "distribution" have the same meaning here as under U.S. copyright law.
** A "contribution" is the original software, or any additions or changes to 
** the software.
** A "contributor" is any person that distributes its contribution under this 
** license.
** "Licensed patents" are a contributor's patent claims that read directly on 
** its contribution.
**
** 2. Grant of Rights
** (A) Copyright Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free copyright license to reproduce its
** contribution, prepare derivative works of its contribution, and distribute 
** its contribution or any derivative works that you create.
** (B) Patent Grant- Subject to the terms of this license, including the 
** license conditions and limitations in section 3, each contributor grants you
** a non-exclusive, worldwide, royalty-free license under its licensed patents
** to make, have made, use, sell, offer for sale, import, and/or otherwise 
** dispose of its contribution in the software or derivative works of the 
** contribution in the software.
**
** 3. Conditions and Limitations
** (A) No Trademark License- This license does not grant you rights to use any 
** contributors' name, logo, or trademarks.
** (B) If you bring a patent claim against any contributor over patents that 
** you claim are infringed by the software, your patent license from such 
** contributor to the software ends automatically.
** (C) If you distribute any portion of the software, you must retain all 
** copyright, patent, trademark, and attribution notices that are present in 
** the software.
** (D) If you distribute any portion of the software in source code form, you 
** may do so only under this license by including a complete copy of this 
** license with your distribution. If you distribute any portion of the 
** software in compiled or object code form, you may only do so under a license
** that complies with this license.
** (E) The software is licensed "as-is." You bear the risk of using it. The 
** contributors give no express warranties, guarantees or conditions. You may 
** have additional consumer rights under your local laws which this license 
** cannot change. To the extent permitted under your local laws, the 
** contributors exclude the implied warranties of merchantability, fitness for 
** a particular purpose and non-infringement.
**
**==============================================================================
*/
/**
    \file        

    \brief       Implements a configuration parser for the runas provider.
    
    \date        2008-08-27 15:51:24

*/
/*----------------------------------------------------------------------------*/
#ifndef SCXRUNASCONFIGURATOR_H
#define SCXRUNASCONFIGURATOR_H

#include <map>
#include <scxcorelib/scxfilepath.h>
#include <scxcorelib/scxhandle.h>
#include <scxcorelib/scxexception.h>

namespace SCXCore
{
    /*----------------------------------------------------------------------------*/
    /**
       Class for parsing generic configuration streams of the form:

       # Comment
       key1 = value1
       key2 = value2
    */
    class ConfigurationParser : public std::map<std::wstring, std::wstring>
    {
    public:
        /**
           Parses configuration.
         */
        virtual void Parse() = 0;
        /**
           Virtual destructor.
        */
        virtual ~ConfigurationParser() {};
    protected:
        virtual void ParseStream(std::wistream& configuration);
    };

    /*----------------------------------------------------------------------------*/
    /**
       Class for parsing files with generic configuration

    */
    class ConfigurationFileParser : public ConfigurationParser
    {
    public:
        ConfigurationFileParser(const SCXCoreLib::SCXFilePath& file);

        /**
           Parses the configuration file. If the file does not exist this is ignored.
        */
        void Parse();
    private:
        //! Path to file containing configuration.
        SCXCoreLib::SCXFilePath m_file;
    };

    /*----------------------------------------------------------------------------*/
    /**
       Class for writing generic configuration streams of the form:

       # Comment
       key1 = value1
       key2 = value2
    */
    class ConfigurationWriter : public std::map<std::wstring, std::wstring>
    {
    public:
        /**
           Writes configuration.
         */
        virtual void Write() = 0;
        /**
           Virtual destructor.
        */
        virtual ~ConfigurationWriter() {};
    protected:
        virtual void WriteToStream(std::wostream& configuration);
    };

    /*----------------------------------------------------------------------------*/
    /**
       Class for writing files with generic configuration

    */
    class ConfigurationFileWriter : public ConfigurationWriter
    {
    public:
        ConfigurationFileWriter(const SCXCoreLib::SCXFilePath& file);

        /**
           Writes the configuration file.
        */
        void Write();
    private:
        //! Path to configuration file.
        SCXCoreLib::SCXFilePath m_file;
    };

    /*----------------------------------------------------------------------------*/
    /**
       Class for parsing and writing configuration for the RunAs provider.
    */
    class RunAsConfigurator
    {
    public:
        RunAsConfigurator();
        RunAsConfigurator(SCXCoreLib::SCXHandle<ConfigurationParser> parser,
                          SCXCoreLib::SCXHandle<ConfigurationWriter> writer);
        const RunAsConfigurator& Parse();
        const RunAsConfigurator& Write();
        void Write(ConfigurationWriter& writer) const;

        bool GetAllowRoot() const;
        void SetAllowRoot(bool allowRoot);
        const SCXCoreLib::SCXFilePath& GetChRootPath() const;
        void SetChRootPath(const SCXCoreLib::SCXFilePath& path);
        void ResetChRootPath();
        const SCXCoreLib::SCXFilePath& GetCWD() const;
        void SetCWD(const SCXCoreLib::SCXFilePath& path);
        void ResetCWD();

    private:
        static const bool s_AllowRootDefault;
        static const SCXCoreLib::SCXFilePath s_ChRootPathDefault;
        static const SCXCoreLib::SCXFilePath s_CWDDefault;

        const std::wstring ResolveEnvVars(const std::wstring& input) const;

        //! Handles the actual parsing.
        SCXCoreLib::SCXHandle<ConfigurationParser> m_Parser;    
        //! Handles the actual writing of configuration.
        SCXCoreLib::SCXHandle<ConfigurationWriter> m_Writer;    
        //! Value of AllowRoot configuration.
        bool m_AllowRoot;
        //! Value of ChRootPath configuration.
        SCXCoreLib::SCXFilePath m_ChRootPath;
        //! Value of CWD configuration.
        SCXCoreLib::SCXFilePath m_CWD;
    };

    /*----------------------------------------------------------------------------*/
    /**
       Exeception for problems with RunAs configuration.
    */
    class SCXRunAsConfigurationException : public SCXCoreLib::SCXException {
    public: 
        /*----------------------------------------------------------------------------*/
        /**
           Ctor
           \param[in] reason Description of the error
           \param[in] l      Source code location object

        */
        SCXRunAsConfigurationException(std::wstring reason, const SCXCoreLib::SCXCodeLocation& l) : 
            SCXException(l), m_Reason(reason)
        { };

        std::wstring What() const {
            return L"Error parsing RunAs configuration: " + m_Reason;
        }

    protected:
        //! Description of error
        std::wstring   m_Reason;
    };        
} // SCXCore

#endif /* SCXRUNASCONFIGURATOR_H */
/*----------------------------E-N-D---O-F---F-I-L-E---------------------------*/
